@@ -1,9 +1,9 @@
 /*
 *
 * kkstrtext string related and text processing routines
-* $Id: kkstrtext.cc,v 1.10 2001/11/13 17:13:31 konst Exp $
+* $Id: kkstrtext.cc,v 1.25 2002/07/03 14:38:58 konst Exp $
 *
-* Copyright (C) 1999-2001 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 1999-2002 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -24,108 +24,6 @@
 
 #include "kkstrtext.h"
 
-int kwordcount(const char *strin, const char *delim) {
-    int count = 0, i = 0, slen = strlen(strin);
-
-    while(i<slen) {
-	for( ; i < slen && strchr(delim, strin[i]); i++);
-	if(i < slen) count++;
-	for( ; i < slen && !strchr(delim, strin[i]); i++);
-    }
-
-    return count;
-}
-
-char *kgetword(int wordn, const char *strin, const char *delim, char *sout, int maxout) {
-    int i = 0, count = 0, len = 0, slen = strlen(strin);
-
-    sout[0] = 0;
-    while(i < slen && count != wordn) {
-	while(i < slen && strchr(delim, strin[i])) i++;
-	if(i < slen) count++;
-	while(i < slen && !strchr(delim, strin[i])) {
-	    if(count == wordn) {
-		sout[++len] = 0;
-		sout[len-1] = strin[i];
-	    }
-	    i++;
-	}
-    }
-    return sout;
-}
-
-int kwordcountq(char *strin, char *delim, char *q) {
-    int slen = strlen(strin), count = 0, i = -1, quoted = 0, prevbs = 0;
-
-    while(i<slen) {
-	while(i<slen) {
-	    i++;
-	    if(strchr(q, strin[i]) && !prevbs) break;
-	    if(!strchr(delim, strin[i])) break;
-	    prevbs = (strin[i] == '\\');
-	}
-    
-	if(i < slen) count++;
-	while(i < slen && (quoted || !strchr(delim, strin[i]))) {
-	    if(strchr(q, strin[i]) && !prevbs) quoted = !quoted;
-	    prevbs = (strin[i] == '\\');
-	    i++;
-	}
-    }
-
-    return count;
-}
-
-int kwordposq(int n, char *s, char *delim, char *q) {
-    int count = 0, slen = strlen(s), i = 0, quoted = 0, retval = -1;
-
-    while(i < slen && count != n) {
-	if(i < slen && !strchr(q, s[i]) && strchr(delim, s[i])) {
-	    while(strchr(delim, s[i]) && i < slen) i++;
-	}
-
-	if(i < slen) count++;
-
-	if(count != n)
-	while(i < slen && (quoted || !strchr(delim, s[i]))) {
-	    if(strchr(q, s[i]))
-	    if(s[i-1] != '\\' || !i) quoted =! quoted;
-	    i++;
-	} else {
-	    if(strchr(delim, s[i])) i++;
-	    retval = i;
-	}
-    }
-  
-    return retval;
-}
-
-char *kgetwordq(int wordn, char *strin, char *delim, char *q, char *sout, int maxout) {
-    int slen = strlen(strin), len = 0, quoted = 0, i = kwordposq(wordn, strin, delim, q);
-    sout[0] = 0;
-
-    if(i >= 0) {
-	while(i < slen && len < maxout && (quoted || !strchr(delim, strin[i]))) {
-	    if(strchr(q, strin[i])) {
-		if(strin[i-1] != '\\' || !i) quoted =! quoted; else
-		if(len > 0) len--;
-	    }
-      
-	    sout[len++] = strin[i++];
-	}
-    }
-
-    sout[len] = 0;
-    return sout;
-}
-
-char *kstrcopy(char *strin, int frompos, int count, char *strout) {
-    strcpy(strout, strin);
-    strout += frompos-1;
-    strout[count] = 0;
-    return strout;
-}
-
 char *strcut(char *strin, int frompos, int count) {
     if(count > 0) {
 	if(count > strlen(strin)-frompos) count = strlen(strin)-frompos;
@@ -138,22 +36,6 @@ char *strcut(char *strin, int frompos, int count) {
     return strin;
 }
 
-char *kcuturls(char *strin, char *strout) {
-    int i, j;
-
-    strout[0] = 0;
-    for(i = 0; i < strlen(strin); i++) {
-	if(!strncasecmp(strin + i, "http://", 7) || !strncasecmp(strin + i, "ftp://", 6))
-	for( ; !strchr(",;*& \000", strin[i]); i++); else {
-	    j = strlen(strout);
-	    strout[j] = strin[i];
-	    strout[j + 1] = 0;
-	}
-    }
-
-    return strout;
-}
-
 char *strimlead(char *str)  { return trimlead(str, " \t");  }
 char *strimtrail(char *str) { return trimtrail(str, " \t"); }
 char *strim(char *str)      { return trim(str, " \t");      }
@@ -250,7 +132,7 @@ time_t str2time(char *sdate, char *mask, time_t *t) {
     return *t;
 }
 
-const string unmime(const string text) {
+string unmime(const string &text) {
     string r;
     char *buf = new char[text.size()+1];
     strcpy(buf, text.c_str());
@@ -259,7 +141,7 @@ const string unmime(const string text) {
     return r;
 }
 
-const string mime(const string text) {
+string mime(const string &text) {
     string r;
     char *buf = new char[text.size()*3+1];
     r = mime(buf, text.c_str());
@@ -315,7 +197,7 @@ char *strccat(char *dest, char c) {
     return dest;
 }
 
-vector<int> getquotelayout(const string haystack, const string qs, const string aescs) {
+vector<int> getquotelayout(const string &haystack, const string &qs, const string &aescs) {
     vector<int> r;
     string needle, escs;
     int pos, prevpos, curpos;
@@ -355,7 +237,7 @@ vector<int> getquotelayout(const string haystack, const string qs, const string
     return r;
 }
 
-vector<int> getsymbolpositions(const string haystack, const string needles, const string qoutes, const string esc) {
+vector<int> getsymbolpositions(const string &haystack, const string &needles, const string &qoutes, const string &esc) {
     vector<int> r, qp, nr;
     vector<int>::iterator iq, ir;
     int pos, st, ed, cpos;
@@ -393,8 +275,7 @@ vector<int> getsymbolpositions(const string haystack, const string needles, cons
 	continue; \
     }
 
-const char *strqpbrk(const char *s, int offset,
-const char *accept, const char *q, const char *esc = "") {
+const char *strqpbrk(const char *s, int offset, const char *accept, const char *q, const char *esc) {
     if(!s) return 0;
     if(!s[0]) return 0;
 
@@ -426,8 +307,7 @@ const char *accept, const char *q, const char *esc = "") {
     return ret;
 }
 
-const char *strqcasestr(const char *s, const char *str,
-const char *q, const char *esc = "") {
+const char *strqcasestr(const char *s, const char *str, const char *q, const char *esc) {
     char quote = 0;
     int i;
 
@@ -445,8 +325,7 @@ const char *q, const char *esc = "") {
     return 0;
 }
 
-const char *strqstr(const char *s, const char *str,
-const char *q, const char *esc = "") {
+const char *strqstr(const char *s, const char *str, const char *q, const char *esc) {
     char quote;
     const char *ret = 0, *p, *ss, *r;
     p = ss = s;
@@ -474,71 +353,6 @@ const char *q, const char *esc = "") {
     return ret;
 }
 
-// select %list from %word where %any
-// insert [(%list)] values (%list) into %word
-// update [table] %word %any where %any
-
-int parseformat(char *command, char *quotes, char *fmt, ...) {
-    const char *cp = command, *fnp, *fp;
-    char *p, *t;
-    const char *cnp;
-    va_list ap;
-
-    va_start(ap, fmt);
-    fnp = fp = fmt;
-
-    while(fp && cp && *fp && *cp) {
-	fnp = strqpbrk(fp+1, 0, "%[]", quotes);
-	for(; *fp && strchr(" ]", *fp); fp++);
-	for(; *cp && *cp == ' '; cp++);
-
-	switch(*fp) {
-	    case '[':
-		if(strncasecmp(fp+1, cp, fnp-fp-1)) {
-		    if(!(fp = strqpbrk(fp+1, 0, "]", quotes)));
-		} else {
-		    cp += fnp-fp-1;
-		    fp += fnp-fp;
-		}
-		break;
-	    case '%':
-		if(!strncmp(fp+1, "list", 4)) {
-		    fp += 5;
-		    fnp = strqpbrk(fp+1, 0, "%[]", quotes);
-		    p = (char *) malloc(fnp-fp+1);
-		    p[fnp-fp] = 0;
-		    strncpy(p, fp, fnp-fp);
-		    cnp = strqcasestr(cp+1, p, quotes);
-		    free(p);
-
-		    t = va_arg(ap, char *);
-		    t[cnp-cp] = 0;
-		    strncpy(t, cp, cnp-cp);
-
-		    cp += cnp-cp;
-		} else if(!strncmp(fp+1, "word", 4)) {
-		    fp += 5;
-		    if(!(cnp = strqpbrk(cp, 0, ";() ,.", "'"))) cnp = cp+strlen(cp);
-		    t = va_arg(ap, char *);
-		    t[cnp-cp] = 0;
-		    strncpy(t, cp, cnp-cp);
-
-		    cp += cnp-cp;
-		} else if(!strncmp(fp+1, "any", 3)) {
-		}
-		break;
-	    default:
-		if(!strncasecmp(fp, cp, fnp-fp)) {
-		    cp += fnp-fp;
-		    fp += fnp-fp;
-		} else;
-	}
-    }
-
-    va_end(ap);
-    return 1;
-}
-
 char *strinsert(char *buf, int pos, char *ins) {
     char *p = strdup(buf+pos);
     memcpy(buf+pos+strlen(ins), p, strlen(p)+1);
@@ -569,11 +383,11 @@ int stralone(char *buf, char *startword, int wordlen, char *delim) {
     return leftdelim && rightdelim;
 }
 
-const string justfname(const string fname) {
+string justfname(const string &fname) {
     return fname.substr(fname.rfind("/")+1);
 }
 
-const string justpathname(const string fname) {
+string justpathname(const string &fname) {
     int pos;
     
     if((pos = fname.rfind("/")) != -1) {
@@ -603,45 +417,39 @@ int intcompare(void *s1, void *s2) {
     return (int) s1 != (int) s2;
 }
 
-const string i2str(int i) {
+string i2str(int i) {
     char buf[64];
     sprintf(buf, "%d", i);
     return (string) buf;
 }
 
-const string ui2str(int i) {
+string ui2str(int i) {
     char buf[64];
     sprintf(buf, "%du", i);
     return (string) buf;
 }
 
-const string textscreen(string text) {
-    for(int i = 0; i < text.size(); i++)
-    if(!isalnum(text[i])) text.insert(i++, "\\");
-    
-    return text;
+string textscreen(const string &text) {
+    string r = text;
+
+    for(int i = 0; i < r.size(); i++) {
+	if(!isalnum(r[i])) r.insert(i++, "\\");
+    }   
+
+    return r;
 }
 
-const string leadcut(string base, string delim = "\t\n\r ") {
-    while(!base.empty()) {
-	if(strchr(delim.c_str(), base[0])) {
-	    base.replace(0, 1, "");
-	} else break;
-    }
-    return base;
+string leadcut(const string &base, const string &delim) {
+    int pos = base.find_first_not_of(delim);
+    return (pos != -1) ? base.substr(pos) : "";
 }
 
-const string trailcut(string base, string delim = "\t\n\r ") {
-    while(!base.empty()) {
-	if(strchr(delim.c_str(), base[base.size()-1])) {
-	    base.resize(base.size()-1);
-	} else break;
-    }
-    
-    return base;
+string trailcut(const string &base, const string &delim) {
+    int pos = base.find_last_not_of(delim);
+    return (pos != -1) ? base.substr(0, pos+1) : "";
 }
 
-const string getword(string &base, string delim = "\t\n\r ") {
+string getword(string &base, const string &delim) {
     string sub;
     int i;
 
@@ -659,7 +467,7 @@ const string getword(string &base, string delim = "\t\n\r ") {
     return sub;
 }
 
-const string getwordquote(string &base, string quote = "\"", string delim = "\t\n\r ") {
+const string getwordquote(string &base, string quote, string delim) {
     string sub;
     bool inquote = false;
     int i;
@@ -681,7 +489,7 @@ const string getwordquote(string &base, string quote = "\"", string delim = "\t\
     return sub;
 }
 
-const string getrword(string &base, string delim = "\t\n\r ") {
+string getrword(string &base, const string &delim) {
     string sub;
     int i;
 
@@ -699,7 +507,7 @@ const string getrword(string &base, string delim = "\t\n\r ") {
     return sub;
 }
 
-const string getrwordquote(string &base, string quote = "\"", string delim = "\t\n\r ") {
+string getrwordquote(string &base, const string &quote, const string &delim) {
     string sub;
     bool inquote = false;
     int i;
@@ -720,7 +528,7 @@ const string getrwordquote(string &base, string quote = "\"", string delim = "\t
     return sub;
 }
 
-int rtabmargin(bool fake, int curpos, const char *p = 0) {
+int rtabmargin(bool fake, int curpos, const char *p) {
     int ret = -1, n, near;
 
     if(p && (curpos != strlen(p))) {
@@ -742,7 +550,7 @@ int rtabmargin(bool fake, int curpos, const char *p = 0) {
     return ret;
 }
 
-int ltabmargin(bool fake, int curpos, const char *p = 0) {
+int ltabmargin(bool fake, int curpos, const char *p) {
     int ret = -1, near, n = 0;
     const char *cp;
     
@@ -773,36 +581,56 @@ int ltabmargin(bool fake, int curpos, const char *p = 0) {
     return ret;
 }
 
-void breakintolines(string text, vector<string> &lst, int linelen = 0) {
-    int npos, dpos, i;
+void breakintolines(string text, vector<string> &lst, int linelen) {
+    int dpos, nlen;
+    string sub;
+    vector<string>::iterator i;
+
+    breakintolines(text, lst);
+
+    if(linelen > 0) {
+	for(i = lst.begin(); i != lst.end(); i++) {
+	    if(i->size() > linelen) {
+		sub = i->substr(0, nlen = linelen);
+
+		if((dpos = sub.rfind(" ")) != -1) {
+		    if(dpos) nlen = dpos; else nlen = 1;
+		}
+
+		sub = i->substr(nlen + ((dpos != -1) ? 1 : 0));
+		i->erase(nlen);
+		lst.insert(i+1, sub);
+		i = lst.begin();
+	    }
+	}
+    }
+}
+
+void breakintolines(const string &text, vector<string> &lst) {
+    int npos, dpos, tpos;
     string sub;
 
+    tpos = 0;
     lst.clear();
-    while((npos = text.find("\r")) != -1) text.erase(npos, 1);
 
-    while(!text.empty()) {
-	if((npos = text.find("\n")) == -1)
+    while(tpos < text.size()) {
+	if((npos = text.find("\n", tpos)) != -1) {
+	    sub = text.substr(tpos, npos-tpos);
+	} else {
+	    sub = text.substr(tpos);
 	    npos = text.size();
-
-	while((dpos = text.substr(0, npos).find("\t")) != -1) {
-	    i = rtabmargin(false, dpos)-dpos;
-	    npos += i-1;
-	    text.erase(dpos, 1);
-	    text.insert(dpos, sub.assign(i, ' '));
 	}
 
-	if(linelen)
-	if(npos > linelen) {
-	    npos = linelen;
-	    if((dpos = text.substr(0, npos).find_last_of(" \t")) != -1)
-		npos = dpos;
+	tpos += npos-tpos+1;
+
+	for(dpos = 0; (dpos = sub.find("\r", dpos)) != -1; ) {
+	    sub.erase(dpos, 1);
 	}
 
-	sub = text.substr(0, npos);
-	text.erase(0, npos);
-
-	if(text.substr(0, 1).find_first_of("\n ") != -1)
-	    text.erase(0, 1);
+	for(dpos = 0; (dpos = sub.find("\t", dpos)) != -1; ) {
+	    sub.erase(dpos, 1);
+	    sub.insert(dpos, string(rtabmargin(false, dpos)-dpos, ' '));
+	}
 
 	lst.push_back(sub);
     }
@@ -835,8 +663,8 @@ const string &quote, const string &escape) {
     }
 }
 
-int find_quoted(const string &str, const string &needle, int offs = 0,
-const string &quote = "\"'", const string &escape = "\\") {
+int find_quoted(const string &str, const string &needle, int offs,
+const string &quote, const string &escape) {
     vector<quotedblock> positions;
     vector<quotedblock>::iterator qi;
     int npos = offs;
@@ -855,8 +683,8 @@ const string &quote = "\"'", const string &escape = "\\") {
     return !found ? npos : -1;
 }
 
-int find_quoted_first_of(const string &str, const string &needle, int offs = 0,
-const string &quote = "\"'", const string &escape = "\\") {
+int find_quoted_first_of(const string &str, const string &needle, int offs,
+const string &quote, const string &escape) {
     vector<quotedblock> positions;
     vector<quotedblock>::iterator qi;
     int npos = offs;
@@ -875,7 +703,7 @@ const string &quote = "\"'", const string &escape = "\\") {
     return !found ? npos : -1;
 }
 
-void splitlongtext(string text, vector<string> &lst, int size = 440, const string cont = "\n[continued]") {
+void splitlongtext(string text, vector<string> &lst, int size, const string cont) {
     string sub;
     int npos;
 
@@ -903,29 +731,30 @@ void splitlongtext(string text, vector<string> &lst, int size = 440, const strin
     }
 }
 
-const string strdateandtime(time_t stamp, string fmt = "") {
+string strdateandtime(time_t stamp, const string &fmt) {
     return strdateandtime(localtime(&stamp), fmt);
 }
 
-const string strdateandtime(struct tm *tms, string fmt = "") {
+string strdateandtime(struct tm *tms, const string &fmt) {
     char buf[512];
     time_t current_time = time(0);
     time_t when = mktime(tms);
+    string afmt = fmt;
 
-    if(fmt.empty()) {
+    if(afmt.empty()) {
 	if(current_time > when + 6L * 30L * 24L * 60L * 60L /* Old. */
 	|| current_time < when - 60L * 60L)                 /* Future. */ {
-	    fmt = "%b %e  %Y";
+	    afmt = "%b %e  %Y";
 	} else {
-	    fmt = "%b %e %H:%M";
+	    afmt = "%b %e %H:%M";
 	}
     }
 
-    strftime(buf, 512, fmt.c_str(), tms);
+    strftime(buf, 512, afmt.c_str(), tms);
     return buf;
 }
 
-bool iswholeword(const string s, int so, int eo) {
+bool iswholeword(const string &s, int so, int eo) {
     bool rm, lm;
     const string wdelims = "[](),.; <>-+{}=|&%~*/:?@";
 
@@ -950,23 +779,26 @@ int hex2int(const string ahex) {
     return r;
 }
 
-bool getconf(string &st, string &buf, ifstream &f) {
+bool getconf(string &st, string &buf, ifstream &f, bool passemptylines) {
     bool ret = false;
     static string sect;
 
     while(!f.eof() && !ret) {
 	getstring(f, buf);
 
-	if(buf.size())
-	switch(buf[0]) {
-	    case '%':
-		sect = buf.substr(1);
-		break;
-	    case '#':
-		break;
-	    default:
-		ret = buf.size();
-		break;
+	if(buf.size()) {
+	    switch(buf[0]) {
+		case '%':
+		    sect = buf.substr(1);
+		    break;
+		case '#':
+		    if(buf[1] != '!') break;
+		default:
+		    ret = buf.size();
+		    break;
+	    }
+	} else if(passemptylines) {
+	    ret = 1;
 	}
     }
 
@@ -990,3 +822,105 @@ bool getstring(istream &f, string &sbuf) {
 
     return r;
 }
+
+string ruscase(const string &s, const string &mode) {
+    static const string lower = "";
+    static const string upper = "";
+    string r, tfrom, tto;
+    int pos, tpos;
+
+    if(mode == "tolower") {
+	tfrom = upper;
+	tto = lower;
+    } else if(mode == "toupper") {
+	tfrom = lower;
+	tto = upper;
+    } else {
+	return s;
+    }
+
+    for(r = s; (pos = r.find_first_of(tfrom)) != -1; ) {
+	char c = r[pos];
+	tpos = tfrom.find(c);
+	r[pos] = tto[tpos];
+    }
+
+    pos = 0;
+
+    while((pos = r.find_first_not_of(tfrom, pos)) != -1) {
+	if(mode == "tolower") r[pos] = tolower(r[pos]); else
+	if(mode == "toupper") r[pos] = toupper(r[pos]);
+	pos++;
+    }
+
+    return r;
+}
+
+string siconv(const string &text, const string &fromcs, const string &tocs) {
+#ifdef HAVE_ICONV_H
+    iconv_t cd = iconv_open(tocs.c_str(), fromcs.c_str());
+
+    if(((int) cd) != -1) {
+	string r;
+	size_t inleft, outleft, soutleft;
+	char *inbuf, *outbuf, *sinbuf, *soutbuf;
+
+	sinbuf = inbuf = strdup(text.c_str());
+	inleft = strlen(inbuf);
+
+	soutleft = outleft = inleft*4;
+	soutbuf = outbuf = new char[outleft];
+
+	size_t res = iconv(cd,
+#ifdef ICONV_CONST
+	    (const char **)
+#endif
+	    &inbuf, &inleft, &outbuf, &outleft);
+
+	iconv_close(cd);
+
+	if(res != -1) {
+	    soutbuf[soutleft-outleft] = 0;
+	    r = soutbuf;
+	} else {
+	    r = text;
+	}
+
+	delete soutbuf;
+	delete sinbuf;
+
+	return r;
+    }
+#endif
+
+    return text;
+}
+
+string cuthtml(const string &html, bool cutbrs) {
+    string r, tag;
+    int npos, pos, tpos;
+
+    for(pos = 0; (npos = html.find("<", pos)) != -1; pos = npos) {
+	tpos = npos;
+	r += html.substr(pos, npos-pos);
+
+	if((npos = html.find(">", ++npos)) != -1) {
+	    npos++;
+
+	    tag = html.substr(tpos+1, npos-tpos-2);
+	    if(tag.substr(0, 1) == "/") tag.erase(0, 1);
+
+	    if((tag == "br") || (tag == "BR")) {
+		r += cutbrs ? "\n" : "<br>";
+	    }
+	} else {
+	    r += html.substr(tpos);
+	    npos = html.size();
+	}
+    }
+
+    if(pos < html.size())
+	r += html.substr(pos);
+
+    return r;
+}
