@@ -1,7 +1,7 @@
 /*
 *
 * kkiproc inter-process communications related routines
-* $Id: kkiproc.cc,v 1.5 2002/02/05 10:50:00 konst Exp $
+* $Id: kkiproc.cc,v 1.9 2002/07/06 15:16:59 konst Exp $
 *
 * Copyright (C) 1999-2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -25,35 +25,12 @@
 #include "kkiproc.h"
 
 #include <sys/ioctl.h>
+#include <signal.h>
 
-bool issuchpid(int pid) {
-    string pdir = "/proc/" + i2str(pid);
-    return !access(pdir.c_str(), F_OK);
-}
-
-int checkpid(char *pidfile) {
-    FILE *f;
-    char dir[20], curdir[500];
-    int pid;
-
-    if((f = fopen(pidfile, "r"))) {
-	fscanf(f, "%s", dir);
-	pid = atoi(dir);
-	fclose(f);
-
-	getcwd(curdir, 500);
-	sprintf(dir, "/proc/%d", pid);
-	if(!chdir(dir)) return 0; else chdir(curdir);
-    }
-
-    if((f = fopen(pidfile, "w"))) {
-	fprintf(f, "%d", getpid());
-	fclose(f);
-	return 1;
-    }
-
-    return 0;
-}
+#ifdef __sun__
+#include <sys/termio.h>
+#include <sys/filio.h>
+#endif
 
 void detach(char *logfile) {
     if(logfile) freopen(logfile, "w", stdout);
@@ -73,9 +50,12 @@ time_t lastkeypress() {
 #ifdef __linux__
 
     struct utmp *u;
-    char tname[12];
+    char tname[32];
+    int tlen;
+
+    if((tlen = readlink("/proc/self/fd/0", tname, 12)) != -1) {
+	tname[tlen] = 0;
 
-    if(readlink("/proc/self/fd/0", tname, 12) != -1) {
 	if(!strncmp(tname, "/dev/tty", 8) && isdigit(tname[8])) {
 	    setutent();
 
@@ -85,12 +65,14 @@ time_t lastkeypress() {
 		    case LOGIN_PROCESS:
 			if(strlen(u->ut_line) > 3)
 			if(!strncmp(u->ut_line, "tty", 3))
-			if(isdigit(u->ut_line[3]))
-			if(*u->ut_user && issuchpid(u->ut_pid)) {
-			    sprintf(tname, "/dev/%s", u->ut_line);
-			    if(!stat(tname, &s))
-			    if(s.st_atime > t) {
-				t = s.st_atime;
+			if(isdigit(u->ut_line[3])) {
+			    sprintf(tname, "/proc/%lu", u->ut_pid);
+
+			    if(*u->ut_user && !access(tname, F_OK)) {
+				sprintf(tname, "/dev/%s", u->ut_line);
+				if(!stat(tname, &s))
+				if(s.st_atime > t)
+				    t = s.st_atime;
 			    }
 			}
 			break;
@@ -99,9 +81,15 @@ time_t lastkeypress() {
 
 	    endutent();
 	} else {
-	    if(!stat(tname, &s)) t = s.st_atime; else time(&t);
+	    if(!stat(tname, &s)) {
+		t = s.st_atime;
+	    } else {
+		time(&t);
+	    }
 	}
-    } else time(&t);
+    } else {
+	time(&t);
+    }
 
 #else
 
