@@ -43,10 +43,11 @@
 #include <libicq2000/ICBMCookieCache.h>
 #include <libicq2000/DirectClient.h>
 #include <libicq2000/DCCache.h>
+#include <libicq2000/SMTPClient.h>
 #include <libicq2000/userinfoconstants.h>
+#include <libicq2000/MessageHandler.h>
 
 using std::string;
-using SigC::Signal1;
 
 namespace ICQ2000 {
   
@@ -79,8 +80,11 @@ namespace ICQ2000 {
 		 UIN_AWAITING_UIN_REPLY
     } m_state;
 
-    Contact m_self;
+    ContactRef m_self;
     string m_password;
+    Status m_status_wanted;
+    bool m_invisible_wanted;
+    bool m_web_aware;
 
     string m_authorizerHostname;
     unsigned short m_authorizerPort;
@@ -97,16 +101,24 @@ namespace ICQ2000 {
     Translator m_translator;
 
     ContactList m_contact_list;
+    
+    ContactList m_visible_list;
+    ContactList m_invisible_list;
+
+    MessageHandler m_message_handler;
 
     unsigned char *m_cookie_data;
     unsigned short m_cookie_length;
 
     unsigned int m_ext_ip;
+    bool m_use_portrange;
+    unsigned short m_upper_port, m_lower_port;
     TCPSocket m_serverSocket;
     TCPServer m_listenServer;
 
-    DCCache m_dccache;   // this is for established connections
-    map<unsigned int, DirectClient*> m_uinmap;
+    SMTPClient m_smtp;
+
+    DCCache m_dccache;
 
     time_t m_last_server_ping;
 
@@ -123,12 +135,11 @@ namespace ICQ2000 {
     void DisconnectAuthorizer();
     void ConnectBOS();
     void DisconnectBOS();
-    void DisconnectInt();
 
     // -- Ping server --
     void PingServer();
 
-    DirectClient* ConnectDirect(Contact *c);
+    DirectClient* ConnectDirect(const ContactRef& c);
     void DisconnectDirectConns();
     void DisconnectDirectConn(int fd);
 
@@ -145,13 +156,20 @@ namespace ICQ2000 {
     void SignalLog(LogEvent::LogType type, const string& msg);
     void SignalUserOnline(BuddyOnlineSNAC *snac);
     void SignalUserOffline(BuddyOfflineSNAC *snac);
-    void SignalUserAdded(Contact *c);
-    void SignalServerBasedContact(Contact *c);
-    void SignalUserRemoved(Contact *c);
+    void SignalServerBasedContactList(const ContactList& l);
     void SignalAddSocket(int fd, SocketEvent::Mode m);
     void SignalRemoveSocket(int fd);
     // ------------------ Outgoing packets -------------------
 
+    // -------------- Callbacks from ContactList -------------
+    void contactlist_cb(ContactListEvent *ev);
+
+    // ------- Callbacks from visible, invisible lists -------
+    void visiblelist_cb(ContactListEvent *ev);
+    void invisiblelist_cb(ContactListEvent *ev);
+
+    // -------------- Callbacks from Contacts ----------------
+
     void SendAuthReq();
     void SendNewUINReq();
     void SendCookie();
@@ -193,11 +211,7 @@ namespace ICQ2000 {
 
     // -------------------------------------------------------
 
-    Contact* lookupICQ(unsigned int uin);
-    Contact* lookupMobile(const string& m);
-    Contact* lookupEmail(const string& m);
-
-    Contact* getUserInfoCacheContact(unsigned int reqid);
+    ContactRef getUserInfoCacheContact(unsigned int reqid);
 
     void ICBMCookieCache_expired_cb(MessageEvent *ev);
     void dccache_expired_cb(DirectClient *dc);
@@ -205,7 +219,6 @@ namespace ICQ2000 {
     void dc_connected_cb(DirectClient *dc);
     void dc_log_cb(LogEvent *ev);
     void dc_socket_cb(SocketEvent *ev);
-    void dc_messaged_cb(MessageEvent *ev);
     void dc_messageack_cb(MessageEvent *ev);
 
     bool SendDirect(MessageEvent *ev);
@@ -214,7 +227,6 @@ namespace ICQ2000 {
     void SendViaServerAdvanced(MessageEvent *ev);
     void SendViaServerNormal(MessageEvent *ev);
     
-    void Connect();
     void Disconnect(DisconnectedEvent::Reason r = DisconnectedEvent::REQUESTED);
 
    public:
@@ -227,7 +239,7 @@ namespace ICQ2000 {
     void setPassword(const string& password);
     string getPassword() const;
 
-    Contact* getSelfContact();
+    ContactRef getSelfContact();
 
     bool setTranslationMap(const string& szMapFileName);
     const string& getTranslationMapFileName() const;
@@ -235,12 +247,19 @@ namespace ICQ2000 {
     bool usingDefaultMap() const;
 
     // -- Signals --
+    /**
+     *  The signal to connect to for listening to ConnectingEvent's.
+     *  A ConnectingEvent is signalled when the client is trying to go online.
+     * @see connected, ConnectingEvent
+     */
+    SigC::Signal1<void,ConnectingEvent*> connecting;
+
     /**
      *  The signal to connect to for listening to ConnectedEvent's.
      *  A ConnectedEvent is signalled when the client is online proper.
      * @see disconnected, ConnectedEvent
      */
-    Signal1<void,ConnectedEvent*> connected;
+    SigC::Signal1<void,ConnectedEvent*> connected;
 
     /**
      *  The signal to connect to for listening to DisconnectedEvent's.
@@ -255,14 +274,14 @@ namespace ICQ2000 {
      *  signalling incorrect password.
      * @see connected, DisconnectedEvent
      */
-    Signal1<void,DisconnectedEvent*> disconnected;
+    SigC::Signal1<void,DisconnectedEvent*> disconnected;
 
     /**
      *  The signal to connect to for listening to incoming
      *  MessageEvent. This includes far more than just messages.
      * @see MessageEvent
      */
-    Signal1<bool,MessageEvent*,StopOnTrueMarshal> messaged;
+    SigC::Signal1<void,MessageEvent*> messaged;
 
     /**
      *  The signal to connect to for listening to the acknowledgements
@@ -272,27 +291,39 @@ namespace ICQ2000 {
      *  messages are being reattempted to be send through the server.
      * @see messaged, MessageEvent
      */
-    Signal1<void,MessageEvent*> messageack;
+    SigC::Signal1<void,MessageEvent*> messageack;
 
     /**
      *  The signal to connect to for listening to Contact list events.
      * @see ContactListEvent
      */
-    Signal1<void,ContactListEvent*> contactlist;
+    SigC::Signal1<void,ContactListEvent*> contactlist;
+
+    /**
+     *  The signal to connect to for listening for Contact Userinfo events.
+     * @see UserInfoChangeEvent
+     */
+    SigC::Signal1<void,UserInfoChangeEvent*> contact_userinfo_change_signal;
+
+    /**
+     *  The signal to connect to for listening for Contact Status change events.
+     * @see StatusChangeEvent
+     */
+    SigC::Signal1<void,StatusChangeEvent*> contact_status_change_signal;
 
     /**
      *  The signal for when registering a new UIN has succeeded or
      *  failed after a call to RegisterUIN().
      * @see NewUINEvent, RegisterUIN
      */
-    Signal1<void,NewUINEvent*> newuin;
+    SigC::Signal1<void,NewUINEvent*> newuin;
 
     /**
      *  The signal for when the server signals the rate at which the client
      *  is sending has been changed.
      * @see RateInfoChangeEvent
      */
-    Signal1<void,RateInfoChangeEvent*> rate;
+    SigC::Signal1<void,RateInfoChangeEvent*> rate;
 
     /**
      *  The signal for all logging messages that are passed back to
@@ -302,7 +333,7 @@ namespace ICQ2000 {
      *  of log messages to display and which to ignore.
      * @see LogEvent
      */
-    Signal1<void,LogEvent*> logger;
+    SigC::Signal1<void,LogEvent*> logger;
 
     /**
      *  The signal for socket events. All clients must listen to this
@@ -313,29 +344,27 @@ namespace ICQ2000 {
      *
      * @see SocketEvent
      */
-    Signal1<void,SocketEvent*> socket;
+    SigC::Signal1<void,SocketEvent*> socket;
 
     /**
-     *  Signal to listen to for self events. This includes when the
-     *  server has accepted a status change request and successful
-     *  fetching of my user info. Your change in status has been
-     *  acknowledged by the server then, and other clients will see
-     *  the new status. In a User Interface the setting of the status
-     *  should be done separately from the updating the display of
-     *  it. The updating the display of it should only be done once
-     *  you have received this signal.
-     *
-     * @see MyStatusChangeEvent, MyUserInfoChangeEvent
+     *  The signal to connect to for listening for Self Contact Userinfo events.
+     * @see UserInfoChangeEvent
      */
-    Signal1<void,SelfEvent*> self_event;
-    
+    SigC::Signal1<void,UserInfoChangeEvent*> self_contact_userinfo_change_signal;
+
+    /**
+     *  The signal to connect to for listening for Self Contact Status change events.
+     * @see StatusChangeEvent
+     */
+    SigC::Signal1<void,StatusChangeEvent*> self_contact_status_change_signal;
+
     /**
      *  Signal when someone requests your away message. The client
      *  should setMessage in the AutoMessageEvent to what your away
      *  message is. This allows dynamic away messages for different
      *  people.
      */
-    Signal1<void,AwayMessageEvent*> want_auto_resp;
+    SigC::Signal1<void,ICQMessageEvent*> want_auto_resp;
 
     /**
      *  Signal when a Search Result has been updated.  The last signal
@@ -343,30 +372,44 @@ namespace ICQ2000 {
      *  SearchResultEvent::isFinished() set to true. After this the
      *  event is finished and deleted from memory by the library.
      */
-    Signal1<void,SearchResultEvent*> search_result;
+    SigC::Signal1<void,SearchResultEvent*> search_result;
+    
+    SigC::Signal1<void,ServerBasedContactEvent*> server_based_contact_list;
+    
     // -------------
 
-    // -- Signal Dispatchers --
-    void SignalUserInfoChange(Contact *c);
-    void SignalMessageQueueChanged(Contact *c);
-    // ------------------------
-
     // -- Send calls --
     void SendEvent(MessageEvent *ev);
 
     // -- Set Status --
-    void setStatus(const Status st, bool inv = false);
+    void setStatus(const Status st);
+    void setStatus(const Status st, bool inv);
+    void setInvisible(bool inv);
+    void setWebAware(bool wa);
+    
     Status getStatus() const;
     bool getInvisible() const;
 
+    Status getStatusWanted() const;
+    bool getInvisibleWanted() const;
+
+    bool getWebAware() const;
+
     void uploadSelfDetails();
     
     // -- Contact List --
-    void addContact(Contact& c);
+    void addContact(ContactRef c);
     void removeContact(const unsigned int uin);
-    Contact* getContact(const unsigned int uin);
-    void fetchSimpleContactInfo(Contact* c);
-    void fetchDetailContactInfo(Contact* c);
+    void addVisible(ContactRef c);
+    void removeVisible(const unsigned int uin);
+    void addInvisible(ContactRef c);
+    void removeInvisible(const unsigned int uin);
+    ContactRef getContact(const unsigned int uin);
+
+    ContactList& getContactList();
+
+    void fetchSimpleContactInfo(ContactRef c);
+    void fetchDetailContactInfo(ContactRef c);
     void fetchServerBasedContactList();
     void fetchSelfSimpleContactInfo();
     void fetchSelfDetailContactInfo();
@@ -384,6 +427,8 @@ namespace ICQ2000 {
 
     SearchResultEvent* searchForContacts(unsigned int uin);
 
+    SearchResultEvent* searchForContacts(const std::string& keyword);
+
     /*
      *  Poll must be called regularly (at least every 60 seconds)
      *  but I recommended 5 seconds, so timeouts work with good
@@ -408,12 +453,26 @@ namespace ICQ2000 {
     void setBOSServerPort(const unsigned short& port);
     unsigned short getBOSServerPort() const;
 
+    void setSMTPServerHost(const string& host);
+    string getSMTPServerHost() const;
+
+    void setSMTPServerPort(unsigned short port);
+    unsigned short getSMTPServerPort() const;
+
     void setAcceptInDC(bool d);
     bool getAcceptInDC() const;
 
     void setUseOutDC(bool d);
     bool getUseOutDC() const;
 
+    void setPortRangeLowerBound(unsigned short lower);
+    void setPortRangeUpperBound(unsigned short upper);
+    unsigned short getPortRangeLowerBound() const;
+    unsigned short getPortRangeUpperBound() const;
+
+    void setUsePortRange(bool b);
+    bool getUsePortRange() const;
+
     void Poll();
     void socket_cb(int fd, SocketEvent::Mode m);
 
