@@ -23,19 +23,18 @@
 #define ICQ_H
 
 #include <string>
+#include <list>
 
 #include <libicq2000/Xml.h>
 #include <libicq2000/buffer.h>
 #include <libicq2000/exceptions.h>
 #include <libicq2000/constants.h>
+#include <libicq2000/Contact.h>
 
 using std::string;
 
 namespace ICQ2000 {
 
-  class Contact;
-  class MessageEvent;
-  
   // ------------- TCP Command Types ------------------
 
   const unsigned short V6_TCP_START     = 0x07ee;
@@ -50,6 +49,7 @@ namespace ICQ2000 {
   const unsigned char MSG_Type_AuthRej = 0x07;
   const unsigned char MSG_Type_AuthAcc = 0x08;
   const unsigned char MSG_Type_UserAdd = 0x0c;
+  const unsigned char MSG_Type_WebPager= 0x0d;
   const unsigned char MSG_Type_EmailEx = 0x0e;
   const unsigned char MSG_Type_SMS     = 0x1a;
 
@@ -66,6 +66,15 @@ namespace ICQ2000 {
   const unsigned short Priority_Urgent        = 0x0002;
   const unsigned short Priority_ToContactList = 0x0004;
 
+  const unsigned short AcceptStatus_Online     = 0x0000; // accepted
+  const unsigned short AcceptStatus_Denied     = 0x0001; // not accepted - denied
+  const unsigned short AcceptStatus_Away       = 0x0004; // accepted in away
+  const unsigned short AcceptStatus_Occupied   = 0x0009; // not accepted in occupied
+  const unsigned short AcceptStatus_DND        = 0x000a; // not accepted in DND
+  const unsigned short AcceptStatus_Occ_Accept = 0x000c; // accepted from a to contact list in occupied
+  const unsigned short AcceptStatus_NA         = 0x000e; // accepted in NA
+
+
   /* ICQSubtype classes
    * An attempt at clearing up the complete
    * mess ICQ have made of bundling everything
@@ -82,7 +91,7 @@ namespace ICQ2000 {
     ICQSubType();
     virtual ~ICQSubType() { }
 
-    static ICQSubType* ParseICQSubType(Buffer& b, bool adv);
+    static ICQSubType* ParseICQSubType(Buffer& b, bool adv, bool ack);
     void Output(Buffer& b) const;
 
     virtual void ParseBody(Buffer& b) = 0;
@@ -103,6 +112,7 @@ namespace ICQ2000 {
     bool m_advanced, m_ack;
     bool m_urgent, m_tocontactlist;
     unsigned short m_status;
+    string m_away_message;
 
    public:
     UINICQSubType();
@@ -120,7 +130,10 @@ namespace ICQ2000 {
     void OutputBody(Buffer& b) const;
 
     virtual void ParseBodyUIN(Buffer& b) = 0;
+    virtual void ParseBodyUINACK(Buffer& b);
+
     virtual void OutputBodyUIN(Buffer& b) const = 0;
+    virtual void OutputBodyUINACK(Buffer& b) const;
 
     bool isAdvanced() const;
     void setAdvanced(bool b);
@@ -130,6 +143,9 @@ namespace ICQ2000 {
     bool isUrgent() const;
     void setToContactList(bool b);
     bool isToContactList() const;
+
+    void setAwayMessage(const std::string& m);
+    string getAwayMessage() const;
   };
 
   class NormalICQSubType : public UINICQSubType {
@@ -153,6 +169,9 @@ namespace ICQ2000 {
 
     void ParseBodyUIN(Buffer& b);
     void OutputBodyUIN(Buffer& b) const;
+    void ParseBodyUINACK(Buffer& b);
+    void OutputBodyUINACK(Buffer& b) const;
+
     unsigned short Length() const;
     unsigned char getType() const;
   };
@@ -189,9 +208,6 @@ namespace ICQ2000 {
     void ParseBodyUIN(Buffer& b);
     void OutputBodyUIN(Buffer& b) const;
 
-    string getMessage() const;
-    void setMessage(const string& msg);
-
     unsigned short Length() const;
     unsigned char getType() const;
     unsigned char getFlags() const;
@@ -243,13 +259,16 @@ namespace ICQ2000 {
 
   class AuthReqICQSubType : public UINICQSubType {
    private:
-    string m_message;
+    std::string m_alias, m_firstname, m_lastname, m_email, m_message;
+    bool m_auth;
 
    public:
     AuthReqICQSubType();
-    AuthReqICQSubType(const string& msg);
+    AuthReqICQSubType(const string& alias, const string& firstname,
+		      const string& lastname, const string& email, bool auth,
+		      const string& msg);
 
-    string getMessage() const;
+    std::string getMessage() const;
 
     void ParseBodyUIN(Buffer& b);
     void OutputBodyUIN(Buffer& b) const;
@@ -304,10 +323,36 @@ namespace ICQ2000 {
     string getSender() const;
   };
 
+  /*
+   *  (why the hell ICQ invented another subtype for this when it's almost identical to
+   *   an email express message is anyones guess..)
+   */
+  class WebPagerICQSubType : public ICQSubType {
+   private:
+    string m_message, m_email, m_sender;
+
+   public:
+    WebPagerICQSubType();
+
+    void ParseBody(Buffer& b);
+    void OutputBody(Buffer& b) const;
+    unsigned short Length() const;
+    unsigned char getType() const;
+
+    string getMessage() const;
+    string getEmail() const;
+    string getSender() const;
+  };
+
   class UserAddICQSubType : public UINICQSubType {
+   private:
+    std::string m_alias, m_firstname, m_lastname, m_email;
+    bool m_auth;
+
    public:
     UserAddICQSubType();
-    UserAddICQSubType(unsigned int source, unsigned int destination);
+    UserAddICQSubType(const std::string& alias, const std::string& firstname,
+		      const std::string& lastname, const std::string& email, bool auth);
 
     void ParseBodyUIN(Buffer& b);
     void OutputBodyUIN(Buffer& b) const;
@@ -315,10 +360,10 @@ namespace ICQ2000 {
     unsigned char getType() const;
   };
 
-  // helper
+  // helper functions
 
-  UINICQSubType* EventToUINICQSubType(MessageEvent *ev);
-  MessageEvent* UINICQSubTypeToEvent(UINICQSubType *st, Contact *contact);
+  void string_split(const std::string& in, const std::string& sep,
+		    int count, std::list<std::string>& fields);
 }
 
 #endif
