@@ -40,6 +40,10 @@ namespace ICQ2000 {
 
   class Contact;
 
+  // ============================================================================
+  //  Event Base class
+  // ============================================================================
+
   /**
    *  The base class for all events.  Basic functionality of all
    *  events is timestamping, so you can tell when an event occurred.
@@ -59,7 +63,9 @@ namespace ICQ2000 {
     void setTime(time_t t);
   };
 
-  // ----------------- SocketEvent ---------------------
+  // ============================================================================
+  //  SocketEvents
+  // ============================================================================
 
   /**
    *  Base class for socket events.
@@ -112,7 +118,21 @@ namespace ICQ2000 {
     RemoveSocketHandleEvent(int fd);
   };
 
-  // ----------------- ConnectedEvent ------------------
+  // ============================================================================
+  //  ConnectingEvent
+  // ============================================================================
+
+  /**
+   *  This event is signalled when the client is connecting to the ICQ network
+   */
+  class ConnectingEvent : public Event {
+   public:
+    ConnectingEvent();
+  };
+
+  // ============================================================================
+  //  ConnectedEvent
+  // ============================================================================
 
   /**
    *  This event is signalled when the client is connected properly to
@@ -123,7 +143,9 @@ namespace ICQ2000 {
     ConnectedEvent();
   };
 
-  // ---------------- DisconnectedEvent ----------------
+  // ============================================================================
+  //  DisconnectedEvent
+  // ============================================================================
 
   /**
    *  This event is signalled when the client has been disconnected
@@ -155,7 +177,9 @@ namespace ICQ2000 {
     Reason getReason() const;
   };
 
-  // ---------------- LogEvent -------------------------
+  // ============================================================================
+  //  LogEvents
+  // ============================================================================
 
   /**
    *  This event is for any logging messages generated by the library.
@@ -184,7 +208,9 @@ namespace ICQ2000 {
     string getMessage() const;
   };
 
-  // ---------------- ContactList Events ----------------
+  // ============================================================================
+  //  ContactListEvents (user added, user removed)
+  // ============================================================================
 
   /**
    *  Base class for Contact List related events.
@@ -195,25 +221,21 @@ namespace ICQ2000 {
      *  An enum of the different contact list event types.
      */
     enum EventType {
-      StatusChange,
-      UserInfoChange,
       UserAdded,
-      UserRemoved,
-      MessageQueueChanged,
-      ServerBasedContact
+      UserRemoved
     };
     
    protected:
     /**
      *  The contact this event refers to.
      */
-    Contact *m_contact;
+    ContactRef m_contact;
 
    public:
-    ContactListEvent(Contact* c);
+    ContactListEvent(ContactRef c);
     virtual ~ContactListEvent();
     
-    Contact* getContact() const;
+    ContactRef getContact() const;
     unsigned int getUIN() const;
 
     /**
@@ -224,30 +246,12 @@ namespace ICQ2000 {
     virtual EventType getType() const = 0;
   };
 
-  /**
-   *  The event signalled when user information changes.
-   */
-  class UserInfoChangeEvent : public ContactListEvent {
-   public:
-    UserInfoChangeEvent(Contact* c);
-    EventType getType() const;
-  };
-
-  /**
-   *  The event signalled when a contact entry from the server-based contact list is received.
-   */
-  class ServerBasedContactEvent : public ContactListEvent {
-   public:
-    ServerBasedContactEvent(Contact *c);
-    EventType getType() const;
-  };
-
   /**
    *  The event signalled when a user is added.
    */
   class UserAddedEvent : public ContactListEvent {
    public:
-    UserAddedEvent(Contact *c);
+    UserAddedEvent(ContactRef c);
     EventType getType() const;
   };
 
@@ -256,52 +260,85 @@ namespace ICQ2000 {
    */
   class UserRemovedEvent : public ContactListEvent {
    public:
-    UserRemovedEvent(Contact *c);
+    UserRemovedEvent(ContactRef c);
     EventType getType() const;
   };
 
+  // ============================================================================
+  //  ContactEvents (queue changes, status change, user info change)
+  // ============================================================================
+
   /**
-   *  The event signalled when a user's message queue changes.
+   *  Base class for Contact events.
    */
-  class MessageQueueChangedEvent : public ContactListEvent {
+  class ContactEvent : public Event {
    public:
-    MessageQueueChangedEvent(Contact *c);
-    EventType getType() const;
+    /**
+     *  An enum of the different contact list event types.
+     */
+    enum EventType {
+      StatusChange,
+      UserInfoChange,
+    };
+    
+   protected:
+    /**
+     *  The contact this event refers to.
+     */
+    ContactRef m_contact;
+
+   public:
+    ContactEvent(ContactRef c);
+    virtual ~ContactEvent();
+    
+    ContactRef getContact() const;
+    unsigned int getUIN() const;
+
+    /**
+     *  get the type of ContactEvent
+     *
+     * @return type of the ContactEvent
+     */
+    virtual EventType getType() const = 0;
   };
-  
+
+  /**
+   *  The event signalled when user information changes.
+   */
+  class UserInfoChangeEvent : public ContactEvent {
+   private:
+    bool m_is_transient_detail;
+   public:
+    UserInfoChangeEvent(ContactRef c, bool is_transient_detail);
+    EventType getType() const;
+    bool isTransientDetail() const;
+  };
+
   /**
    *  The event signalled when a user's status changes.
    */
-  class StatusChangeEvent : public ContactListEvent {
+  class StatusChangeEvent : public ContactEvent {
    private:
     Status m_status;
     Status m_old_status;
     
    public:
-    StatusChangeEvent(Contact* contact, Status status, Status old_status);
+    StatusChangeEvent(ContactRef contact, Status status, Status old_status);
 
     EventType getType() const;
     Status getStatus() const;
     Status getOldStatus() const;
   };
 
-  // ---------------- MessageEvent(s) -------------------
+  // ============================================================================
+  //  MessageEvents
+  // ============================================================================
 
   /**
    *  A message event.  MessageEvents are used for messages, URLs,
    *  SMSs, Authorisation request/responses and away messages.
    */
   class MessageEvent : public Event {
-   protected:
-    /// the contact related to the MessageEvent
-    Contact* m_contact; 
-    /// whether the event is finished    
-    bool m_finished;
-    /// whether the event was delivered
-    bool m_delivered;
-    /// whether the event was sent direct
-    bool m_direct;
-
    public:
     /**
      *  enum of the type of the message
@@ -315,10 +352,36 @@ namespace ICQ2000 {
       AuthAck,
       AwayMessage,
       EmailEx,
-      UserAdd
+      UserAdd,
+      Email,
+      WebPager
     };
 
-    MessageEvent(Contact* c);
+    enum DeliveryFailureReason {
+      Failed,                  // general failure
+      Failed_NotConnected,     // you are not connected!
+      Failed_ClientNotCapable, // remote client is not capable (away messages)
+      Failed_Denied,           // denied outright
+      Failed_Ignored,          // ignore completely - send no ACKs back either
+      Failed_Occupied,         // resend as to contactlist/urgent
+      Failed_DND,              // resend as to contactlist/urgent
+      Failed_SMTP
+    };
+
+   protected:
+    /// the contact related to the MessageEvent
+    ContactRef m_contact; 
+    /// whether the event is finished    
+    bool m_finished;
+    /// whether the event was delivered
+    bool m_delivered;
+    /// whether the event was sent direct
+    bool m_direct;
+
+    DeliveryFailureReason m_failure_reason;
+
+   public:
+    MessageEvent(ContactRef c);
     virtual ~MessageEvent();
 
     /**
@@ -327,16 +390,18 @@ namespace ICQ2000 {
      * @return the type of the message
      */
     virtual MessageType getType() const = 0;
-    Contact* getContact();
+    ContactRef getContact();
     
-    bool isFinished() const;
+    bool isFinished()  const;
     bool isDelivered() const;
-    bool isDirect() const;
+    bool isDirect()    const;
     
     void setFinished(bool f);
     void setDelivered(bool f);
     void setDirect(bool f);
 
+    DeliveryFailureReason getDeliveryFailureReason() const;
+    void setDeliveryFailureReason(DeliveryFailureReason d);
   };
 
   /**
@@ -344,16 +409,23 @@ namespace ICQ2000 {
    */
   class ICQMessageEvent : public MessageEvent {
    private:
-    bool m_urgent, m_tocontactlist;
+    bool m_urgent, m_tocontactlist, m_offline;
+    std::string m_away_message;
     
    public:
-    ICQMessageEvent(Contact *c);
+    ICQMessageEvent(ContactRef c);
     
     bool isUrgent() const;
     void setUrgent(bool b);
     bool isToContactList() const;
     void setToContactList(bool b);
+    bool isOfflineMessage() const;
+    void setOfflineMessage(bool b);
     unsigned int getSenderUIN() const;
+    std::string getAwayMessage() const;
+    void setAwayMessage(const std::string& msg);
+
+    virtual ICQMessageEvent* copy() const = 0;
   };
 
   /**
@@ -362,22 +434,23 @@ namespace ICQ2000 {
   class NormalMessageEvent : public ICQMessageEvent {
    private:
     string m_message;
-    bool m_offline, m_multi;
+    bool m_multi;
     unsigned int m_foreground, m_background;
     
    public:
-    NormalMessageEvent(Contact* c, const string& msg, bool multi = false);
-    NormalMessageEvent(Contact* c, const string& msg, time_t t, bool multi);
-    NormalMessageEvent(Contact* c, const string& msg, unsigned int fg, unsigned int bg);
+    NormalMessageEvent(ContactRef c, const string& msg, bool multi = false);
+    NormalMessageEvent(ContactRef c, const string& msg, time_t t, bool multi);
+    NormalMessageEvent(ContactRef c, const string& msg, unsigned int fg, unsigned int bg);
 
     string getMessage() const;
     MessageType getType() const;
-    bool isOfflineMessage() const;
     bool isMultiParty() const;
     unsigned int getForeground() const;
     unsigned int getBackground() const;
     void setForeground(unsigned int f);
     void setBackground(unsigned int b);
+
+    ICQMessageEvent* copy() const;
   };
   
   /**
@@ -386,16 +459,16 @@ namespace ICQ2000 {
   class URLMessageEvent : public ICQMessageEvent {
    private:
     string m_message, m_url;
-    bool m_offline;
     
    public:
-    URLMessageEvent(Contact* c, const string& msg, const string& url);
-    URLMessageEvent(Contact* c, const string& msg, const string& url, time_t t);
+    URLMessageEvent(ContactRef c, const string& msg, const string& url);
+    URLMessageEvent(ContactRef c, const string& msg, const string& url, time_t t);
 
     string getMessage() const;
     string getURL() const;
     MessageType getType() const;
-    bool isOfflineMessage() const;
+
+    ICQMessageEvent* copy() const;
   };
   
   /**
@@ -404,11 +477,12 @@ namespace ICQ2000 {
   class SMSMessageEvent : public MessageEvent {
    private:
     string m_message, m_source, m_sender, m_senders_network;
+    string m_smtp_from, m_smtp_to, m_smtp_subject;
     bool m_rcpt;
 
    public:
-    SMSMessageEvent(Contact* c, const string& msg, bool rcpt);
-    SMSMessageEvent(Contact* c, const string& msg, const string& source,
+    SMSMessageEvent(ContactRef c, const string& msg, bool rcpt);
+    SMSMessageEvent(ContactRef c, const string& msg, const string& source,
 		    const string& senders_network, const string& time);
 
     string getMessage() const;
@@ -417,6 +491,15 @@ namespace ICQ2000 {
     string getSender() const;
     string getSenders_network() const;
     bool getRcpt() const;
+
+    void setSMTPFrom(const string& from);
+    string getSMTPFrom() const;
+
+    void setSMTPTo(const string& to);
+    string getSMTPTo() const;
+
+    void setSMTPSubject(const string& subj);
+    string getSMTPSubject() const;
   };
 
   /**
@@ -428,7 +511,7 @@ namespace ICQ2000 {
     bool m_delivered;
     
    public:
-    SMSReceiptEvent(Contact* c, const string& msg, const string& message_id,
+    SMSReceiptEvent(ContactRef c, const string& msg, const string& message_id,
 		    const string& submission_time, const string& delivery_time, bool del);
     
     MessageType getType() const;
@@ -441,19 +524,19 @@ namespace ICQ2000 {
   };
 
   /**
-   *  An Away message
+   *  An Away message. The way away messages work in ICQ is they are
+   *  just sending a special blank message to the other end, and the
+   *  away message comes back in the ACK, as it would for other
+   *  messages when sent to someone who is away (N/A, etc..).
+   *  
    */
   class AwayMessageEvent : public ICQMessageEvent {
-   private:
-    Contact *m_contact;
-    string m_message;
-
    public:
-    AwayMessageEvent(Contact *c);
+    AwayMessageEvent(ContactRef c);
 
     MessageType getType() const;
-    string getMessage() const;
-    void setMessage(const string& msg);
+
+    ICQMessageEvent* copy() const;
   };
 
   /**
@@ -462,15 +545,15 @@ namespace ICQ2000 {
   class AuthReqEvent : public ICQMessageEvent {
    private:
     string m_message;
-    bool m_offline;
 
    public:
-    AuthReqEvent(Contact* c, const string& msg);
-    AuthReqEvent(Contact* c, const string& msg, time_t time);
+    AuthReqEvent(ContactRef c, const string& msg);
+    AuthReqEvent(ContactRef c, const string& msg, time_t time);
 
     string getMessage() const;
     MessageType getType() const;
-    bool isOfflineMessage() const;
+
+    ICQMessageEvent* copy() const;
   };
   
   /**
@@ -479,19 +562,19 @@ namespace ICQ2000 {
   class AuthAckEvent : public ICQMessageEvent {
    private:
     string m_message;
-    bool m_offline;
     bool m_granted;
 
    public:
-    AuthAckEvent(Contact* c, bool granted);
-    AuthAckEvent(Contact* c, bool granted, time_t time);
-    AuthAckEvent(Contact* c, const string& msg, bool granted);
-    AuthAckEvent(Contact* c, const string& msg, bool granted, time_t time);
+    AuthAckEvent(ContactRef c, bool granted);
+    AuthAckEvent(ContactRef c, bool granted, time_t time);
+    AuthAckEvent(ContactRef c, const string& msg, bool granted);
+    AuthAckEvent(ContactRef c, const string& msg, bool granted, time_t time);
 
     string getMessage() const;
     MessageType getType() const;
-    bool isOfflineMessage() const;
     bool isGranted() const;
+
+    ICQMessageEvent* copy() const;
   };
 
   /**
@@ -502,7 +585,7 @@ namespace ICQ2000 {
     string m_sender, m_email, m_message;
 
    public:
-    EmailExEvent(Contact* c, const string &email, const string &sender, const string &msg);
+    EmailExEvent(ContactRef c, const string &email, const string &sender, const string &msg);
 
     string getMessage() const;
     string getEmail() const;
@@ -512,77 +595,54 @@ namespace ICQ2000 {
     unsigned int getSenderUIN() const;
   };
 
+  /**
+   *  A Web Pager message
+   */
+  class WebPagerEvent : public MessageEvent {
+   private:
+    string m_sender, m_email, m_message;
+
+   public:
+    WebPagerEvent(ContactRef c, const string& email, const string& sender, const string& msg);
+
+    string getMessage() const;
+    string getEmail() const;
+    string getSender() const;
+
+    MessageType getType() const;
+  };
+
   /**
    *  A "You were added" message
    */
-  class UserAddEvent : public MessageEvent {
+  class UserAddEvent : public ICQMessageEvent {
    public:
-    UserAddEvent(Contact* c);
+    UserAddEvent(ContactRef c);
 
     MessageType getType() const;
     unsigned int getSenderUIN() const;
-  };
 
-  // --------------------- Self Events ----------------------
+    ICQMessageEvent* copy() const;
+  };
 
   /**
-   *  Base class for Self events.
+   *  An E-mail message, sent with SMTP
    */
-  class SelfEvent : public Event {
-   protected:
-    Contact *m_self_contact;
+  class EmailMessageEvent : public MessageEvent {
+   private:
+    string m_message;
 
    public:
-    /**
-     *  An enum of the different self event types.
-     */
-    enum EventType {
-      MyStatusChange,
-      MyUserInfoChange
-    };
-    
-   public:
-    SelfEvent(Contact *self);
-    virtual ~SelfEvent();
-    
-    /**
-     *  get the type of SelfEvent
-     *
-     * @return type of the SelfEvent
-     */
-    virtual EventType getType() const = 0;
+    EmailMessageEvent(ContactRef c, const string &msg);
 
-    Contact *getSelfContact() const;
+    string getMessage() const;
+
+    MessageType getType() const;
   };
 
-  // --------------------- Status Change Event ----------------------
-
-  /**
-   *  Your status change
-   */
-  class MyStatusChangeEvent : public SelfEvent {
-   public:
-    MyStatusChangeEvent(Contact *self);
-
-    Status getStatus() const;
-    bool getInvisible() const;
-
-    EventType getType() const;
-  };
-
-  // --------------------- My UserInfo Change Event ----------------------
-
-  /**
-   *  Your details fetched from the server
-   */
-  class MyUserInfoChangeEvent : public SelfEvent {
-   public:
-    MyUserInfoChangeEvent(Contact *self);
-
-    EventType getType() const;
-  };
-
-  // --------------------- Search Events ----------------------------
+  // ============================================================================
+  //  Search Events
+  // ============================================================================
 
   /**
    *  The event signalled when a user-search result is received.
@@ -592,14 +652,15 @@ namespace ICQ2000 {
     enum SearchType {
       ShortWhitepage,
       FullWhitepage,
-      UIN
+      UIN,
+      Keyword
     };
 	
    private:
     bool m_finished, m_expired;
     SearchType m_searchtype;
     ContactList m_clist;
-    Contact *m_last_contact;
+    ContactRef m_last_contact;
     unsigned int m_more_results;
     
    public:
@@ -607,8 +668,8 @@ namespace ICQ2000 {
 
     SearchType getSearchType() const;
     ContactList& getContactList();
-    Contact* getLastContactAdded() const;
-    void setLastContactAdded(Contact *c);
+    ContactRef getLastContactAdded() const;
+    void setLastContactAdded(ContactRef c);
     unsigned int getNumberMoreResults() const;
 
     bool isFinished() const;
@@ -618,7 +679,20 @@ namespace ICQ2000 {
     void setNumberMoreResults(unsigned int m);
   };
 
-  // --------------------- NewUIN Event -----------------------------
+  /**
+   *  The event signalled when entries from the server-based contact list is received.
+   */
+  class ServerBasedContactEvent : public Event {
+   private:
+    ContactList m_clist;
+   public:
+    ServerBasedContactEvent(const ContactList& l);
+    ContactList& getContactList();
+  };
+
+  // ============================================================================
+  //  NewUINEvent
+  // ============================================================================
 
   /**
    *  Registration of a new UIN
