@@ -22,6 +22,7 @@
 #include <libicq2000/DirectClient.h>
 
 #include <libicq2000/ICQ.h>
+#include <libicq2000/constants.h>
 
 #include "sstream_fix.h"
 
@@ -45,24 +46,25 @@ namespace ICQ2000 {
   /*
    * Constructor when receiving an incoming connection
    */
-  DirectClient::DirectClient(Contact& self, TCPSocket *sock, ContactList *cl,
-			     unsigned int ext_ip, unsigned short server_port,
+  DirectClient::DirectClient(ContactRef self, TCPSocket *sock, MessageHandler *mh,
+			     ContactList *cl, unsigned int ext_ip, unsigned short server_port,
 			     Translator* translator)
-    : m_state(WAITING_FOR_INIT), m_socket(sock), m_recv(translator),
-      m_self_contact(self), m_contact(NULL), m_contact_list(cl),
-      m_incoming(true), m_local_ext_ip(ext_ip),
+    : m_state(WAITING_FOR_INIT), m_recv(translator),
+      m_self_contact(self), m_contact(NULL), m_contact_list(cl), 
+      m_message_handler(mh), m_incoming(true), m_local_ext_ip(ext_ip),
       m_local_server_port(server_port), m_translator(translator)
   {
+    m_socket = sock;
     Init();
   }
 
   /*
    * Constructor for making an outgoing connection
    */
-  DirectClient::DirectClient(Contact& self, Contact *c, unsigned int ext_ip,
+  DirectClient::DirectClient(ContactRef self, ContactRef c, MessageHandler *mh, unsigned int ext_ip,
 			     unsigned short server_port, Translator *translator)
     : m_state(NOT_CONNECTED), m_recv(translator), m_self_contact(self), 
-      m_contact(c), m_incoming(false), m_local_ext_ip(ext_ip),
+      m_contact(c), m_message_handler(mh), m_incoming(false), m_local_ext_ip(ext_ip),
       m_local_server_port(server_port), m_translator(translator)
       
   {
@@ -83,16 +85,6 @@ namespace ICQ2000 {
     delete m_socket;
   }
 
-  void DirectClient::SignalAddSocket(int fd, SocketEvent::Mode m) {
-    AddSocketHandleEvent ev( fd, m );
-    socket.emit(&ev);
-  }
-
-  void DirectClient::SignalRemoveSocket(int fd) {
-    RemoveSocketHandleEvent ev(fd);
-    socket.emit(&ev);
-  }
-
   void DirectClient::Init() {
     m_seqnum = 0xFFFF;
     m_msgcache.setDefaultTimeout(30);
@@ -126,24 +118,16 @@ namespace ICQ2000 {
   }
   
   void DirectClient::expired_cb(MessageEvent *ev) {
-    if ( m_contact != NULL ) {
-      ev->setFinished(false);
-      ev->setDelivered(false);
-      ev->setDirect(true);
-      messageack.emit(ev);
-    } else {
-      // discard
-      delete ev;
-    }
+    ev->setFinished(false);
+    ev->setDelivered(false);
+    ev->setDirect(true);
+    messageack.emit(ev);
   }
 
   void DirectClient::Recv() {
     try {
       while ( m_socket->connected() ) {
 	if ( !m_socket->Recv(m_recv) ) break;
-	ostringstream ostr;
-	ostr << "Received packet from " << IPtoString( m_socket->getRemoteIP() ) << ":" << m_socket->getRemotePort() << endl << m_recv;
-	SignalLog(LogEvent::DIRECTPACKET, ostr.str());
 	Parse();
       }
     } catch(SocketException e) {
@@ -157,15 +141,6 @@ namespace ICQ2000 {
     }
   }
 
-  void DirectClient::SignalLog(LogEvent::LogType type, const string& msg) {
-    LogEvent ev(type,msg);
-    logger.emit(&ev);
-  }
-
-  void DirectClient::SignalMessageEvent(MessageEvent *ev) {
-    messaged.emit(ev);
-  }
-
   void DirectClient::Parse() {
     if (m_recv.empty()) return;
 
@@ -176,11 +151,17 @@ namespace ICQ2000 {
 
       m_recv.setLittleEndian();
       m_recv >> length;
+      if (length > Incoming_Packet_Limit) throw ParseException("Received too long incoming packet");
       if (m_recv.remains() < length) return; // waiting for more of the packet
 
+
       Buffer sb(m_translator);
       m_recv.chopOffBuffer( sb, length+2 );
 
+      ostringstream ostr;
+      ostr << "Received packet from " << IPtoString( m_socket->getRemoteIP() ) << ":" << m_socket->getRemotePort() << endl << sb;
+      SignalLog(LogEvent::DIRECTPACKET, ostr.str());
+
       if (m_state == WAITING_FOR_INIT) {
 	ParseInitPacket(sb);
 
@@ -254,18 +235,18 @@ namespace ICQ2000 {
 
   void DirectClient::ConfirmUIN() {
     if ( m_contact_list->exists(m_remote_uin) ) {
-      Contact &c = (*m_contact_list)[ m_remote_uin ];
-      if ( (c.getExtIP() == m_local_ext_ip && c.getLanIP() == getIP() )
+      ContactRef c = (*m_contact_list)[ m_remote_uin ];
+      if ( (c->getExtIP() == m_local_ext_ip && c->getLanIP() == getIP() )
 	   /* They are behind the same masquerading box,
 	    * and the Lan IP matches
 	    */
-	       || c.getExtIP() == getIP()) {
-	m_contact = &c;
+	       || c->getExtIP() == getIP()) {
+	m_contact = c;
       } else {
 	// spoofing attempt most likely
 	ostringstream ostr;
 	ostr << "Refusing direct connection from someone that claims to be UIN "
-	     << m_remote_uin << " since their IP " << IPtoString( getIP() ) << " != " << IPtoString( c.getExtIP() );
+	     << m_remote_uin << " since their IP " << IPtoString( getIP() ) << " != " << IPtoString( c->getExtIP() );
 	throw DisconnectedException( ostr.str() );
       }
       
@@ -278,17 +259,17 @@ namespace ICQ2000 {
   void DirectClient::SendInitPacket() {
     Buffer b(m_translator);
     b.setLittleEndian();
-    b << (unsigned short)( m_eff_tcp_version == 7 ? 0x0030 : 0x002c ); // length
+    Buffer::marker m1 = b.getAutoSizeShortMarker();
 
     b << (unsigned char)0xff;    // start byte
     b << (unsigned short)0x0007; // tcp version
-    b << (unsigned short)( m_eff_tcp_version == 7 ? 0x002b : 0x0027 ); // second length
+    Buffer::marker m2 = b.getAutoSizeShortMarker();
     
     b << m_remote_uin;
     b << (unsigned short)0x0000;
     b << (unsigned int)m_local_server_port;
 
-    b << m_self_contact.getUIN();
+    b << m_self_contact->getUIN();
     b.setBigEndian();
     b << m_local_ext_ip;
     b << m_socket->getLocalIP();
@@ -301,6 +282,9 @@ namespace ICQ2000 {
     b << (unsigned int)0x00000003; // unknown
     if (m_eff_tcp_version == 7) 
       b << (unsigned int)0x00000000; // unknown
+
+    b.setAutoSizeMarker(m1);
+    b.setAutoSizeMarker(m2);
     
     Send(b);
   }
@@ -329,7 +313,7 @@ namespace ICQ2000 {
     
     unsigned int our_uin;
     b >> our_uin;
-    if (our_uin != m_self_contact.getUIN()) throw ParseException("Local UIN in Init Packet not same as our Local UIN");
+    if (our_uin != m_self_contact->getUIN()) throw ParseException("Local UIN in Init Packet not same as our Local UIN");
 
     // 00 00
     // xx xx       senders open port
@@ -403,7 +387,7 @@ namespace ICQ2000 {
   void DirectClient::SendInit2() {
     Buffer b(m_translator);
     b.setLittleEndian();
-    b << (unsigned short)0x0021;
+    Buffer::marker m1 = b.getAutoSizeShortMarker();
     b << (unsigned char) 0x03       // start byte
       << (unsigned int)  0x0000000a // unknown
       << (unsigned int)  0x00000001 // unknown
@@ -419,6 +403,7 @@ namespace ICQ2000 {
 	<< (unsigned int) 0x00000000 // unknown
 	<< (unsigned int) 0x00040001; // unknown
     }
+    b.setAutoSizeMarker(m1);
     Send(b);
   }
 
@@ -453,75 +438,29 @@ namespace ICQ2000 {
 
     b.advance(12); // unknown 3 ints
 
-    ICQSubType *i = ICQSubType::ParseICQSubType(b, true);
+    ICQSubType *i = ICQSubType::ParseICQSubType(b, true, (command == V6_TCP_ACK));
     if (dynamic_cast<UINICQSubType*>(i) == NULL) throw ParseException("Unknown ICQ subtype");
     UINICQSubType *icqsubtype = dynamic_cast<UINICQSubType*>(i);
 
     icqsubtype->setSeqNum(seqnum);
+    icqsubtype->setSource( m_contact->getUIN() );
 
     if (command == 0) throw ParseException("Invalid TCP Packet");
 
-    unsigned short type = icqsubtype->getType();
-
-    MessageEvent *ev;
-
     switch(command) {
 
     case V6_TCP_START:
-
-      if (type == MSG_Type_Normal
-	  || type == MSG_Type_URL) {
-	UINICQSubType *ist = static_cast<UINICQSubType*>(icqsubtype);
-	ev = UINICQSubTypeToEvent(ist,m_contact);
-	SignalMessageEvent(ev);
-
-	SendPacketAck(icqsubtype);
-
-      } else if (type == MSG_Type_AutoReq_Away
-	       || type == MSG_Type_AutoReq_Occ
-	       || type == MSG_Type_AutoReq_NA
-	       || type == MSG_Type_AutoReq_DND
-	       || type == MSG_Type_AutoReq_FFC)
-      {
-        AwayMsgSubType *ast = static_cast<AwayMsgSubType*>(icqsubtype);
-	AwayMessageEvent aev(m_contact);
-	want_auto_resp.emit(&aev);
-        ast->setMessage(aev.getMessage());
-
-	ostringstream ostr;
-	ostr << "Sending direct auto response to "
-	   << m_contact->getAlias() << " (" << m_contact->getStringUIN() << ")";
-	SignalLog(LogEvent::INFO, ostr.str());
-	
-        SendPacketAck(icqsubtype);
-      }
-
+    {
+      bool ack = m_message_handler->handleIncoming( icqsubtype );
+      if (ack) SendPacketAck(icqsubtype);
       break;
+    }
 
     case V6_TCP_ACK:
       if ( m_msgcache.exists(seqnum) ) {
 	MessageEvent *ev = m_msgcache[seqnum];
-	ev->setFinished(true);
-	ev->setDelivered(true);
 	ev->setDirect(true);
-
-	if (ev->getType() == MessageEvent::AwayMessage) {
-	  AwayMessageEvent *aev = static_cast<AwayMessageEvent*>(ev);
-
-	  if (icqsubtype->getType() == MSG_Type_AutoReq_Away
-	      || icqsubtype->getType() == MSG_Type_AutoReq_Occ
-	      || icqsubtype->getType() == MSG_Type_AutoReq_NA
-	      || icqsubtype->getType() == MSG_Type_AutoReq_DND
-	      || icqsubtype->getType() == MSG_Type_AutoReq_FFC) {
-	    AwayMsgSubType *ast = static_cast<AwayMsgSubType*>(icqsubtype);
-	    aev->setMessage( ast->getMessage() ); // fill out the away message in the ACK
-	  } else {
-	    SignalLog(LogEvent::WARN, "Away Message ACKed by remote client as wrong type");
-	  }
-	}
-
-	messageack.emit(ev);
-
+	m_message_handler->handleIncomingACK( ev, icqsubtype );
 	m_msgcache.remove(seqnum);
 	delete ev;
       } else {
@@ -682,12 +621,13 @@ namespace ICQ2000 {
   void DirectClient::SendInitAck() {
     Buffer b(m_translator);
     b.setLittleEndian();
-    b << (unsigned short)0x0004;
+    Buffer::marker m1 = b.getAutoSizeShortMarker();
     b << (unsigned int)0x00000001;
+    b.setAutoSizeMarker(m1);
     Send(b);
   }
 
-  void DirectClient::SendPacketAck(UINICQSubType *icqsubtype) {
+  void DirectClient::SendPacketAck(ICQSubType *icqsubtype) {
     Buffer b(m_translator);
 
     b.setLittleEndian();
@@ -698,8 +638,6 @@ namespace ICQ2000 {
       << (unsigned int)0x00000000
       << (unsigned int)0x00000000
       << (unsigned int)0x00000000;
-    icqsubtype->setACK(true);
-    icqsubtype->setStatus(0); // quick fix until accept-statuses are implemented
     
     icqsubtype->Output(b);
     Buffer c(m_translator);
@@ -712,12 +650,10 @@ namespace ICQ2000 {
 
     unsigned short seqnum = NextSeqNum();
 
-    UINICQSubType *ist = EventToUINICQSubType(ev);
+    UINICQSubType *ist = m_message_handler->handleOutgoing(ev);
     if (ist == NULL) return;
 
     ist->setAdvanced(true);
-    ist->setStatus( Contact::MapStatusToICQStatus( m_self_contact.getStatus(),
-						   m_self_contact.isInvisible() ) );
 
     b.setLittleEndian();
     b << (unsigned int)0x00000000 // checksum (filled in by Encrypt)
@@ -784,15 +720,8 @@ namespace ICQ2000 {
 
   TCPSocket* DirectClient::getSocket() const { return m_socket; }
 
-  void DirectClient::setContact(Contact *c) { m_contact = c; }
+  void DirectClient::setContact(ContactRef c) { m_contact = c; }
 
-  Contact* DirectClient::getContact() const { return m_contact; }
+  ContactRef DirectClient::getContact() const { return m_contact; }
 
-  DirectClientException::DirectClientException() { }
-  DirectClientException::DirectClientException(const string& text) : m_errortext(text) { }
-
-  const char* DirectClientException::what() const throw() { return m_errortext.c_str(); }
-
-  DisconnectedException::DisconnectedException(const string& text) : DirectClientException(text) { }
-  
 }
