@@ -24,8 +24,6 @@
 #include "sstream_fix.h"
 #include <memory>
 
-#include <libicq2000/events.h>
-
 using std::ostringstream;
 using std::auto_ptr;
 
@@ -36,7 +34,7 @@ namespace ICQ2000 {
   ICQSubType::ICQSubType()
     : m_flags(0x0000) { }
 
-  ICQSubType* ICQSubType::ParseICQSubType(Buffer& b, bool adv) {
+  ICQSubType* ICQSubType::ParseICQSubType(Buffer& b, bool adv, bool ack) {
     unsigned char type, flags;
     b >> type
       >> flags;
@@ -66,6 +64,9 @@ namespace ICQ2000 {
     case MSG_Type_EmailEx:
       ist = new EmailExICQSubType();
       break;
+    case MSG_Type_WebPager:
+      ist = new WebPagerICQSubType();
+      break;
     case MSG_Type_UserAdd:
       ist = new UserAddICQSubType();
       break;
@@ -87,6 +88,11 @@ namespace ICQ2000 {
     if (dynamic_cast<UINICQSubType*>(ist) != NULL) {
       UINICQSubType *ust = dynamic_cast<UINICQSubType*>(ist);
       ust->setAdvanced(adv);
+      ust->setACK(ack);
+      /* There is nothing in the encoding of the ICQ subtype that
+	 distinguishes whether it is an ack or not - this is implied
+	 by the protocol layer above this. As such it has to be passed
+	 as an argument and set body the rest of parsing continues */
     }
     ist->setFlags(flags);
     ist->ParseBody(b);
@@ -151,6 +157,16 @@ namespace ICQ2000 {
     m_tocontactlist = b;
   }
 
+  void UINICQSubType::setAwayMessage(const string& m)
+  {
+    m_away_message = m;
+  }
+
+  string UINICQSubType::getAwayMessage() const
+  {
+    return m_away_message;
+  }
+
   void UINICQSubType::ParseBody(Buffer& b)
   {
     if (m_advanced) {
@@ -159,9 +175,17 @@ namespace ICQ2000 {
 	>> priority;
       m_urgent = priority & Priority_Urgent;
       m_tocontactlist = priority & Priority_ToContactList;
+      if (!m_urgent && priority != Priority_Normal && priority != 0)
+	m_tocontactlist = true;
+      // the official clients seem fairly lenient on what they'll accept in status flags
     }
 
-    ParseBodyUIN(b);
+    if (!m_ack) {
+      ParseBodyUIN(b);
+    } else {
+      ParseBodyUINACK(b);
+    }
+    
   }
 
   void UINICQSubType::OutputBody(Buffer& b) const
@@ -179,7 +203,24 @@ namespace ICQ2000 {
       b << priority;
     }
 
-    OutputBodyUIN(b);
+    if (!m_ack) {
+      OutputBodyUIN(b);
+    } else {
+      OutputBodyUINACK(b);
+    }
+    
+  }
+
+  void UINICQSubType::OutputBodyUINACK(Buffer& b) const
+  {
+    b.PackUint16TranslatedNull(m_away_message);
+  }
+
+  void UINICQSubType::ParseBodyUINACK(Buffer& b)
+  {
+    /* whatever the message type, ACKs are always the same
+       - the current away message */
+    b.UnpackUint16TranslatedNull(m_away_message);
   }
 
   NormalICQSubType::NormalICQSubType(bool multi)
@@ -210,22 +251,28 @@ namespace ICQ2000 {
   }
 
   void NormalICQSubType::OutputBodyUIN(Buffer& b) const {
-    if (m_ack) {
-      b.PackUint16StringNull("");
-    } else {
-      string m_text = m_message;
-      b.ClientToServer(m_text);
-      b.PackUint16StringNull(m_text);
-    }
-    
+    b.PackUint16TranslatedNull(m_message);
+
     if (m_advanced) {
-      if (m_ack) {
-	b << (unsigned int)0x00000000
-	  << (unsigned int)0xffffffff;
-      } else {
-	b << (unsigned int)m_foreground
-	  << (unsigned int)m_background;
-      }
+      b << (unsigned int)m_foreground
+	<< (unsigned int)m_background;
+    }
+  }
+
+  void NormalICQSubType::ParseBodyUINACK(Buffer& b)
+  {
+    b.UnpackUint16TranslatedNull(m_away_message);
+    b.advance(8);
+  }
+
+  void NormalICQSubType::OutputBodyUINACK(Buffer& b) const
+  {
+    /* hmmff.. Normal messages differ from all the other ACKs by
+       having these two pointless ints. */
+    b.PackUint16TranslatedNull(m_away_message);
+    if (m_advanced) {
+      b << (unsigned int)0x00000000
+	<< (unsigned int)0xffffffff;
     }
   }
 
@@ -285,11 +332,14 @@ namespace ICQ2000 {
       b.PackUint16StringNull("");
     } else {
       ostringstream ostr;
-      string m_text = m_message;
-      b.ClientToServer(m_text);
-      ostr << m_text << (unsigned char)0xfe << m_url;
-      m_text = ostr.str();
-      b.PackUint16StringNull(m_text);
+
+      string message = m_message;
+      string url = m_url;
+      b.ClientToServer(message);
+      b.ClientToServer(url);     // I think url should be translated too ??
+      ostr << message << (unsigned char)0xfe << url;
+
+      b.PackUint16StringNull( ostr.str() );
     }
   }
 
@@ -331,14 +381,15 @@ namespace ICQ2000 {
   }
 
   void AwayMsgSubType::ParseBodyUIN(Buffer& b) {
-    b.UnpackUint16StringNull(m_message);
-    b.ServerToClient(m_message);
+    // dummy
+    string dummy;
+    b.UnpackUint16StringNull(dummy);
   }
 
   void AwayMsgSubType::OutputBodyUIN(Buffer& b) const {
-    string tekst = m_message;
-    b.ClientToServer(tekst);
-    b.PackUint16StringNull(tekst);
+    // dummy
+    string dummy;
+    b.PackUint16StringNull( dummy );
   }
 
   unsigned short AwayMsgSubType::Length() const {
@@ -351,10 +402,6 @@ namespace ICQ2000 {
 
   unsigned char AwayMsgSubType::getFlags() const { return 0x03; }
 
-  string AwayMsgSubType::getMessage() const { return m_message; }
-
-  void AwayMsgSubType::setMessage(const string& msg) { m_message = msg; }
-
   SMSICQSubType::SMSICQSubType() { }
 
   string SMSICQSubType::getMessage() const { return m_message; }
@@ -500,42 +547,50 @@ namespace ICQ2000 {
   AuthReqICQSubType::AuthReqICQSubType()
     { }
 
-  AuthReqICQSubType::AuthReqICQSubType(const string& msg)
-    : m_message(msg)  { }
+  AuthReqICQSubType::AuthReqICQSubType(const string& alias, const string& firstname,
+				       const string& lastname, const string& email, bool auth,
+				       const string& msg)
+    : m_alias(alias), m_firstname(firstname), m_lastname(lastname),
+      m_email(email), m_message(msg), m_auth(auth)  { }
 
   string AuthReqICQSubType::getMessage() const { return m_message; }
   
   void AuthReqICQSubType::ParseBodyUIN(Buffer& b) {
-    int pos;
     string text;
 
-    b.advance(3);
     b.UnpackUint16StringNull(text);
 
     /*
-    *
-    * The packet does have nickname, first and last name fields,
-    * but they're better to fetch with user details lookup request
-    * for different versions of Windows client either have or don't
-    * have them in packets they send. <konst>
-    *
-    * Here we read only the last field, which contains a message.
-    *
-    */
+     *
+     * The packet does have nickname, first and last name fields,
+     * but they're better to fetch with user details lookup request
+     * for different versions of Windows client either have or don't
+     * have them in packets they send. <konst>
+     *
+     */
 
-    if((pos = text.rfind(0xfe)) != -1) {
-	m_message = text.substr(pos+1);
-    } else {
-	m_message = "";
-    }
+    list<string> fields;
+    string_split( text, string("\xfe"), 6, fields);
 
-    b.ServerToClient(m_message);
+    list<string>::iterator iter = fields.begin();
+    m_alias = b.ServerToClientCC(*(iter++));
+    m_firstname = b.ServerToClientCC(*(iter++));
+    m_lastname = b.ServerToClientCC(*(iter++));
+    m_email = b.ServerToClientCC(*(iter++));
+    m_auth = ((*(iter++)) == "1");
+    m_message = b.ServerToClientCC(*(iter++));
   }
 
   void AuthReqICQSubType::OutputBodyUIN(Buffer& b) const {
-    string m_text = m_message;
-    b.ClientToServer(m_text);
-    b.PackUint16StringNull(m_text);
+    ostringstream ostr;
+    ostr << b.ClientToServerCC(m_alias) << (unsigned char)0xfe
+	 << b.ClientToServerCC(m_firstname) << (unsigned char)0xfe
+	 << b.ClientToServerCC(m_lastname) << (unsigned char)0xfe
+	 << b.ClientToServerCC(m_email) << (unsigned char)0xfe
+	 << (m_auth ? "1" : "0") << (unsigned char)0xfe
+	 << b.ClientToServerCC(m_message);
+    
+    b.PackUint16StringNull( ostr.str() );
   }
 
   unsigned short AuthReqICQSubType::Length() const {
@@ -545,7 +600,7 @@ namespace ICQ2000 {
   unsigned char AuthReqICQSubType::getType() const { return MSG_Type_AuthReq; }
   
   AuthRejICQSubType::AuthRejICQSubType()
-    { }
+  { }
 
   AuthRejICQSubType::AuthRejICQSubType(const string& msg)
     : m_message(msg)  { }
@@ -560,9 +615,7 @@ namespace ICQ2000 {
   }
 
   void AuthRejICQSubType::OutputBodyUIN(Buffer& b) const {
-    string m_text = m_message;
-    b.ClientToServer(m_text);
-    b.PackUint16StringNull(m_text);
+    b.PackUint16TranslatedNull(m_message);
   }
 
   unsigned short AuthRejICQSubType::Length() const {
@@ -575,9 +628,12 @@ namespace ICQ2000 {
   { }
 
   void AuthAccICQSubType::ParseBodyUIN(Buffer& b) {
+    string nothing_much;
+    b.UnpackUint16StringNull(nothing_much);
   }
 
   void AuthAccICQSubType::OutputBodyUIN(Buffer& b) const {
+    b.PackUint16StringNull("");
   }
 
   unsigned short AuthAccICQSubType::Length() const {
@@ -586,36 +642,32 @@ namespace ICQ2000 {
 
   unsigned char AuthAccICQSubType::getType() const { return MSG_Type_AuthAcc; }
 
+  // ============================================================================
+  //  Email Express message
+  // ============================================================================
+
   EmailExICQSubType::EmailExICQSubType() { }
 
   void EmailExICQSubType::ParseBody(Buffer& b) {
-    int pos, i;
-    string text, fld;
+    string text;
 
     b.UnpackUint16StringNull(text);
 
-    for(i = 0; i < 6; i++) {
-	if((pos = text.find(0xfe)) != -1) {
-	  fld = text.substr(0, pos);
-	  text.erase(0, pos+1);
-	} else {
-	  fld = text;
-	  text = "";
-	}
+    list<string> fields;
+    string_split( text, string("\xfe"), 6, fields);
 
-	switch(i) {
-	  case 0: m_sender = fld; break;
-	  case 3: m_email = fld; break;
-	  case 5: m_message = fld; break;
-	}
-    }
-
-    b.ServerToClient(m_sender);
-    b.ServerToClient(m_email);
-    b.ServerToClient(m_message);
+    list<string>::iterator iter = fields.begin();
+    m_sender = b.ServerToClientCC(*(iter++));
+    ++iter; // ???
+    ++iter; // ???
+    m_email = b.ServerToClientCC(*(iter++));
+    ++iter; // ???
+    m_message = b.ServerToClientCC(*(iter++));
   }
 
-  void EmailExICQSubType::OutputBody(Buffer& b) const { }
+  void EmailExICQSubType::OutputBody(Buffer& b) const {
+    // never sent
+  }
 
   unsigned short EmailExICQSubType::Length() const {
     return 0;
@@ -629,106 +681,118 @@ namespace ICQ2000 {
 
   string EmailExICQSubType::getSender() const { return m_sender; }
 
-  UserAddICQSubType::UserAddICQSubType() { }
+  // ============================================================================
+  //  Web Pager message
+  // ============================================================================
 
-  UserAddICQSubType::UserAddICQSubType(unsigned int source, unsigned int destination)
-    : UINICQSubType(source, destination) { }
+  WebPagerICQSubType::WebPagerICQSubType() { }
 
-  void UserAddICQSubType::ParseBodyUIN(Buffer& b) {
-  /*
-  *
-  * The packet does have nickname, first and last name fields,
-  * but they're better to fetch with user details lookup request
-  * for different versions of Windows client either have or don't
-  * have them in packets they send. <konst>
-  *
-  */
+  void WebPagerICQSubType::ParseBody(Buffer& b) {
+    string text;
+
+    b.UnpackUint16StringNull(text);
+
+    list<string> fields;
+    string_split( text, string("\xfe"), 6, fields);
+
+    list<string>::iterator iter = fields.begin();
+    m_sender = b.ServerToClientCC(*(iter++));
+    ++iter; // ???
+    ++iter; // ???
+    m_email = b.ServerToClientCC(*(iter++));
+    ++iter; // ???
+    m_message = b.ServerToClientCC(*(iter++));
   }
 
-  void UserAddICQSubType::OutputBodyUIN(Buffer& b) const { }
+  void WebPagerICQSubType::OutputBody(Buffer& b) const {
+    // never sent
+  }
+
+  unsigned short WebPagerICQSubType::Length() const {
+    return 0;
+  }
+
+  unsigned char WebPagerICQSubType::getType() const { return MSG_Type_WebPager; }
+
+  string WebPagerICQSubType::getMessage() const { return m_message; }
+
+  string WebPagerICQSubType::getEmail() const { return m_email; }
+
+  string WebPagerICQSubType::getSender() const { return m_sender; }
+
+  // ============================================================================
+  //  "You were added" message
+  // ============================================================================
+
+  UserAddICQSubType::UserAddICQSubType() { }
+
+  UserAddICQSubType::UserAddICQSubType(const string& alias, const string& firstname,
+				       const string& lastname, const string& email, bool auth)
+    : m_alias(alias), m_firstname(firstname), m_lastname(lastname),
+      m_email(email), m_auth(auth) { }
+
+  void UserAddICQSubType::ParseBodyUIN(Buffer& b)
+  {
+    string text;
+
+    b.UnpackUint16StringNull(text);
+
+    /*
+     *
+     * The packet does have nickname, first and last name fields,
+     * but they're better to fetch with user details lookup request
+     * for different versions of Windows client either have or don't
+     * have them in packets they send. <konst>
+     *
+     */
+
+    list<string> fields;
+    string_split( text, string("\xfe"), 5, fields);
+
+    list<string>::iterator iter = fields.begin();
+    m_alias = b.ServerToClientCC(*(iter++));
+    m_firstname = b.ServerToClientCC(*(iter++));
+    m_lastname = b.ServerToClientCC(*(iter++));
+    m_email = b.ServerToClientCC(*(iter++));
+    m_auth = ((*(iter++)) == "1");
+  }
+
+  void UserAddICQSubType::OutputBodyUIN(Buffer& b) const {
+    ostringstream ostr;
+    ostr << b.ClientToServerCC(m_alias) << (unsigned char)0xfe
+	 << b.ClientToServerCC(m_firstname) << (unsigned char)0xfe
+	 << b.ClientToServerCC(m_lastname) << (unsigned char)0xfe
+	 << b.ClientToServerCC(m_email) << (unsigned char)0xfe
+	 << (m_auth ? "1" : "0") << (unsigned char)0xfe;
+    
+    b.PackUint16StringNull( ostr.str() );
+  }
 
   unsigned short UserAddICQSubType::Length() const { return 0; }
 
   unsigned char UserAddICQSubType::getType() const { return MSG_Type_UserAdd; }
 
-  // helper
-
-  MessageEvent* UINICQSubTypeToEvent(UINICQSubType *st, Contact *contact)
+  void string_split(const string& in, const string& sep, int count, std::list<string>& fields)
   {
-    ICQMessageEvent *e = NULL;
-    unsigned short type = st->getType();
-    
-    if (type == MSG_Type_Normal) {
-      NormalICQSubType *nst = static_cast<NormalICQSubType*>(st);
-      e = new NormalMessageEvent(contact,
-				 nst->getMessage(), nst->isMultiParty() );
-
-    } else if (type == MSG_Type_URL) {
-      URLICQSubType *ust = static_cast<URLICQSubType*>(st);
-      e = new URLMessageEvent(contact,
-			      ust->getMessage(),
-			      ust->getURL());
-
-    } else if (type == MSG_Type_AuthReq) {
-      AuthReqICQSubType *ust = static_cast<AuthReqICQSubType*>(st);
-      e = new AuthReqEvent(contact, ust->getMessage());
-
-    } else if (type == MSG_Type_AuthRej) {
-      AuthRejICQSubType *ust = static_cast<AuthRejICQSubType*>(st);
-      e = new AuthAckEvent(contact, ust->getMessage(), false);
-
-    } else if (type == MSG_Type_AuthAcc) {
-      e = new AuthAckEvent(contact, true);
+    // oh for perl.. :-)
+    string::size_type lpos = 0, pos = 0;
+    while (lpos != in.size()) {
+      pos = in.find(sep, lpos);
+      fields.push_back(in.substr(lpos, pos-lpos));
+      if (pos == string::npos) {
+	lpos = in.size();
+      } else {
+	lpos = pos + sep.size();
+      }
     }
 
-    if (e != NULL) {
-      e->setUrgent( st->isUrgent() );
-      e->setToContactList( st->isToContactList() );
+    // add blank fields if necessary
+    count -= fields.size();
+    while(count > 0) {
+      fields.push_back(string());
+      --count;
     }
     
-    return e;
   }
   
-  UINICQSubType* EventToUINICQSubType(MessageEvent *ev)
-  {
-    Contact *c = ev->getContact();
-    UINICQSubType *ist = NULL;
-
-    if (ev->getType() == MessageEvent::Normal) {
-
-      NormalMessageEvent *nv = static_cast<NormalMessageEvent*>(ev);
-      ist = new NormalICQSubType(nv->getMessage());
-
-    } else if (ev->getType() == MessageEvent::URL) {
-
-      URLMessageEvent *uv = static_cast<URLMessageEvent*>(ev);
-      ist = new URLICQSubType(uv->getMessage(), uv->getURL());
-
-    } else if (ev->getType() == MessageEvent::AwayMessage) {
-
-      ist = new AwayMsgSubType( c->getStatus() );
-
-    } else if (ev->getType() == MessageEvent::AuthReq) {
-
-      AuthReqEvent *uv = static_cast<AuthReqEvent*>(ev);
-      ist = new AuthReqICQSubType(uv->getMessage());
-
-    } else if (ev->getType() == MessageEvent::AuthAck) {
-
-      AuthAckEvent *uv = static_cast<AuthAckEvent*>(ev);
-      if(uv->isGranted())
-        ist = new AuthAccICQSubType();
-      else
-        ist = new AuthRejICQSubType(uv->getMessage());
-    }
-    
-    ICQMessageEvent *iev;
-    if (ist != NULL && (iev = dynamic_cast<ICQMessageEvent*>(ev)) != NULL) {
-      ist->setUrgent( iev->isUrgent() );
-      ist->setToContactList( iev->isToContactList() );
-    }
-    
-    return ist;
-  }
-
 }
