@@ -27,6 +27,8 @@
 #include <libicq2000/TLV.h>
 #include <libicq2000/Xml.h>
 
+#include <libicq2000/time_extra.h>
+
 using std::auto_ptr;
 using std::istringstream;
 
@@ -243,6 +245,34 @@ namespace ICQ2000 {
     b.setAutoSizeMarker(m2);
   }
 
+  SrvRequestKeywordSearch::SrvRequestKeywordSearch(unsigned int my_uin, const string& keyword)
+    : m_my_uin(my_uin), m_keyword(keyword)
+  { }
+  
+  void SrvRequestKeywordSearch::OutputBody(Buffer& b) const
+  { 
+    b << (unsigned short)0x0001;
+
+    Buffer::marker m1 = b.getAutoSizeShortMarker();
+
+    b.setLittleEndian();
+    Buffer::marker m2 = b.getAutoSizeShortMarker();
+
+    b << m_my_uin;
+
+    b << (unsigned short)2000	     /* type 9808 */
+      << (unsigned short)m_requestID /* low word of the request ID */
+      << (unsigned short)0x055F      /* subtype keyword-search */
+      << (unsigned short)0x0226;     /* unknown */
+
+    Buffer::marker m3 = b.getAutoSizeShortMarker();
+    b.PackUint16TranslatedNull(m_keyword);
+    b.setAutoSizeMarker(m3);
+    
+    b.setAutoSizeMarker(m1);
+    b.setAutoSizeMarker(m2);
+  }
+  
   SrvRequestDetailUserInfo::SrvRequestDetailUserInfo(unsigned int my_uin, unsigned int user_uin)
     : m_my_uin(my_uin), m_user_uin(user_uin) { }
 
@@ -261,7 +291,7 @@ namespace ICQ2000 {
    
   }
 
-  SrvUpdateMainHomeInfo::SrvUpdateMainHomeInfo(unsigned int my_uin, const MainHomeInfo& main_home_info)
+  SrvUpdateMainHomeInfo::SrvUpdateMainHomeInfo(unsigned int my_uin, const Contact::MainHomeInfo& main_home_info)
     : m_my_uin(my_uin), m_main_home_info(main_home_info) { }
     
   void SrvUpdateMainHomeInfo::OutputBody(Buffer& b) const {
@@ -296,7 +326,7 @@ namespace ICQ2000 {
         b.setAutoSizeMarker(m2);
     }
     
-  SrvUpdateWorkInfo::SrvUpdateWorkInfo(unsigned int my_uin, const WorkInfo& work_info)
+  SrvUpdateWorkInfo::SrvUpdateWorkInfo(unsigned int my_uin, const Contact::WorkInfo& work_info)
     : m_my_uin(my_uin), m_work_info(work_info) { }
     
   void SrvUpdateWorkInfo::OutputBody(Buffer& b) const {
@@ -328,7 +358,7 @@ namespace ICQ2000 {
         b.setAutoSizeMarker(m2);
     }
     
-  SrvUpdateHomepageInfo::SrvUpdateHomepageInfo(unsigned int my_uin, const HomepageInfo& homepage_info)
+  SrvUpdateHomepageInfo::SrvUpdateHomepageInfo(unsigned int my_uin, const Contact::HomepageInfo& homepage_info)
     : m_my_uin(my_uin), m_homepage_info(homepage_info) { }
     
   void SrvUpdateHomepageInfo::OutputBody(Buffer& b) const {
@@ -450,12 +480,19 @@ namespace ICQ2000 {
     timetm.tm_mday = day;
     timetm.tm_mon = month-1;
     timetm.tm_year = year-1900;
-    
-    m_time = mktime(&timetm);
+    timetm.tm_isdst = 0;
+
+    m_time = gmt_mktime(&timetm);
 
     m_type = OfflineMessage;
-    m_icqsubtype = ICQSubType::ParseICQSubType(b, false);
+    m_icqsubtype = ICQSubType::ParseICQSubType(b, false, false);
+    /* offline message is non-advanced, and not an ack */
     b.advance(2); // unknown
+
+    if (m_icqsubtype != NULL && dynamic_cast<UINICQSubType*>(m_icqsubtype) != NULL) {
+      UINICQSubType *ust = dynamic_cast<UINICQSubType*>(m_icqsubtype);
+      ust->setSource( m_sender_UIN );
+    }
   }
 
   void SrvResponseSNAC::ParseICQResponse(Buffer& b) {
@@ -565,11 +602,14 @@ namespace ICQ2000 {
 
     XmlLeaf *deliverable = sms_response->getLeaf("deliverable");
     m_deliverable = false;
+    m_smtp_deliverable = false;
+    
     if (deliverable != NULL) {
       if (deliverable->getValue() == "Yes") m_deliverable = true;
-      if (deliverable->getValue() == "SMTP")
-	throw ParseException("SMS messages for your provider must be sent via an SMTP (email) proxy, "
-			     "ickle doesn't support that yet, but may in the future.");
+      if (deliverable->getValue() == "SMTP") {
+	m_deliverable = false;
+	m_smtp_deliverable = true;
+      }
     }
 
     if (m_deliverable) {
@@ -585,6 +625,18 @@ namespace ICQ2000 {
       if (messages_left != NULL) m_messages_left = messages_left->getValue();
       // always 0, unsurprisingly
 
+    } else if (m_smtp_deliverable) {
+      // -- deliverable = SMTP --
+      
+      XmlLeaf *smtp_from = sms_response->getLeaf("from");
+      if (smtp_from != NULL) m_smtp_from = smtp_from->getValue();
+
+      XmlLeaf *smtp_to = sms_response->getLeaf("to");
+      if (smtp_to != NULL) m_smtp_to = smtp_to->getValue();
+
+      XmlLeaf *smtp_subject = sms_response->getLeaf("subject");
+      if (smtp_subject != NULL) m_smtp_subject = smtp_subject->getValue();
+
     } else {
       // -- deliverable = No --
 
@@ -786,7 +838,8 @@ namespace ICQ2000 {
       m_status = STATUS_ONLINE;
       break;
     case 2:
-      m_status = STATUS_NA;
+      // status disabled
+      m_status = STATUS_OFFLINE;
       break;
     default:
       m_status = STATUS_OFFLINE;
@@ -794,6 +847,12 @@ namespace ICQ2000 {
 
     b >> wb; // unknown
 
+    if (b.remains() == 3 || b.remains() == 7) {
+      b >> m_sex;
+      b >> m_age;
+      b >> wb;       // unknown
+    }
+
     if (subtype == SrvResponse_SimpleUI_Done || subtype == SrvResponse_SearchUI_Done) {
       b >> m_more_results;
     }
