@@ -225,23 +225,26 @@ namespace ICQ2000 {
   PasswordTLV::PasswordTLV(const string& pw) : m_password(pw) { }
   void PasswordTLV::OutputValue(Buffer& b) const {
     b << (unsigned short)m_password.size();
-    for(int i = 0; i < m_password.size(); i++)
+    for(unsigned int i = 0; i < m_password.size(); i++)
       b << (unsigned char)(m_password[i] ^ XORtable[i%16]);
 
   }
 
   // ----------------- UserInfoCapabilities TLV -----------
 
+  UserInfoCapabilitiesTLV::UserInfoCapabilitiesTLV()
+  {
+    m_capabilities.default_icq2000_capabilities();
+  }
+  
+  unsigned short UserInfoCapabilitiesTLV::Length() const
+  {
+    return m_capabilities.get_length();
+  }
+
   void UserInfoCapabilitiesTLV::OutputValue(Buffer& b) const {
     b << Length();
-    b << (unsigned int)0x09461349
-      << (unsigned int)0x4c7f11d1
-      << (unsigned int)0x82224445
-      << (unsigned int)0x53540000
-      << (unsigned int)0x09461344
-      << (unsigned int)0x4c7f11d1
-      << (unsigned int)0x82224445
-      << (unsigned int)0x53540000;
+    m_capabilities.Output(b);
   }
 
   // ----------------- Capabilities TLV -----------
@@ -254,13 +257,14 @@ namespace ICQ2000 {
     // ICQ2000 sends out 32 bytes
     // ICQLite sends out 16, so this is what we'll use for now until someone
     // figures out what those capabilities mean
-    if (l > 16)
-      m_accept_adv_msgs = true;
-    else
-      m_accept_adv_msgs = false;
+
+    m_capabilities.Parse(b, l);
   }
 
-  bool CapabilitiesTLV::getAcceptAdvMsgs() const { return m_accept_adv_msgs; }
+  Capabilities CapabilitiesTLV::get_capabilities() const
+  {
+    return m_capabilities;
+  }
 
   // ----------------- Status TLV -----------------
 
@@ -330,10 +334,10 @@ namespace ICQ2000 {
   // ----------------- LAN Details TLV ------------
 
   LANDetailsTLV::LANDetailsTLV()
-    : m_firewall(0x0400), m_tcp_version(7) { }
+    : m_firewall(0x0400), m_tcp_version(7), m_dc_cookie(0) { }
 
   LANDetailsTLV::LANDetailsTLV(unsigned int ip, unsigned short port)
-    : m_tcp_version(7), m_lan_ip(ip), m_lan_port(port), m_firewall(0x0400) { }
+    : m_lan_ip(ip), m_lan_port(port), m_firewall(0x0400), m_tcp_version(7), m_dc_cookie(0) { }
 
   void LANDetailsTLV::ParseValue(Buffer& b) {
     unsigned short length;
@@ -346,17 +350,22 @@ namespace ICQ2000 {
       b >> m_lan_port;
     }
 
-    unsigned int waste_int;
-    unsigned short waste_short;
+    unsigned short no_timestamps;
     b >> m_firewall
       >> m_tcp_version
-      >> waste_int // unknown
-      >> waste_int // always 0x00000050
-      >> waste_int // always 0x00000003
-      >> waste_int // unknown
-      >> waste_int // unknown
-      >> waste_int // unknown
-      >> waste_short; // unknown
+      >> m_dc_cookie;
+    b.advance(4); // always 0x00000050
+    b.advance(2); // unknown
+
+    b >> no_timestamps;
+    if (no_timestamps > 100) no_timestamps = 100; // place a sensible limit
+    
+    while(no_timestamps-- > 0) {
+      b.advance(4);
+      // don't really know the purpose of these timestamps
+    }
+    
+    b.advance(2); // unknown
   }
 
   void LANDetailsTLV::OutputValue(Buffer& b) const {
@@ -365,21 +374,13 @@ namespace ICQ2000 {
     b << (unsigned int)m_lan_port;
     b << m_firewall
       << m_tcp_version
-      << (unsigned int)0x279c6996
+      << (unsigned int)0x00000000 // dc cookie (server picks it for us (apparently))
       << (unsigned int)0x00000050
-      << (unsigned int)0x00000003
-
-      //      << (unsigned int)0x3bb9d506
-      //      << (unsigned int)0x3bb9d4f9
-      //      << (unsigned int)0x3bb9d4fa
-      << (unsigned int)0x3AA773EE
-      << (unsigned int)0x3AA66380
-      << (unsigned int)0x3A877A42
-
-
-      //      << (unsigned int)0x00000000
-      //      << (unsigned int)0x00000000
-      //      << (unsigned int)0x00000000
+      << (unsigned short)0x0000
+      << (unsigned short)0x0003   // number of timestamps
+      << (unsigned int)0x3AA773EE // timestamp1
+      << (unsigned int)0x3AA66380 // timestamp1
+      << (unsigned int)0x3A877A42 // timestamp1
       << (unsigned short)0x0000;
   }
 
@@ -485,7 +486,11 @@ namespace ICQ2000 {
 
     b.advance(12); // unknown - all zeroes
 
-    m_icqsubtype = ICQSubType::ParseICQSubType(b, true);
+    m_icqsubtype = ICQSubType::ParseICQSubType(b, true, false);
+    /* this TLV occurs in the MessageSNAC
+       it is in advanced form and is not an ack
+       (otherwise it'd be in MessageACKSNAC) */
+    
     if (m_icqsubtype != NULL) m_icqsubtype->setSeqNum(seqnum);
     
     if (unknown == 0x0012) {
@@ -529,8 +534,8 @@ namespace ICQ2000 {
     unsigned int uin;
     b >> uin;
 
-    m_icqsubtype = ICQSubType::ParseICQSubType(b, false);
-    
+    m_icqsubtype = ICQSubType::ParseICQSubType(b, false, false);
+    /* non-advanced, non-ack message */
   }
 
   // ----------------- TLV List -------------------
