@@ -25,7 +25,9 @@
 
 namespace ICQ2000 {
 
-  // --------------- Event ---------------------------
+  // ============================================================================
+  //  Event base class
+  // ============================================================================
 
   /**
    *  Base constructor for events, timestamp set to now.
@@ -54,7 +56,9 @@ namespace ICQ2000 {
    */
   void Event::setTime(time_t t) { m_time = t; }
 
-  // --------------- Socket Event --------------------
+  // ============================================================================
+  //  Socket Event
+  // ============================================================================
 
   /**
    *  Base constructor for socket events.
@@ -120,14 +124,27 @@ namespace ICQ2000 {
   RemoveSocketHandleEvent::RemoveSocketHandleEvent(int fd)
     : SocketEvent(fd) { }
 
-  // --------------- Connnected Event ----------------
+  // ============================================================================
+  //  Connecting Event
+  // ============================================================================
+
+  /**
+   *  Simple constructor for a ConnectingEvent.
+   */
+  ConnectingEvent::ConnectingEvent() { }
+
+  // ============================================================================
+  //  Connected Event
+  // ============================================================================
 
   /**
    *  Simple constructor for a ConnectedEvent.
    */
   ConnectedEvent::ConnectedEvent() { }
 
-  // --------------- Disconnected Event --------------
+  // ============================================================================
+  //  Disconnected Event
+  // ============================================================================
 
   /**
    *  Constructor for a DisconnectedEvent.
@@ -141,7 +158,9 @@ namespace ICQ2000 {
    */
   DisconnectedEvent::Reason DisconnectedEvent::getReason() const { return m_reason; }
   
-  // --------------- Log Event -----------------------
+  // ============================================================================
+  //  Log Event
+  // ============================================================================
 
   /**
    *  Constructor for a LogEvent.
@@ -166,21 +185,23 @@ namespace ICQ2000 {
    */
   string LogEvent::getMessage() const { return m_msg; }
 
-  // --------------- ContactList Event ---------------
+  // ============================================================================
+  //  Contact List Event
+  // ============================================================================
 
   /**
    *  Base constructor for contact list events.
    *
    * @param c the contact
    */
-  ContactListEvent::ContactListEvent(Contact *c) { m_contact = c; }
+  ContactListEvent::ContactListEvent(ContactRef c) { m_contact = c; }
 
   /**
    *  get the contact
    *
    * @return the contact
    */
-  Contact *ContactListEvent::getContact() const { return m_contact; }
+  ContactRef ContactListEvent::getContact() const { return m_contact; }
 
   /**
    *  get the uin of the contact. This could be done just as easily,
@@ -195,7 +216,71 @@ namespace ICQ2000 {
    */
   ContactListEvent::~ContactListEvent() { }
 
-  // ----------------- StatusChange Event ----------------
+  // ============================================================================
+  //  UserAdded Event
+  // ============================================================================
+
+  /**
+   *  Constructor for UserAddedEvent
+   *
+   * @param contact the contact that has just been added
+   */
+  UserAddedEvent::UserAddedEvent(ContactRef contact) : ContactListEvent(contact) { }
+  ContactListEvent::EventType UserAddedEvent::getType() const { return UserAdded; }
+
+  // ============================================================================
+  //  UserRemoved Event
+  // ============================================================================
+
+  /**
+   *  Constructor for UserRemovedEvent
+   *
+   * @param contact the contact that is about to be removed
+   */
+  UserRemovedEvent::UserRemovedEvent(ContactRef contact) : ContactListEvent(contact) { }
+  ContactListEvent::EventType UserRemovedEvent::getType() const { return UserRemoved; }
+
+  // ============================================================================
+  //  ServerBasedContactEvent
+  // ============================================================================
+
+  ServerBasedContactEvent::ServerBasedContactEvent(const ContactList& l) : m_clist(l) { }
+  ContactList& ServerBasedContactEvent::getContactList() { return m_clist; }
+
+  // ============================================================================
+  //  Contact Event
+  // ============================================================================
+
+  /**
+   *  Base constructor for contact list events.
+   *
+   * @param c the contact
+   */
+  ContactEvent::ContactEvent(ContactRef c) { m_contact = c; }
+
+  /**
+   *  get the contact
+   *
+   * @return the contact
+   */
+  ContactRef ContactEvent::getContact() const { return m_contact; }
+
+  /**
+   *  get the uin of the contact. This could be done just as easily,
+   *  with getContact()->getUIN(), provided for convenience.
+   *
+   * @return
+   */
+  unsigned int ContactEvent::getUIN() const { return m_contact->getUIN(); }
+    
+  /**
+   *  Destructor for ContactEvent
+   */
+  ContactEvent::~ContactEvent() { }
+
+  // ============================================================================
+  //  Status Change Event
+  // ============================================================================
 
   /**
    *  Constructor for StatusChangeEvent
@@ -204,10 +289,10 @@ namespace ICQ2000 {
    * @param st the new status
    * @param old_st the old status
    */
-  StatusChangeEvent::StatusChangeEvent(Contact* contact, Status st, Status old_st)
-    : ContactListEvent(contact), m_status(st), m_old_status(old_st) { }
+  StatusChangeEvent::StatusChangeEvent(ContactRef contact, Status st, Status old_st)
+    : ContactEvent(contact), m_status(st), m_old_status(old_st) { }
   
-  ContactListEvent::EventType StatusChangeEvent::getType() const { return StatusChange; }
+  ContactEvent::EventType StatusChangeEvent::getType() const { return StatusChange; }
 
   /**
    *  get the new status of the contact
@@ -223,42 +308,22 @@ namespace ICQ2000 {
    */
   Status StatusChangeEvent::getOldStatus() const { return m_old_status; }
 
-  // ----------------- UserAdded Event -------------------
-
-  /**
-   *  Constructor for UserAddedEvent
-   *
-   * @param contact the contact that has just been added
-   */
-  UserAddedEvent::UserAddedEvent(Contact* contact) : ContactListEvent(contact) { }
-  ContactListEvent::EventType UserAddedEvent::getType() const { return UserAdded; }
-
-  // ----------------- ServerBasedContact Event -------------------
-
-  ServerBasedContactEvent::ServerBasedContactEvent(Contact* contact) : ContactListEvent(contact) { }
-  ContactListEvent::EventType ServerBasedContactEvent::getType() const { return ServerBasedContact; }
-
-  // ----------------- UserRemoved Event -------------------
-
-  /**
-   *  Constructor for UserRemovedEvent
-   *
-   * @param contact the contact that is about to be removed
-   */
-  UserRemovedEvent::UserRemovedEvent(Contact* contact) : ContactListEvent(contact) { }
-  ContactListEvent::EventType UserRemovedEvent::getType() const { return UserRemoved; }
-
-  // ----------------- UserInfoChange Event -------------------
+  // ============================================================================
+  //  User Info Change Event
+  // ============================================================================
 
   /**
    *  Constructor for UserInfoChangeEvent
    *
    * @param contact the contact whose information has changed
    */
-  UserInfoChangeEvent::UserInfoChangeEvent(Contact* contact) : ContactListEvent(contact) { }
-  ContactListEvent::EventType UserInfoChangeEvent::getType() const { return UserInfoChange; }
+  UserInfoChangeEvent::UserInfoChangeEvent(ContactRef contact, bool is_transient_detail) : ContactEvent(contact), m_is_transient_detail(is_transient_detail) { }
+  ContactEvent::EventType UserInfoChangeEvent::getType() const { return UserInfoChange; }
+  bool UserInfoChangeEvent::isTransientDetail() const { return m_is_transient_detail; }
 
-  // ----------------- SearchResult Event -------------------
+  // ============================================================================
+  //  Search Result Event
+  // ============================================================================
 
   /**
    *  Constructor for a SearchResultEvent
@@ -268,11 +333,11 @@ namespace ICQ2000 {
       m_last_contact(NULL), m_more_results(0)
   { }
 
-  Contact* SearchResultEvent::getLastContactAdded() const { return m_last_contact; }
+  ContactRef SearchResultEvent::getLastContactAdded() const { return m_last_contact; }
   
   ContactList& SearchResultEvent::getContactList() { return m_clist; }
   
-  void SearchResultEvent::setLastContactAdded(Contact *c) { m_last_contact = c; }
+  void SearchResultEvent::setLastContactAdded(ContactRef c) { m_last_contact = c; }
   
   SearchResultEvent::SearchType SearchResultEvent::getSearchType() const { return m_searchtype; }
   
@@ -288,24 +353,18 @@ namespace ICQ2000 {
   
   void SearchResultEvent::setNumberMoreResults(unsigned int m) { m_more_results = m; }
 
-  // ----------------- MessageQueueChangedEvent -------------------
-
-  /**
-   *  Constructor for MessageQueueChangedEvent
-   *
-   * @param contact the contact whose message queue changed
-   */
-  MessageQueueChangedEvent::MessageQueueChangedEvent(Contact* contact) : ContactListEvent(contact) { }
-  ContactListEvent::EventType MessageQueueChangedEvent::getType() const { return MessageQueueChanged; }
-
-  // --------------- Message Event -------------------
+  // ============================================================================
+  //  Message Event
+  // ============================================================================
 
   /**
    *  Constructor for a MessageEvent
    *
    * @param c the contact related to this event
    */
-  MessageEvent::MessageEvent(Contact *c) : m_contact(c) { }
+  MessageEvent::MessageEvent(ContactRef c)
+    : m_contact(c)
+  { }
 
   /**
    *  Destructor for MessageEvent
@@ -317,7 +376,7 @@ namespace ICQ2000 {
    *
    * @return the contact related to the event
    */
-  Contact* MessageEvent::getContact() { return m_contact; }
+  ContactRef MessageEvent::getContact() { return m_contact; }
 
   /**
    *  get if a message event is finished.  This is used in the message
@@ -352,9 +411,7 @@ namespace ICQ2000 {
   void MessageEvent::setFinished(bool f) { m_finished = f; }
 
   /**
-   *  set whether the message has been delivered.  This is used
-   *  internally by the library and is of no interest to the client.
-   *
+   *  set whether the message has been delivered.
    * @param f if message was delivered
    */
   void MessageEvent::setDelivered(bool f) { m_delivered = f; }
@@ -367,15 +424,42 @@ namespace ICQ2000 {
    */
   void MessageEvent::setDirect(bool f) { m_direct = f; }
 
-  // --------------- ICQ Message Event -------------------
+  /**
+   *  set the reason for delivery failure. Used to indicate the
+   * failure reason when a client marks a message as not accepted (by
+   * setDeliverd(false) in the messaged callback).
+   *
+   * @param d delivery failure reason
+   */
+  void MessageEvent::setDeliveryFailureReason(DeliveryFailureReason d)
+  {
+    m_failure_reason = d;
+  }
+
+  /**
+   *  get the reason for delivery failure. Defined when getFinished ==
+   *  false and getDelivered == false too.  This is used in the
+   *  message ack'ing system.
+   *
+   * @return reason for delivery failing
+   */
+  MessageEvent::DeliveryFailureReason MessageEvent::getDeliveryFailureReason() const
+  {
+    return m_failure_reason;
+  }
+
+  // ============================================================================
+  //  ICQ Message Event
+  // ============================================================================
 
   /**
    *  Constructor for a ICQMessageEvent
    *
    * @param c the contact related to this event
    */
-  ICQMessageEvent::ICQMessageEvent(Contact *c)
-    : MessageEvent(c), m_urgent(false), m_tocontactlist(false) { }
+  ICQMessageEvent::ICQMessageEvent(ContactRef c)
+    : MessageEvent(c), m_urgent(false), m_tocontactlist(false), m_offline(false)
+  { }
 
   /**
    *  get whether the message was sent marked as urgent
@@ -389,6 +473,7 @@ namespace ICQ2000 {
 
   /**
    *  set whether the message shall be marked as urgent
+   * @param b urgent
    */
   void ICQMessageEvent::setUrgent(bool b)
   {
@@ -413,6 +498,18 @@ namespace ICQ2000 {
     m_tocontactlist = b;
   }
 
+  /**
+   *  get if this was an offline message
+   *
+   * @return if this was an offline message
+   */
+  bool ICQMessageEvent::isOfflineMessage() const { return m_offline; }
+    
+  /**
+   *  set whether this was an offline message
+   */
+  void ICQMessageEvent::setOfflineMessage(bool b) { m_offline = b; }
+    
   /**
    *  get the uin of the sender.  This is really miss-named, if you
    *  were sending the message, this would be the UIN of the recipient.
@@ -421,7 +518,23 @@ namespace ICQ2000 {
    */
   unsigned int ICQMessageEvent::getSenderUIN() const { return m_contact->getUIN(); }
 
-  // ---------------- Normal Message ---------------------
+  /**
+   *  get the away message
+   *
+   * @return the away message
+   */
+  string ICQMessageEvent::getAwayMessage() const { return m_away_message; }
+
+  /**
+   *  set the away message
+   *
+   * @param msg the away message
+   */
+  void ICQMessageEvent::setAwayMessage(const string& msg) { m_away_message = msg; }
+
+  // ============================================================================
+  //  Normal Message
+  // ============================================================================
 
   /**
    *  Construct a NormalMessageEvent.
@@ -430,8 +543,8 @@ namespace ICQ2000 {
    * @param msg the message
    * @param multi tag message as a multireceipt message
    */
-  NormalMessageEvent::NormalMessageEvent(Contact* c, const string& msg, bool multi)
-    : ICQMessageEvent(c), m_message(msg), m_offline(false), m_multi(multi),
+  NormalMessageEvent::NormalMessageEvent(ContactRef c, const string& msg, bool multi)
+    : ICQMessageEvent(c), m_message(msg), m_multi(multi),
       m_foreground(0x00000000), m_background(0x00ffffff) {
     setDirect(false);
   }
@@ -444,10 +557,11 @@ namespace ICQ2000 {
    * @param multi tag message as a multireceipt message
    * @param t the time the message was sent
    */
-  NormalMessageEvent::NormalMessageEvent(Contact *c, const string& msg, time_t t, bool multi)
-    : ICQMessageEvent(c), m_message(msg), m_offline(true), m_multi(multi),
+  NormalMessageEvent::NormalMessageEvent(ContactRef c, const string& msg, time_t t, bool multi)
+    : ICQMessageEvent(c), m_message(msg), m_multi(multi),
       m_foreground(0x00000000), m_background(0x00ffffff) {
     setDirect(false);
+    setOfflineMessage(true);
     m_time = t;
   }
 
@@ -459,8 +573,8 @@ namespace ICQ2000 {
    * @param fg foreground colour for the message
    * @param bg background colour for the message
    */
-  NormalMessageEvent::NormalMessageEvent(Contact *c, const string& msg, unsigned int fg, unsigned int bg)
-    : ICQMessageEvent(c), m_message(msg), m_offline(false), m_multi(false) /* todo */,
+  NormalMessageEvent::NormalMessageEvent(ContactRef c, const string& msg, unsigned int fg, unsigned int bg)
+    : ICQMessageEvent(c), m_message(msg), m_multi(false) /* todo */,
       m_foreground(fg), m_background(bg) {
     setDirect(true);
   }
@@ -474,13 +588,6 @@ namespace ICQ2000 {
    */
   string NormalMessageEvent::getMessage() const { return m_message; }
 
-  /**
-   *  get if the message was an offline message
-   *
-   * @return if the message was an offline message
-   */
-  bool NormalMessageEvent::isOfflineMessage() const { return m_offline; }
-
   /**
    *  get if the message is a multiparty message
    *
@@ -516,7 +623,14 @@ namespace ICQ2000 {
    */
   void NormalMessageEvent::setBackground(unsigned int b) { m_background = b; }
 
-  // ---------------- URL Message ---------------------
+  ICQMessageEvent* NormalMessageEvent::copy() const
+  {
+    return new NormalMessageEvent(*this);
+  }
+
+  // ============================================================================
+  //  URL Message
+  // ============================================================================
 
   /**
    *  Construct an URLMessageEvent
@@ -525,8 +639,8 @@ namespace ICQ2000 {
    * @param msg the message
    * @param url the url
    */
-  URLMessageEvent::URLMessageEvent(Contact* c, const string& msg, const string& url)
-    : ICQMessageEvent(c), m_message(msg), m_url(url), m_offline(false) { }
+  URLMessageEvent::URLMessageEvent(ContactRef c, const string& msg, const string& url)
+    : ICQMessageEvent(c), m_message(msg), m_url(url) { }
 
   /**
    *  Construct an URLMessageEvent. This constructor is only used by the library.
@@ -536,8 +650,9 @@ namespace ICQ2000 {
    * @param url the url
    * @param t time of sending
    */
-  URLMessageEvent::URLMessageEvent(Contact *c, const string& msg, const string& url, time_t t)
-    : ICQMessageEvent(c), m_message(msg), m_url(url), m_offline(true) {
+  URLMessageEvent::URLMessageEvent(ContactRef c, const string& msg, const string& url, time_t t)
+    : ICQMessageEvent(c), m_message(msg), m_url(url) {
+    setOfflineMessage(true);
     m_time = t;
   }
 
@@ -557,14 +672,14 @@ namespace ICQ2000 {
    */
   string URLMessageEvent::getURL() const { return m_url; }
 
-  /**
-   *  get if the message was an offline message
-   *
-   * @return if the message was an offline message
-   */
-  bool URLMessageEvent::isOfflineMessage() const { return m_offline; }
+  ICQMessageEvent* URLMessageEvent::copy() const
+  {
+    return new URLMessageEvent(*this);
+  }
 
-  // ---------------- SMS Message ------------------------
+  // ============================================================================
+  //  SMS Message
+  // ============================================================================
 
   /**
    *  Construct an SMSMessageEvent.
@@ -577,7 +692,7 @@ namespace ICQ2000 {
    *
    * @todo fix parsing of time
    */
-  SMSMessageEvent::SMSMessageEvent(Contact *c, const string& msg, const string& source,
+  SMSMessageEvent::SMSMessageEvent(ContactRef c, const string& msg, const string& source,
 				   const string& senders_network, const string& time)
     : MessageEvent(c), m_message(msg), m_source(source),
       m_senders_network(senders_network) {
@@ -591,7 +706,7 @@ namespace ICQ2000 {
    * @param msg the message
    * @param rcpt whether to request a delivery receipt
    */
-  SMSMessageEvent::SMSMessageEvent(Contact *c, const string& msg, bool rcpt)
+  SMSMessageEvent::SMSMessageEvent(ContactRef c, const string& msg, bool rcpt)
     : MessageEvent(c), m_message(msg), m_rcpt(rcpt) { }
 
   MessageEvent::MessageType SMSMessageEvent::getType() const { return MessageEvent::SMS; }
@@ -632,7 +747,21 @@ namespace ICQ2000 {
    */
   bool SMSMessageEvent::getRcpt() const { return m_rcpt; }
 
-  // ---------------- SMS Receipt ------------------------
+  void SMSMessageEvent::setSMTPFrom(const string& from) { m_smtp_from = from; }
+
+  string SMSMessageEvent::getSMTPFrom() const { return m_smtp_from; }
+
+  void SMSMessageEvent::setSMTPTo(const string& to) { m_smtp_to = to; }
+
+  string SMSMessageEvent::getSMTPTo() const { return m_smtp_to; }
+
+  void SMSMessageEvent::setSMTPSubject(const string& subj) { m_smtp_subject = subj; }
+
+  string SMSMessageEvent::getSMTPSubject() const { return m_smtp_subject; }
+
+  // ============================================================================
+  //  SMS Receipt Event
+  // ============================================================================
 
   /**
    *  Construct an SMSReceiptEvent
@@ -644,7 +773,7 @@ namespace ICQ2000 {
    * @param delivery_time time of delivery
    * @param del if the message was delivered
    */
-  SMSReceiptEvent::SMSReceiptEvent(Contact *c, const string& msg, const string& message_id,
+  SMSReceiptEvent::SMSReceiptEvent(ContactRef c, const string& msg, const string& message_id,
 				   const string& submission_time, const string& delivery_time, bool del)
     : MessageEvent(c), m_message(msg), m_message_id(message_id),
       m_submission_time(submission_time), m_delivery_time(delivery_time), m_delivered(del) { }
@@ -694,33 +823,28 @@ namespace ICQ2000 {
   bool SMSReceiptEvent::delivered() const { return m_delivered; }
 
 
-  // ---------------- Away Message -----------------------
+  // ============================================================================
+  //  Away Message
+  // ============================================================================
 
   /**
    *  Construct an Away message
    *
    * @param c the contact
    */
-  AwayMessageEvent::AwayMessageEvent(Contact *c)
+  AwayMessageEvent::AwayMessageEvent(ContactRef c)
     : ICQMessageEvent(c) { }
 
   MessageEvent::MessageType AwayMessageEvent::getType() const { return MessageEvent::AwayMessage; }
 
-  /**
-   *  get the away message
-   *
-   * @return the away message
-   */
-  string AwayMessageEvent::getMessage() const { return m_message; }
+  ICQMessageEvent* AwayMessageEvent::copy() const
+  {
+    return new AwayMessageEvent(*this);
+  }
 
-  /**
-   *  set the away message
-   *
-   * @param msg the away message
-   */
-  void AwayMessageEvent::setMessage(const string& msg) { m_message = msg; }
-
-  // ---------------- Authorisation Request -----------------------
+  // ============================================================================
+  //  Authorisation Request
+  // ============================================================================
 
   /**
    *  Constructor for the Authorisation Request
@@ -728,7 +852,7 @@ namespace ICQ2000 {
    * @param c the contact
    * @param msg authorisation message
    */
-  AuthReqEvent::AuthReqEvent(Contact* c, const string& msg)
+  AuthReqEvent::AuthReqEvent(ContactRef c, const string& msg)
     : ICQMessageEvent(c), m_message(msg) {}
     
   /**
@@ -737,9 +861,10 @@ namespace ICQ2000 {
    * @param c the contact
    * @param msg authorisation message
    */
-  AuthReqEvent::AuthReqEvent(Contact* c, const string& msg, time_t t)
-    : ICQMessageEvent(c), m_message(msg), m_offline(true) {
-    m_time=t;  
+  AuthReqEvent::AuthReqEvent(ContactRef c, const string& msg, time_t t)
+    : ICQMessageEvent(c), m_message(msg) {
+    setOfflineMessage(true);
+    m_time = t;  
   }
 
   MessageEvent::MessageType AuthReqEvent::getType() const { 
@@ -753,14 +878,14 @@ namespace ICQ2000 {
    */
   string AuthReqEvent::getMessage() const { return m_message; }
 
-  /**
-   *  get if this was an offline message
-   *
-   * @return if this was an offline message
-   */
-  bool AuthReqEvent::isOfflineMessage() const { return m_offline; }
-    
-  // ---------------- Authorisation Acknowledgement ---------------
+  ICQMessageEvent* AuthReqEvent::copy() const
+  {
+    return new AuthReqEvent(*this);
+  }
+
+  // ============================================================================
+  //  Authorisation Acknowledgement
+  // ============================================================================
 
   /**
    *  Constructor for the Authorisation Acknowledgement
@@ -768,8 +893,8 @@ namespace ICQ2000 {
    * @param c the contact
    * @param granted if authorisation was granted
    */
-  AuthAckEvent::AuthAckEvent(Contact* c, bool granted)
-    : ICQMessageEvent(c), m_offline(false), m_granted(granted) {}
+  AuthAckEvent::AuthAckEvent(ContactRef c, bool granted)
+    : ICQMessageEvent(c), m_granted(granted) {}
       
   /**
    *  Constructor for the Authorisation Acknowledgement
@@ -778,9 +903,8 @@ namespace ICQ2000 {
    * @param msg the authorisation message
    * @param granted if authorisation was granted
    */
-  AuthAckEvent::AuthAckEvent(Contact* c, const string& msg, bool granted)
-    : ICQMessageEvent(c),  m_message(msg), m_offline(false), 
-      m_granted(granted) {}
+  AuthAckEvent::AuthAckEvent(ContactRef c, const string& msg, bool granted)
+    : ICQMessageEvent(c),  m_message(msg), m_granted(granted) {}
       
   /**
    *  Constructor for the Authorisation Acknowledgement
@@ -789,9 +913,10 @@ namespace ICQ2000 {
    * @param granted if authorisation was granted
    * @param t time the message was sent
    */
-  AuthAckEvent::AuthAckEvent(Contact* c, bool granted, time_t t)
-    : ICQMessageEvent(c), m_offline(true), m_granted(granted) {
-    m_time=t;  
+  AuthAckEvent::AuthAckEvent(ContactRef c, bool granted, time_t t)
+    : ICQMessageEvent(c), m_granted(granted) {
+    setOfflineMessage(true);
+    m_time = t;
   }
 
   /**
@@ -802,10 +927,11 @@ namespace ICQ2000 {
    * @param granted if authorisation was granted
    * @param t time the message was sent
    */
-  AuthAckEvent::AuthAckEvent(Contact* c, const string& msg,
+  AuthAckEvent::AuthAckEvent(ContactRef c, const string& msg,
                              bool granted, time_t t)
-    : ICQMessageEvent(c), m_message(msg), m_offline(true), m_granted(granted) {
-    m_time=t;  
+    : ICQMessageEvent(c), m_message(msg), m_granted(granted) {
+    setOfflineMessage(true);
+    m_time = t;
   }
   
   MessageEvent::MessageType AuthAckEvent::getType() const { 
@@ -828,16 +954,16 @@ namespace ICQ2000 {
    */
   string AuthAckEvent::getMessage() const { return m_message; }
 
-  /**
-   *  get if the message was sent offline
-   *
-   * @return if the message was sent offline
-   */
-  bool AuthAckEvent::isOfflineMessage() const { return m_offline; }
+  ICQMessageEvent* AuthAckEvent::copy() const
+  {
+    return new AuthAckEvent(*this);
+  }
 
-  // ---------------- E-mail express -------------------
+  // ============================================================================
+  //  E-mail Express message
+  // ============================================================================
 
-  EmailExEvent::EmailExEvent(Contact* c, const string &email,
+  EmailExEvent::EmailExEvent(ContactRef c, const string &email,
 			     const string &sender, const string &msg)
   : MessageEvent(c), m_sender(sender), m_email(email), m_message(msg) {
   }
@@ -850,76 +976,53 @@ namespace ICQ2000 {
 
   MessageEvent::MessageType EmailExEvent::getType() const { return MessageEvent::EmailEx; }
 
-  unsigned int EmailExEvent::getSenderUIN() const { return m_contact->getUIN(); }
+  // ============================================================================
+  //  Web Pager message
+  // ============================================================================
 
-  // ---------------- "You were added" message -------------------
+  WebPagerEvent::WebPagerEvent(ContactRef c, const string& email,
+			       const string& sender, const string& msg)
+    : MessageEvent(c), m_sender(sender), m_email(email), m_message(msg)
+  { }
 
-  UserAddEvent::UserAddEvent(Contact* c) : MessageEvent(c) { }
+  string WebPagerEvent::getMessage() const { return m_message; }
+
+  string WebPagerEvent::getEmail() const { return m_email; }
+
+  string WebPagerEvent::getSender() const { return m_sender; }
+
+  MessageEvent::MessageType WebPagerEvent::getType() const { return MessageEvent::WebPager; }
+
+  // ============================================================================
+  //  "You were added" message
+  // ============================================================================
+
+  UserAddEvent::UserAddEvent(ContactRef c) : ICQMessageEvent(c) { }
 
   MessageEvent::MessageType UserAddEvent::getType() const { return MessageEvent::UserAdd; }
 
   unsigned int UserAddEvent::getSenderUIN() const { return m_contact->getUIN(); }
 
-
-  // ---------------- Self Event -------------------------
-
-  SelfEvent::SelfEvent(Contact *self)
-    : m_self_contact(self) { }
-
-  SelfEvent::~SelfEvent() { }
-
-  Contact* SelfEvent::getSelfContact() const
+  ICQMessageEvent* UserAddEvent::copy() const
   {
-    return m_self_contact;
+    return new UserAddEvent(*this);
   }
 
-  // ---------------- My Status Change -------------------
+  // ============================================================================
+  //  Email Message
+  // ============================================================================
 
-  /**
-   *  Constructor for MyStatusChangeEvent
-   *
-   * @param s your status
-   */
-  MyStatusChangeEvent::MyStatusChangeEvent(Contact *self)
-    : SelfEvent(self)
-  {
-  }
-  
-
-  /**
-   *  get your status
-   *
-   * @return your status
-   */
-  Status MyStatusChangeEvent::getStatus() const { return m_self_contact->getStatus(); }
-
-  /**
-   *  get your invisibility
-   *
-   * @return your invisibility
-   */
-  bool MyStatusChangeEvent::getInvisible() const { return m_self_contact->isInvisible(); }
-
-  SelfEvent::EventType MyStatusChangeEvent::getType() const 
-  {
-    return SelfEvent::MyStatusChange;
+  EmailMessageEvent::EmailMessageEvent(ContactRef c, const string &msg)
+  : MessageEvent(c), m_message(msg) {
   }
 
-  // ---------------- My User Info Change -------------------
+  string EmailMessageEvent::getMessage() const { return m_message; }
 
-  /**
-   *  Constructor for MyUserInfoChangeEvent
-   *
-   */
-  MyUserInfoChangeEvent::MyUserInfoChangeEvent(Contact *self)
-    : SelfEvent(self) { }
+  EmailMessageEvent::MessageType EmailMessageEvent::getType() const { return MessageEvent::Email; }
 
-  SelfEvent::EventType MyUserInfoChangeEvent::getType() const 
-  {
-    return SelfEvent::MyUserInfoChange;
-  }
-
-  // ---------------- New UIN ----------------------------
+  // ============================================================================
+  //  New UIN
+  // ============================================================================
 
   /**
    *  Constructor for a NewUINEvent
@@ -945,7 +1048,9 @@ namespace ICQ2000 {
   bool NewUINEvent::isSuccess() const { return m_success; }
 
 
-  // ---------------- Rate Info Change -------------------
+  // ============================================================================
+  //  Rate Info Change
+  // ============================================================================
 
   /**
    *  Constructor for a RateInfoChangeEvent
