@@ -41,6 +41,9 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <stdarg.h>
+#include <unistd.h>
+#include <sys/time.h>
+#include <sys/types.h>
 
 #if defined(WITH_GTK)
 #include <gtk/gtk.h>
@@ -98,16 +101,19 @@ void SOCKSinit(char *argv0);
 /* pager server host */
 #define YAHOO_PAGER_HOST "cs.yahoo.com"
 #define YAHOO_PAGER_PORT        5050
-/* pager server host for http connections */
-#define YAHOO_PAGER_HTTP_HOST "http.pager.yahoo.com"
-#define YAHOO_PAGER_HTTP_PORT 80
+
 /* authentication/login host */
 #define YAHOO_AUTH_HOST         "msg.edit.yahoo.com"
 #define YAHOO_AUTH_PORT         80
+
 /* buddy/identity/config host */
 #define YAHOO_DATA_HOST         YAHOO_AUTH_HOST
 #define YAHOO_DATA_PORT     80
 
+/* pager server host for http connections */
+#define YAHOO_PAGER_HTTP_HOST "http.pager.yahoo.com"
+#define YAHOO_PAGER_HTTP_PORT 80
+
 /* User agent to use for HTTP connections */
 /* It needs to have Mozilla/4 in it, otherwise it fails */
 #ifndef VERSION
@@ -652,10 +658,23 @@ static int yahoo_tcp_readline(char *ptr, int maxlen, int fd)
 {
 	int n, rc;
 	char c;
+	fd_set fds;
+	struct timeval tv;
 
 	for (n = 1; n < maxlen; n++)
 	{
 	  again:
+		FD_ZERO(&fds);
+		FD_SET(fd, &fds);
+
+		tv.tv_sec = 3;
+		tv.tv_usec = 0;
+
+		if(select(fd+1, &fds, 0, 0, &tv) < 0)
+		    return(0);
+
+		if(!FD_ISSET(fd, &fds))
+		    return(0);
 
 		if ((rc = read(fd, &c, 1)) == 1)
 		{
@@ -690,6 +709,12 @@ static int yahoo_tcp_readline(char *ptr, int maxlen, int fd)
  * Published library interfaces
  */
 
+int yahoo_errno;
+
+int yahoo_geterrno() {
+    return yahoo_errno;
+}
+
 /* Initialize interface to yahoo library, sortof like a class object
    creation routine. */
 struct yahoo_context *yahoo_init(const char *user, const char *password,
@@ -697,6 +722,8 @@ struct yahoo_options *options)
 {
 	struct yahoo_context *tmp;
 
+	yahoo_errno = 0;
+
 	if (!user || !password)
 	{
 		return NULL;
@@ -842,6 +869,7 @@ int yahoo_fetchcookies(struct yahoo_context *ctx)
 	if (!servfd)
 	{
 		DBG_Print(ctx, "failed to connect to pager auth server");
+		yahoo_errno = YERR_CONNECT;
 		return (0);
 	}
 
@@ -886,6 +914,7 @@ int yahoo_fetchcookies(struct yahoo_context *ctx)
 
 		if (!strcasecmp(buffer, "ERROR: Invalid NCC Login"))
 		{
+			yahoo_errno = YERR_INVALID_LOGIN;
 			DBG_Print(ctx, "yahoo_fetchcookies: password was invalid");
 			return (0);
 		}
@@ -2659,206 +2688,172 @@ int yahoo_parsepacket_newcontact(struct yahoo_context *ctx,
 }
 
 int yahoo_parsepacket_status(struct yahoo_context *ctx,
-	struct yahoo_packet *pkt, struct yahoo_rawpacket *inpkt)
-{
-	char *content;
-	char *tmpc;
-	char *tmp1;
-	int i;
-	int len;
-	int index;
-	int realcount;
+struct yahoo_packet *pkt, struct yahoo_rawpacket *inpkt) {
+    char *content, *tmpc, *tmp1;
+    int i, len, index, realcount, contover;
 
-	/* Make working copy of content */
-	content = strdup(inpkt->content);
-	len = strlen(content);
+    /* Make working copy of content */
+    content = strdup(inpkt->content);
+    len = strlen(content);
 
-	/* Pull off the flag from the initial part of the content */
-	pkt->flag = 0;
-	tmpc = content;
-	while (tmpc[0] && isdigit(tmpc[0]))
-	{
-		pkt->flag = pkt->flag * 10 + (content[0] - '0');
-		tmpc++;
-	}
-	if (tmpc[0] && tmpc[0] == ',')
-	{
-		tmpc++;
+    /* Pull off the flag from the initial part of the content */
+    pkt->flag = 0;
+    tmpc = content;
+
+    while(tmpc[0] && isdigit(tmpc[0])) {
+	pkt->flag = pkt->flag * 10 + (content[0] - '0');
+	tmpc++;
+    }
+
+    if(tmpc[0] && tmpc[0] == ',') {
+	tmpc++;
+    }
+
+    /* First, count the '(' characters to size the idstatus array */
+    /* then allocate the array */
+    pkt->idstatus_count = 0;
+    for(i = 0; i < strlen(tmpc); i++) {
+	/* this will possible be incorrect for YPNS2.0 - since the
+	   message could contain a (, adjust this value later */
+	if(tmpc[i] == '(') {
+	    pkt->idstatus_count++;
 	}
+    }
 
-	/* First, count the '(' characters to size the idstatus array */
-	/* then allocate the array */
+    /* print an error if I get the was not AWAY */
+    if(strstr(tmpc, "was not AWAY")) {
 	pkt->idstatus_count = 0;
-	for (i = 0; i < strlen(tmpc); i++)
-	{
-		/* this will possible be incorrect for YPNS2.0 - since the
-		   message could contain a (, adjust this value later */
-		if (tmpc[i] == '(')
-		{
-			pkt->idstatus_count++;
-		}
+	DBG_Print(ctx, "yahoo_parsepacket_status: got a 'was not AWAY' message");
+    }
+
+    if(pkt->idstatus_count == 0) {
+	/* No entries, so no array needed */
+	pkt->idstatus = NULL;
+    } else {
+	/* Allocate the array */
+	pkt->idstatus = (struct yahoo_idstatus **) calloc(sizeof(struct yahoo_idstatus), pkt->idstatus_count);
+	for(i = 0; i < pkt->idstatus_count; i++) {
+	    pkt->idstatus[i] = (struct yahoo_idstatus *) calloc(1, sizeof(struct yahoo_idstatus));
+	}
+    }
+
+    index = 0;
+    tmp1 = NULL;
+    realcount = 0;
+
+    while(tmpc && tmpc[0] && pkt->idstatus) {
+	struct yahoo_idstatus *tmpid;
+
+	/* Get pointer to allocated structure to hold status data */
+	tmpid = pkt->idstatus[index++];
+
+	if(!tmpid) {
+	    /* shortcut, we know there can't be any more status entries
+	       at this point */
+	    /* DBG_Print("status", "null tmpid"); */
+	    break;
 	}
 
-	/* print an error if I get the was not AWAY */
-	if (strstr(tmpc, "was not AWAY"))
-	{
-		pkt->idstatus_count = 0;
-		DBG_Print(ctx, "yahoo_parsepacket_status: got a 'was not AWAY' message");
+	/* YPNS2.0 nick(status,msg,connection_id,UNK,in_pager,in_chat,in_game) */
+	/* tnneul(99,test,message^A,6AD68325,0,1,0,0) */
+	/*         0 1               2       3 4 5 6 */
+
+	/* YPNS1.0 nick(status,connection_id,UNK,in_pager,in_chat,in_game) */
+	/* nneul(0,7081F531,0,1,0,0) */
+	/*       0 2        3 4 5 6 */
+
+	/* rewrite this whole section in a less ugly fashion */
+	/* first pull off the id */
+
+	/* YUCK - YPNS2.0 has variable format status records, if type is 99,
+	   it has 7 fields, second is msg */
+
+//        DBG_Print(ctx, "whole status string = '%s'\n", DBG_NullCheck(tmpc));
+
+	if(tmp1) {
+	    tmp1 = strtok(NULL, "(");
+	} else {
+	    tmp1 = strtok(tmpc, "(");
 	}
 
-	if (pkt->idstatus_count == 0)
-	{
-		/* No entries, so no array needed */
-		pkt->idstatus = NULL;
-	}
-	else
-	{
-		/* Allocate the array */
-		pkt->idstatus = (struct yahoo_idstatus **)
-			calloc(sizeof(struct yahoo_idstatus), pkt->idstatus_count);
-
-		for (i = 0; i < pkt->idstatus_count; i++)
-		{
-			pkt->idstatus[i] = (struct yahoo_idstatus *)
-
-				calloc(1, sizeof(struct yahoo_idstatus));
-		}
+	if(tmp1 && tmp1[0] == ',') {
+	    tmp1++;
 	}
 
-	index = 0;
-	tmp1 = NULL;
-	realcount = 0;
-	while (tmpc && tmpc[0] && pkt->idstatus)
-	{
-		struct yahoo_idstatus *tmpid;
+	if(tmp1) {
+	    tmpid->id = strdup(tmp1);
+	    realcount++;
+	    contover = 0;
 
-		/* Get pointer to allocated structure to hold status data */
-		tmpid = pkt->idstatus[index++];
-		if (!tmpid)
-		{
-			/* shortcut, we know there can't be any more status entries
-			   at this point */
-			/* DBG_Print("status", "null tmpid"); */
-			break;
-		}
+	    for(i = 0; tmp1 && !contover; i++) {
+//                if(i == 6) /* end of status area */ {
+//                    tmp1 = strtok(NULL, "),");
+//                } else
+		if(i == 1) {
+		    char delim[3];
 
-		/* YPNS2.0 nick(status,msg,connection_id,UNK,in_pager,in_chat,in_game) */
-		/* tnneul(99,test,message^A,6AD68325,0,1,0,0) */
-		/*         0 1               2       3 4 5 6 */
-
-		/* YPNS1.0 nick(status,connection_id,UNK,in_pager,in_chat,in_game) */
-		/* nneul(0,7081F531,0,1,0,0) */
-		/*       0 2        3 4 5 6 */
-
-		/* rewrite this whole section in a less ugly fashion */
-		/* first pull off the id */
-
-		/* YUCK - YPNS2.0 has variable format status records, if type is 99,
-		   it has 7 fields, second is msg */
-
-#if 0
-		DBG_Print("status", "whole string = '%s'\n", DBG_NullCheck(tmpc));
-#endif
-
-		if (tmp1)
-		{
-			tmp1 = strtok(NULL, "(");
-		}
-		else
-		{
-			tmp1 = strtok(tmpc, "(");
-		}
-		if (tmp1 && tmp1[0] == ',')
-		{
-			tmp1++;
-		}
-
-		if (tmp1)
-		{
-			tmpid->id = strdup(tmp1);
-			realcount++;
-
-			for (i = 0; i <= 6 && tmp1; i++)
-			{
-#if 0
-				DBG_Print("status", "i==%d\n", i);
-#endif
-
-				if (i == 6)             /* end of status area */
-				{
-					tmp1 = strtok(NULL, "),");
-				}
-				else if (i == 1)
-				{
-					char delim[3];
-
-					if (tmpid->status == YAHOO_STATUS_CUSTOM)
-					{
-						delim[0] = 1;
-						delim[1] = ',';
-						delim[1] = 0;
-						tmp1 = strtok(NULL, delim);
-					}
-					else
-					{
-						i = 2;
-						tmp1 = strtok(NULL, ",");
-					}
-				}
-				else
-				{
-
-					tmp1 = strtok(NULL, ",");
-				}
-
-				/* then pull off the particular element of the list */
-				if (tmp1)
-				{
-					switch (i)
-					{
-						case 0: /* status */
-							tmpid->status = atoi(tmp1);
-							break;
-						case 1: /* msg */
-							if (tmpid->status == YAHOO_STATUS_CUSTOM)
-							{
-								tmpid->status_msg = strdup(tmp1);
-							}
-							break;
-						case 2: /* session id */
-							tmpid->connection_id = strdup(tmp1);
-							break;
-						case 3: /* dunno what this is */
-							break;
-						case 4:
-							tmpid->in_pager = atoi(tmp1);
-							break;
-						case 5:
-							tmpid->in_chat = atoi(tmp1);
-							break;
-						case 6:
-							tmpid->in_game = atoi(tmp1);
-							break;
-					}
-				}
+		    if(tmpid->status == YAHOO_STATUS_CUSTOM) {
+			delim[0] = 1;
+			delim[1] = ',';
+			delim[1] = 0;
+			tmp1 = strtok(NULL, delim);
+		    } else {
+			i = 2;
+			tmp1 = strtok(NULL, ",");
+		    }
+		} else {
+		    tmp1 = strtok(NULL, ",");
+		    if(strlen(tmp1)) {
+			if(tmp1[strlen(tmp1)-1] == ')') {
+			    contover = 1;
 			}
+		    }
 		}
-	}
 
-	for (i = realcount; i <= pkt->idstatus_count; i++)
-	{
-		if (pkt->idstatus && pkt->idstatus[i])
-		{
-			FREE(pkt->idstatus[i]);
+		/* then pull off the particular element of the list */
+		if(tmp1) {
+		    switch(i) {
+			case 0: /* status */
+			    tmpid->status = atoi(tmp1);
+			    break;
+			case 1: /* msg */
+			    if(tmpid->status == YAHOO_STATUS_CUSTOM) {
+				tmpid->status_msg = strdup(tmp1);
+			    }
+			    break;
+			case 2: /* session id */
+			    tmpid->connection_id = strdup(tmp1);
+			    break;
+			case 3: /* dunno what this is */
+			    break;
+			case 4:
+			    tmpid->in_pager = atoi(tmp1);
+			    break;
+			case 5:
+			    tmpid->in_chat = atoi(tmp1);
+			    break;
+			case 6:
+			    tmpid->in_game = atoi(tmp1);
+			    break;
+		    }
 		}
+	    }
 	}
-	pkt->idstatus_count = realcount;
+    }
 
-	/* Free working copy of content */
-	FREE(content);
+    for(i = realcount; i <= pkt->idstatus_count; i++) {
+	if(pkt->idstatus && pkt->idstatus[i]) {
+	    FREE(pkt->idstatus[i]);
+	}
+    }
 
-	/* Return ok for success */
-	return (0);
+    pkt->idstatus_count = realcount;
+
+    /* Free working copy of content */
+    FREE(content);
+
+    /* Return ok for success */
+    return (0);
 }
 
 int yahoo_parsepacket_message(struct yahoo_context *ctx,
@@ -3098,9 +3093,7 @@ int yahoo_getdata(struct yahoo_context *ctx)
 	/* this assumes that data is ready */
 	/* Read from the connection to the server and get any new data */
 
-	alarm(3);
 	res = read(ctx->sockfd, buf, 1000);
-	alarm(0);
 
 	if (res == -1)
 	{
@@ -3126,7 +3119,7 @@ struct yahoo_rawpacket *yahoo_getpacket(struct yahoo_context *ctx)
 {
 	struct yahoo_rawpacket *pkt;
 	struct yahoo_rawpacket *retpkt;
-	int *buflen = &ctx->io_buf_curlen;
+	int *buflen = &ctx->io_buf_curlen, i;
 	char *buffer = ctx->io_buf;
 	unsigned int contentlen;
 
@@ -3154,23 +3147,30 @@ struct yahoo_rawpacket *yahoo_getpacket(struct yahoo_context *ctx)
 
 /* print out the beginning of the buffer */
 #if 0
-	DBG_Print("Buffer (buflen = %d):\n", *buflen);
+	DBG_Print(ctx, "Buffer (buflen = %d):\n", *buflen);
+
 	for (i = 0; i < *buflen; i++)
 	{
 		if ((i) % 10 == 0)
 		{
-			DBG_Print("\n%.4d: ", i);
+			DBG_Print(ctx, "\n%.4d: ", i);
 		}
+
+		DBG_Print(ctx, "%c", buffer[i]);
+
+/*
 		if (isprint(buffer[i]))
 		{
-			DBG_Print("%-3d %c  ", buffer[i], buffer[i]);
+			DBG_Print(ctx, "%-3d %c  ", buffer[i], buffer[i]);
 		}
 		else
 		{
-			DBG_Print("%-3d    ", buffer[i]);
+			DBG_Print(ctx, "%-3d    ", buffer[i]);
 		}
+*/
 	}
-	DBG_Print("\n");
+
+	DBG_Print(ctx, "\n");
 #endif
 	/* Make pkt point to buffer for ease of use */
 	pkt = (struct yahoo_rawpacket *) buffer;
