@@ -1,7 +1,7 @@
 /*
 *
 * centericq core routines
-* $Id: centericq.cc,v 1.68 2002/02/07 17:33:55 konst Exp $
+* $Id: centericq.cc,v 1.103 2002/06/27 13:56:44 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -33,10 +33,11 @@
 #include "icqgroups.h"
 #include "accountmanager.h"
 
-centericq::centericq() {
+centericq::centericq()
+    : timer_checkmail(0), timer_update(0), timer_resend(0),
+      regmode(false)
+{
 //    timer_keypress = time(0)-50;
-    timer_checkmail = timer_update = timer_resend = 0;
-    regmode = false;
 }
 
 centericq::~centericq() {
@@ -50,8 +51,12 @@ void centericq::exec() {
     sigaction(SIGINT, &sact, 0);
     sigaction(SIGCHLD, &sact, 0);
     sigaction(SIGALRM, &sact, 0);
+    sigaction(SIGUSR1, &sact, 0);
+    sigaction(SIGUSR2, &sact, 0);
+
+    kinterface();
+    raw();
 
-    conf.initpairs();
     conf.load();
     face.init();
 
@@ -105,20 +110,21 @@ bool centericq::checkpasswords() {
     r = regmode = true;
 
     for(pname = icq; pname != protocolname_size; (int) pname += 1) {
-	ia = conf.getourid(pname);
-	if(!ia.empty()) {
-	    if(ia.password.empty()) {
-		conf.setsavepwd(false);
-
-		ia.password = face.inputstr("[" +
-		    conf.getprotocolname(pname) + "] " +
-		    _("password: "), "", '*');
-
+	if(!(gethook(pname).getcapabilities() & hoptNoPasswords)) {
+	    if(!(ia = conf.getourid(pname)).empty()) {
 		if(ia.password.empty()) {
-		    r = false;
-		    break;
-		} else {
-		    conf.setourid(ia);
+		    conf.setsavepwd(false);
+
+		    ia.password = face.inputstr("[" +
+			conf.getprotocolname(pname) + "] " +
+			_("password: "), "", '*');
+
+		    if(ia.password.empty()) {
+			r = false;
+			break;
+		    } else {
+			conf.setourid(ia);
+		    }
 		}
 	    }
 	}
@@ -236,6 +242,7 @@ void centericq::mainloop() {
 
 	    case ACT_INFO:
 		if(c->getdesc() != contactroot)
+		if(!ischannel(c))
 		    userinfo(c->getdesc());
 		break;
 
@@ -280,13 +287,18 @@ void centericq::mainloop() {
 
 	    case ACT_MSG:
 	    case 0:
-		if(c->getmsgcount()) {
-		    readevents(c->getdesc());
+		if(conf.getchatmode()) {
+		    face.chat(c->getdesc());
+
 		} else {
-		    if(c->getdesc() != contactroot)
-		    if(c->getdesc().pname != infocard) {
-			sendevent(immessage(c->getdesc(), imevent::outgoing,
-			    c->getpostponed()), icqface::ok);
+		    if(c->getmsgcount()) {
+			readevents(c->getdesc());
+		    } else {
+			if(c->getdesc() != contactroot)
+			if(c->getdesc().pname != infocard) {
+			    sendevent(immessage(c->getdesc(), imevent::outgoing,
+				c->getpostponed()), icqface::ok);
+			}
 		    }
 		}
 		break;
@@ -296,11 +308,56 @@ void centericq::mainloop() {
 
 void centericq::changestatus() {
     imstatus st;
-    protocolname pname;
+    protocolname ipname, pname;
+    bool proceed, setaway;
+    string tmp, prompt;
+    icqconf::imaccount ia;
 
     if(face.changestatus(pname, st)) {
-	gethook(pname).setstatus(st);
-	face.update();
+	if(!conf.enoughdiskspace()) {
+	    face.log("! cannot connect, free disk space is less than 10k");
+
+	} else {
+	    proceed = true;
+
+	    setaway = conf.getaskaway();
+	    if(pname != proto_all) {
+		setaway = setaway &&
+		    (gethook(pname).getcapabilities() & hoptCanSetAwayMsg);
+	    }
+
+	    if(setaway)
+	    switch(st) {
+		case away:
+		case notavail:
+		case occupied:
+		case dontdisturb:
+		    if(pname != proto_all)
+			prompt = conf.getprotocolname(pname) + ": ";
+		    prompt += _("away message");
+		    proceed = setaway = face.edit(tmp = conf.getawaymsg(pname), prompt);
+		    break;
+		default:
+		    setaway = false;
+		    break;
+	    }
+
+	    if(proceed)
+	    for(ipname = icq; ipname != protocolname_size; (int) ipname += 1) {
+		if(!(ia = conf.getourid(ipname)).empty())
+		if((pname == proto_all) || (ipname == pname)) {
+		    abstracthook &hook = gethook(ipname);
+
+		    if(setaway && !tmp.empty())
+			if(hook.getcapabilities() & hoptCanSetAwayMsg)
+			    conf.setawaymsg(ipname, tmp);
+
+		    hook.setstatus(st);
+		}
+	    }
+
+	    face.update();
+	}
     }
 }
 
@@ -328,7 +385,17 @@ void centericq::find() {
 			s.nick.erase(s.nick.size()-12);
 		    }
 
+		case irc:
+		    if(!s.nick.empty()) {
+			addcontact(imcontact(s.nick, s.pname));
+			ret = false;
+		    } else {
+			ret = face.findresults(s);
+		    }
+		    break;
+
 		case yahoo:
+		case aim:
 		    if(!s.nick.empty()) {
 			addcontact(imcontact(s.nick, s.pname));
 			ret = false;
@@ -337,6 +404,8 @@ void centericq::find() {
 
 		case infocard:
 		    c = clist.addnew(imcontact(0, infocard), true);
+		    c->setnick(s.nick);
+		    c->setdispnick(s.nick);
 
 		    if(face.updatedetails(c)) {
 			addcontact(c->getdesc());
@@ -348,6 +417,7 @@ void centericq::find() {
 			c->save();
 		    }
 
+		    face.update();
 		    ret = false;
 		    break;
 	    }
@@ -355,7 +425,7 @@ void centericq::find() {
     }
 }
 
-void centericq::userinfo(const imcontact cinfo) {
+void centericq::userinfo(const imcontact &cinfo) {
     imcontact realuin = cinfo;
     icqcontact *c = clist.get(cinfo);
 
@@ -434,8 +504,16 @@ void centericq::checkmail() {
 	    }
 
 	    if(mcount) {
-		face.log(_("+ new mail arrived, %d messages"), mcount);
+		time_t tnow;
+		const struct tm *lt;
+
+		lt = localtime(&(tnow = time(0)));
+
+		face.log(_("+ [%02d:%02d:%02d] new mail arrived, %d messages"),
+		    lt->tm_hour, lt->tm_min, lt->tm_sec, mcount);
+
 		face.log(_("+ last msg from %s"), lastfrom.c_str());
+
 		clist.get(contactroot)->playsound(imevent::email);
 	    }
 	}
@@ -451,7 +529,13 @@ void centericq::handlesignal(int signum) {
 	case SIGCHLD:
 	    while(wait3(&status, WNOHANG, 0) > 0);
 	    break;
+
+	case SIGUSR1:
+	    cicq.rereadstatus();
+	    break;
+
 	case SIGALRM:
+	case SIGUSR2:
 	    break;
     }
 }
@@ -485,7 +569,20 @@ void centericq::checkparallel() {
     }
 }
 
-void centericq::sendevent(const imevent &ev, icqface::eventviewresult r) {
+void centericq::rereadstatus() {
+    protocolname pname;
+    icqconf::imaccount ia;
+
+    for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+	ia = conf.getourid(pname);
+
+	if(!ia.empty()) {
+	    gethook(pname).setstatus(conf.getstatus(pname));
+	}
+    }
+}
+
+bool centericq::sendevent(const imevent &ev, icqface::eventviewresult r) {
     bool proceed;
     string text, fwdnote;
     imevent *sendev;
@@ -549,7 +646,7 @@ void centericq::sendevent(const imevent &ev, icqface::eventviewresult r) {
 
 		if((b.cellular.find_first_of("0123456789") == -1)
 		|| (face.getlastinputkey() == KEY_ESC))
-		    return;
+		    return false;
 
 		c->setbasicinfo(b);
 		c->save();
@@ -584,12 +681,16 @@ void centericq::sendevent(const imevent &ev, icqface::eventviewresult r) {
 	}
 
 	if(proceed) {
-	    if(face.eventedit(*sendev))
-	    if(!sendev->empty()) {
+	    if(proceed = face.eventedit(*sendev))
+	    if(proceed = !sendev->empty()) {
 		for(i = face.muins.begin(); i != face.muins.end(); i++) {
 		    sendev->setcontact(*i);
 		    em.store(*sendev);
 		}
+
+		face.muins.clear();
+		face.muins.push_back(ev.getcontact());
+
 	    }
 	}
     }
@@ -597,6 +698,8 @@ void centericq::sendevent(const imevent &ev, icqface::eventviewresult r) {
     if(sendev) {
 	delete sendev;
     }
+
+    return proceed;
 }
 
 void centericq::readevents(const imcontact &cont) {
@@ -662,8 +765,8 @@ void centericq::readevents(const imcontact &cont) {
 	    }
 
 	    while(!events.empty()) {
-		delete *events.begin();
-		events.erase(events.begin());
+		delete events.back();
+		events.pop_back();
 	    }
 	}
 
@@ -673,10 +776,11 @@ void centericq::readevents(const imcontact &cont) {
 }
 
 void centericq::history(const imcontact &cont) {
-    vector<imevent *> events;
     icqface::eventviewresult r;
     imevent *im;
     bool enough;
+    vector<imevent *> events;
+    vector<imevent *>::iterator i;
 
     events = em.getevents(cont, 0);
 
@@ -687,7 +791,9 @@ void centericq::history(const imcontact &cont) {
 	    enough = false;
 
 	    while(!enough) {
-		r = face.eventview(im);
+		vector<icqface::eventviewresult> buttons(1, icqface::next);
+
+		r = face.eventview(im, buttons);
 
 		switch(r) {
 		    case icqface::forward:
@@ -716,6 +822,16 @@ void centericq::history(const imcontact &cont) {
 			if(const imurl *m = static_cast<const imurl *>(im))
 			    conf.openurl(m->geturl());
 			break;
+
+		    case icqface::next:
+			i = ::find(events.begin(), events.end(), im);
+			enough = i == events.begin();
+
+			if(!enough) {
+			    i--;
+			    im = *i;
+			}
+			break;
 		}
 	    }
 	}
@@ -723,8 +839,8 @@ void centericq::history(const imcontact &cont) {
 	face.status("");
 
 	while(!events.empty()) {
-	    delete *events.begin();
-	    events.erase(events.begin());
+	    delete events.back();
+	    events.pop_back();
 	}
 
     } else {
@@ -732,12 +848,12 @@ void centericq::history(const imcontact &cont) {
     }
 }
 
-const string centericq::quotemsg(const string text) {
+string centericq::quotemsg(const string &text) {
     string ret;
     vector<string> lines;
     vector<string>::iterator i;
 
-    breakintolines(text, lines, WORKAREA_X2-WORKAREA_X1-4);
+    breakintolines(text, lines, 50/*face.sizeWArea.x2-face.sizeWArea.x1-4*/);
 
     for(i = lines.begin(); i != lines.end(); i++) {
 	if(!i->empty()) ret += (string) "> " + *i;
@@ -748,7 +864,7 @@ const string centericq::quotemsg(const string text) {
     return ret;
 }
 
-icqcontact *centericq::addcontact(const imcontact ic) {
+icqcontact *centericq::addcontact(const imcontact &ic) {
     icqcontact *c;
     int groupid = 0;
 
@@ -780,7 +896,6 @@ icqcontact *centericq::addcontact(const imcontact ic) {
 	c->includeintolist();
     }
 
-    hook.sendnewuser(c->getdesc());
     c->setgroupid(groupid);
     face.log(_("+ %s has been added to the list"), ic.totext().c_str());
     face.update();
@@ -788,7 +903,7 @@ icqcontact *centericq::addcontact(const imcontact ic) {
     return c;
 }
 
-bool centericq::idle(int options = 0) {
+bool centericq::idle(int options ) {
     bool keypressed, online, fin;
     fd_set rfds, wfds, efds;
     struct timeval tv;
@@ -818,7 +933,7 @@ bool centericq::idle(int options = 0) {
 	    }
 	}
 
-	tv.tv_sec = online ? PERIOD_SELECT : PERIOD_RECONNECT/3;
+	tv.tv_sec = face.updaterequested() ? 1 : 30;
 	tv.tv_usec = 0;
 
 	select(hsockfd+1, &rfds, &wfds, &efds, &tv);
@@ -846,15 +961,15 @@ void centericq::setauto(imstatus astatus) {
     protocolname pname;
     imstatus stcurrent;
     static bool autoset = false;
-    bool nautoset;
+    bool nautoset, changed = false;
 
-    if((astatus == available) && !autoset)
-	return;
+    if(astatus == available && !autoset) return;
 
     nautoset = autoset;
 
     if(autoset && (astatus == available)) {
 	face.log(_("+ the user is back"));
+	nautoset = false;
     }
 
     for(pname = icq; pname != protocolname_size; (int) pname += 1) {
@@ -874,6 +989,7 @@ void centericq::setauto(imstatus astatus) {
 
 		    hook.restorestatus();
 		    nautoset = false;
+		    changed = true;
 		} else {
 		    if(astatus == away && stcurrent == notavail)
 			break;
@@ -881,7 +997,7 @@ void centericq::setauto(imstatus astatus) {
 		    if(astatus != stcurrent)
 		    if(!autoset || (autoset && (stcurrent == away))) {
 			hook.setautostatus(astatus);
-			nautoset = true;
+			nautoset = changed = true;
 
 			face.log(_("+ [%s] automatically set %s"),
 			    conf.getprotocolname(pname).c_str(),
@@ -892,7 +1008,10 @@ void centericq::setauto(imstatus astatus) {
 	}
     }
 
-    autoset = nautoset;
+    if(changed) {
+	autoset = nautoset;
+	face.relaxedupdate();
+    }
 }
 
 #define MINCK0(x, y)       (x ? (y ? (x > y ? y : x) : x) : y)
@@ -901,24 +1020,125 @@ void centericq::exectimers() {
     time_t timer_current = time(0);
     protocolname pname;
     int paway, pna;
+    bool fonline = false;
+
+    /*
+    *
+    * Execute regular actions for all the hooks.
+    *
+    */
 
     for(pname = icq; pname != protocolname_size; (int) pname += 1) {
 	if(!conf.getourid(pname).empty()) {
-	    gethook(pname).exectimers();
+	    abstracthook &hook = gethook(pname);
+
+	    /*
+	    *
+	    * A hook's own action goes first.
+	    *
+	    */
+
+	    hook.exectimers();
+
+	    static map<protocolname, reconnectInfo> reconnect;
+
+	    if(!hook.logged()) {
+		/*
+		*
+		* Any need to try auto re-connecting?
+		*
+		*/
+
+		if(timer_current-reconnect[pname].timer > reconnect[pname].period) {
+		    time(&reconnect[pname].timer);
+		    reconnect[pname].period += reconnect[pname].period/2;
+
+		    if(hook.online()) {
+			hook.disconnect();
+
+		    } else if(conf.getstatus(pname) != offline) {
+			if(conf.enoughdiskspace() && !manager.isopen()) {
+			    hook.connect();
+			}
+
+		    }
+		}
+	    } else {
+		fonline = true;
+		reconnect[pname] = reconnectInfo();
+	    }
 	}
     }
 
-    conf.getauto(paway, pna);
+    /*
+    *
+    * How let's find out how are the auto-away mode is doing.
+    *
+    */
 
-    if(paway || pna) {
-	if(paway && (timer_current-timer_keypress > paway*60))
-	    setauto(away);
+    if(fonline) {
+	conf.getauto(paway, pna);
 
-	if(pna && (timer_current-timer_keypress > pna*60))
-	    setauto(notavail);
+	if(paway || pna) {
+	    imstatus toset = offline;
+	    static map<imstatus, bool> autostat;
 
-	if(timer_current-timer_keypress < MINCK0(paway, pna)*60)
-	    setauto(available);
+	    if(autostat.empty()) {
+		autostat[available] = false;
+		autostat[notavail] = false;
+		autostat[away] = false;
+	    }
+
+	    if(paway && timer_current-timer_keypress > paway*60)
+		toset = away;
+
+	    if(pna && timer_current-timer_keypress > pna*60)
+		toset = notavail;
+
+	    if(timer_current-timer_keypress < MINCK0(paway, pna)*60)
+		toset = available;
+
+	    if(toset != offline && !autostat[toset]) {
+		setauto(toset);
+
+		if(toset == available) {
+		    autostat[away] = autostat[notavail] = false;
+		} else {
+		    autostat[toset] = true;
+		}
+	    }
+	}
+    }
+
+    if(timer_current-timer_resend > PERIOD_RESEND) {
+	/*
+	*
+	* We'll check for free disk space here too.
+	*
+	*/
+
+	conf.checkdiskspace();
+
+	if(!conf.enoughdiskspace()) {
+	    if(fonline) {
+		for(pname = icq; pname != protocolname_size; (int) pname += 1)
+		    gethook(pname).disconnect();
+
+		face.log(_("! free disk space is less than 10k, going offline"));
+		face.log(_("! otherwise we can lose events and configuration"));
+	    }
+	} else {
+	    /*
+	    *
+	    * Re-send the idle IM events.
+	    *
+	    */
+
+	    em.resend();
+	    face.relaxedupdate();
+	}
+
+	time(&timer_resend);
     }
 
     if(timer_current-timer_checkmail > PERIOD_CHECKMAIL) {
@@ -926,12 +1146,6 @@ void centericq::exectimers() {
 	time(&timer_checkmail);
     }
 
-    if(timer_current-timer_resend > PERIOD_RESEND) {
-	em.resend();
-	face.relaxedupdate();
-	time(&timer_resend);
-    }
-
     if(face.updaterequested())
     if(timer_current-timer_update > PERIOD_DISPUPDATE) {
 	face.update();
@@ -941,7 +1155,53 @@ void centericq::exectimers() {
 
 // ----------------------------------------------------------------------------
 
-const string rusconv(const string tdir, const string text) {
+string rushtmlconv(const string &tdir, const string &text) {
+    int pos;
+    string r = rusconv(tdir, text);
+
+    if(tdir == "kw") {
+	pos = 0;
+	while((pos = r.find_first_of("&<>", pos)) != -1) {
+	    switch(r[pos]) {
+		case '&':
+		    if(r.substr(pos, 4) != "&lt;"
+		    && r.substr(pos, 4) != "&gt;")
+			r.replace(pos, 1, "&amp;");
+		    break;
+		case '<': r.replace(pos, 1, "&lt;"); break;
+		case '>': r.replace(pos, 1, "&gt;"); break;
+	    }
+	    pos++;
+	}
+
+    } else if(tdir == "wk") {
+	pos = 0;
+	while((pos = r.find("&", pos)) != -1) {
+	    if(r.substr(pos+1, 4) == "amp;") r.replace(pos, 5, "&"); else
+	    if(r.substr(pos+1, 3) == "lt;") r.replace(pos, 4, "<"); else
+	    if(r.substr(pos+1, 3) == "gt;") r.replace(pos, 4, ">"); 
+	    pos++;
+	}
+    }
+
+    return r;
+}
+
+string ruscrlfconv(const string &tdir, const string &text) {
+    string r = rusconv(tdir, text);
+    int pos;
+
+    for(pos = 0; (pos = r.find("\r")) != -1; pos++) {
+	r.erase(pos, 1);
+    }
+
+    return r;
+}
+
+string rusconv(const string &tdir, const string &text) {
+    string r;
+
+#ifndef HAVE_ICONV_H
     static unsigned char kw[] = {
 	128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,
 	144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,
@@ -964,7 +1224,6 @@ const string rusconv(const string tdir, const string text) {
 	210,211,212,213,198,200,195,222,219,221,223,217,216,220,192,209
     };
 
-    string r;
     unsigned char c;
     string::const_iterator i;
     unsigned char *table = 0;
@@ -982,6 +1241,25 @@ const string rusconv(const string tdir, const string text) {
     } else {
 	r = text;
     }
+#else
+
+    if(conf.getrussian()) {
+	if(tdir == "kw") r = siconv(text, "koi8-r", "cp1251"); else
+	if(tdir == "wk") r = siconv(text, "cp1251", "koi8-r"); else
+	    r = text;
+    } else {
+	r = text;
+    }
+
+#endif
 
     return r;
 }
+
+bool ischannel(const imcontact &cont) {
+    return (cont.nickname.substr(0, 1) == "#") && (cont.pname == irc);
+}
+
+bool ischannel(const icqcontact *c) {
+    return c ? ischannel(c->getdesc()) : false;
+}
