@@ -23,6 +23,13 @@ class centericq {
 	};
 
     protected:
+	struct reconnectInfo {
+	    time_t timer;
+	    int period;
+
+	    reconnectInfo(): timer(0), period(40) {}
+	};
+
 	time_t timer_checkmail, timer_keypress, timer_update, timer_resend;
 	bool regmode;
 
@@ -31,8 +38,9 @@ class centericq {
 	void checkparallel();
 	void inithooks();
 	bool checkpasswords();
+	void rereadstatus();
 
-	const string quotemsg(const string text);
+	string quotemsg(const string &text);
 
 	void setauto(imstatus astatus);
 
@@ -44,14 +52,14 @@ class centericq {
 	void exec();
 	void reg();
 	void mainloop();
-	void userinfo(const imcontact cinfo);
+	void userinfo(const imcontact &cinfo);
 	void changestatus();
 	bool updateconf();
 
 	void find();
 	void checkmail();
 
-	icqcontact *addcontact(const imcontact ic);
+	icqcontact *addcontact(const imcontact &ic);
 
 	bool idle(int options = 0);
 	void exectimers();
@@ -59,11 +67,16 @@ class centericq {
 	void readevents(const imcontact &cont);
 	void history(const imcontact &cont);
 
-	void sendevent(const imevent &ev, icqface::eventviewresult r);
+	bool sendevent(const imevent &ev, icqface::eventviewresult r);
 };
 
 extern centericq cicq;
 
-const string rusconv(const string tdir, const string text);
+string rusconv(const string &tdir, const string &text);
+string rushtmlconv(const string &tdir, const string &text);
+string ruscrlfconv(const string &tdir, const string &text);
+
+bool ischannel(const imcontact &cont);
+bool ischannel(const icqcontact *c);
 
 #endif
