@@ -5,6 +5,7 @@
 #include "icqcontacts.h"
 #include "abstracthook.h"
 #include "imlogger.h"
+#include "imexternal.h"
 
 imeventmanager em;
 
@@ -21,6 +22,8 @@ void imeventmanager::store(const imevent &ev) {
 
     if(ev.getdirection() == imevent::incoming) {
 	if(!lst.inlist(ev.getcontact(), csignore)) {
+	    logger.putevent(ev);
+
 	    c = clist.get(ev.getcontact());
 
 	    if(!c) {
@@ -33,8 +36,12 @@ void imeventmanager::store(const imevent &ev) {
 
 	    if(c) {
 		eventwrite(ev, history);
+
 		c->setmsgcount(c->getmsgcount()+1);
 		c->playsound(ev.gettype());
+
+		external.exec(ev);
+
 		face.relaxedupdate();
 	    }
 	}
@@ -44,6 +51,7 @@ void imeventmanager::store(const imevent &ev) {
 	if(hook.logged()) {
 	    if(hook.send(ev)) {
 		eventwrite(ev, history);
+		logger.putevent(ev);
 	    } else {
 		eventwrite(ev, offline);
 	    }
@@ -51,8 +59,6 @@ void imeventmanager::store(const imevent &ev) {
 	    eventwrite(ev, offline);
 	}
     }
-
-    logger.putevent(ev);
 }
 
 // if incoming, store to the history immediately
@@ -70,6 +76,10 @@ vector<imevent *> imeventmanager::getevents(const imcontact &cont, time_t lastre
 	fhist.open((c->getdirname() + "history").c_str());
 
 	if(fhist.is_open()) {
+	    if(lastread) {
+		fhist.seekg(c->gethistoffset(), ios::beg);
+	    }
+
 	    while(!fhist.eof()) {
 		rev = eventread(fhist);
 
@@ -93,7 +103,7 @@ vector<imevent *> imeventmanager::getevents(const imcontact &cont, time_t lastre
 
 void imeventmanager::eventwrite(const imevent &ev, eventwritemode mode) {
     ofstream fhist;
-    string fname;
+    string fname, lockfname;
     icqcontact *c;
 
     if(c = clist.get(ev.getcontact())) {
@@ -108,35 +118,21 @@ void imeventmanager::eventwrite(const imevent &ev, eventwritemode mode) {
 		break;
 	}
 
+	setlock(fname);
+
 	fhist.open(fname.c_str(), ios::app);
 
 	if(fhist.is_open()) {
-	    if(ev.gettype() == imevent::message) {
-		const immessage *m = static_cast<const immessage *>(&ev);
-		m->write(fhist);
-	    } else if(ev.gettype() == imevent::url) {
-		const imurl *m = static_cast<const imurl *>(&ev);
-		m->write(fhist);
-	    } else if(ev.gettype() == imevent::sms) {
-		const imsms *m = static_cast<const imsms *>(&ev);
-		m->write(fhist);
-	    } else if(ev.gettype() == imevent::authorization) {
-		const imauthorization *m = static_cast<const imauthorization *>(&ev);
-		m->write(fhist);
-	    } else if(ev.gettype() == imevent::email) {
-		const imemail *m = static_cast<const imemail *>(&ev);
-		m->write(fhist);
-	    }
-
+	    ev.write(fhist);
 	    fhist.close();
 	}
+
+	releaselock(fname);
     }
 }
 
 imevent *imeventmanager::eventread(ifstream &f) const {
-    imevent *rev, ev;
-
-    ev.read(f);
+    imevent *rev, ev(f);
 
     if(rev = ev.getevent()) {
 	rev->read(f);
@@ -192,3 +188,22 @@ void imeventmanager::resend() {
 int imeventmanager::getunsentcount() const {
     return unsent;
 }
+
+void imeventmanager::setlock(const string &fname) const {
+    string lockfname = fname + ".lock";
+    int ntries = 0;
+    ofstream f;
+
+    while(!access(lockfname.c_str(), F_OK) && (ntries < 10)) {
+	usleep(10);
+	ntries++;
+    }
+
+    f.open(lockfname.c_str());
+    if(f.is_open()) f.close();
+}
+
+void imeventmanager::releaselock(const string &fname) const {
+    string lockfname = fname + ".lock";
+    unlink(lockfname.c_str());
+}
