@@ -1,7 +1,7 @@
 /*
 *
 * centericq icq protocol handling class
-* $Id: icqhook.cc,v 1.56 2002/02/13 17:45:43 konst Exp $
+* $Id: icqhook.cc,v 1.86 2002/07/03 15:31:05 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -25,13 +25,11 @@
 #include "icqhook.h"
 #include "icqconf.h"
 #include "icqface.h"
-#include "icqcontacts.h"
 #include "accountmanager.h"
 #include "centericq.h"
 #include "imlogger.h"
 
-#include "libicq2000/userinfoconstants.h"
-#include "libicq2000/userinfohelpers.h"
+#include <libicq2000/userinfohelpers.h>
 
 #define PERIOD_ICQPOLL  5
 
@@ -49,10 +47,17 @@ static const Status stat2int[imstatus_size] = {
 };
 
 icqhook::icqhook() {
-    timer_reconnect = 0;
     fonline = false;
 
-    fcapabilities = hoptCanSendURL | hoptCanSendSMS | hoptCanSetAwayMsg;
+    fcapabilities =
+	hoptCanSendURL |
+	hoptCanSendSMS |
+	hoptCanSetAwayMsg |
+	hoptCanFetchAwayMsg |
+	hoptCanChangeNick |
+	hoptCanUpdateDetails |
+	hoptChangableServer |
+	hoptControlableVisibility;
 
     cli.connected.connect(slot(this, &icqhook::connected_cb));
     cli.disconnected.connect(slot(this, &icqhook::disconnected_cb));
@@ -60,11 +65,15 @@ icqhook::icqhook() {
     cli.messaged.connect(slot(this, &icqhook::messaged_cb));
     cli.messageack.connect(slot(this, &icqhook::messageack_cb));
     cli.contactlist.connect(slot(this, &icqhook::contactlist_cb));
+    cli.contact_userinfo_change_signal.connect(slot(this, &icqhook::contact_userinfo_change_signal_cb));
+    cli.contact_status_change_signal.connect(slot(this, &icqhook::contact_status_change_signal_cb));
     cli.newuin.connect(slot(this, &icqhook::newuin_cb));
     cli.rate.connect(slot(this, &icqhook::rate_cb));
     cli.want_auto_resp.connect(slot(this, &icqhook::want_auto_resp_cb));
     cli.search_result.connect(slot(this, &icqhook::search_result_cb));
-    cli.self_event.connect(slot(this, &icqhook::self_event_cb));
+    cli.server_based_contact_list.connect(slot(this, &icqhook::server_based_contact_list_cb));
+    cli.self_contact_userinfo_change_signal.connect(slot(this, &icqhook::self_contact_userinfo_change_cb));
+    cli.self_contact_status_change_signal.connect(slot(this, &icqhook::self_contact_status_change_cb));
 
 #ifdef DEBUG
     cli.logger.connect(slot(this, &icqhook::logger_cb));
@@ -79,11 +88,15 @@ icqhook::~icqhook() {
     cli.messaged.clear();
     cli.messageack.clear();
     cli.contactlist.clear();
+    cli.contact_userinfo_change_signal.clear();
+    cli.contact_status_change_signal.clear();
     cli.newuin.clear();
     cli.rate.clear();
-    cli.self_event.clear();
     cli.search_result.clear();
     cli.want_auto_resp.clear();
+    cli.self_contact_userinfo_change_signal.clear();
+    cli.self_contact_status_change_signal.clear();
+    cli.server_based_contact_list.clear();
 }
 
 void icqhook::init() {
@@ -99,30 +112,33 @@ void icqhook::connect() {
 	c = (icqcontact *) clist.at(i);
 
 	if(c->getdesc().pname == icq) {
-	    Contact ic(c->getdesc().uin);
-	    cli.addContact(ic);
+	    cli.addContact(new Contact(c->getdesc().uin));
 	}
     }
 
     if(!acc.server.empty()) cli.setLoginServerHost(acc.server);
     if(acc.port) cli.setLoginServerPort(acc.port);
 
-    cli.setUIN(acc.uin);
-    cli.setPassword(acc.password);
+    if(!conf.getsmtphost().empty()) cli.setSMTPServerHost(conf.getsmtphost());
+    if(conf.getsmtpport()) cli.setSMTPServerPort(conf.getsmtpport());
 
     face.log(_("+ [icq] connecting to the server"));
 
+    cli.self_contact_userinfo_change_signal.clear();
+
+    cli.setUIN(acc.uin);
+    cli.setPassword(acc.password);
+
     sendinvisible();
     cli.setStatus(stat2int[manualstatus], manualstatus == invisible);
-    logger.putourstatus(icq, offline, manualstatus);
 
-    time(&timer_reconnect);
+    cli.self_contact_userinfo_change_signal.connect(slot(this, &icqhook::self_contact_userinfo_change_cb));
+
     fonline = true;
     flogged = false;
 }
 
 void icqhook::disconnect() {
-    logger.putourstatus(icq, getstatus(), offline);
     cli.setStatus(STATUS_OFFLINE);
 }
 
@@ -143,26 +159,45 @@ void icqhook::resolve() {
 }
 
 void icqhook::sendinvisible() {
+    vector<visInfo> todo, nvislist;
+    vector<visInfo>::iterator iv, it;
     icqlist::iterator i;
-    Contact *c;
-    int k;
-/*
-    for(k = 0; k < clist.count; k++) {
-	if(c = cli.getContact(((icqcontact *) clist.at(k))->getdesc().uin)) {
-	    c->setInvisible(false);
+
+    for(i = lst.begin(); i != lst.end(); i++) {
+	if(i->getdesc().pname == icq) {
+	    switch(i->getstatus()) {
+		case csvisible:
+		case csinvisible:
+		    todo.push_back(pair<unsigned int, contactstatus>
+			(i->getdesc().uin, i->getstatus()));
+		    break;
+	    }
 	}
     }
 
-    for(i = lst.begin(); i != lst.end(); i++) {
-	switch(i->getstatus()) {
-	    case csinvisible:
-		if(c = cli.getContact(i->getdesc().uin)) {
-		    c->setInvisible(true);
-		}
-		break;
+    nvislist = todo;
+
+    for(iv = vislist.begin(); iv != vislist.end(); iv++) {
+	it = find(todo.begin(), todo.end(), *iv);
+
+	if(it != todo.end()) {
+	    todo.erase(it);
+	} else {
+	    switch(iv->second) {
+		case csvisible: cli.removeVisible(iv->first); break;
+		case csinvisible: cli.removeInvisible(iv->first); break;
+	    }
 	}
     }
-*/
+
+    for(it = todo.begin(); it != todo.end(); it++) {
+	switch(it->second) {
+	    case csvisible: cli.addVisible(new Contact(it->first)); break;
+	    case csinvisible: cli.addInvisible(new Contact(it->first)); break;
+	}
+    }
+
+    vislist = nvislist;
 }
 
 void icqhook::exectimers() {
@@ -173,19 +208,6 @@ void icqhook::exectimers() {
 	    cli.Poll();
 	    sendinvisible();
 	    time(&timer_poll);
-	} else if(tcurrent-timer_resolve > PERIOD_RESOLVE) {
-	    resolve();
-	    time(&timer_resolve);
-	}
-    } else {
-	if(tcurrent-timer_reconnect > PERIOD_RECONNECT) {
-	    if(online() && !logged()) {
-		disconnect();
-	    } else if(manualstatus != offline) {
-		if(!manager.isopen()) {
-		    connect();
-		}
-	    }
 	}
     }
 }
@@ -297,52 +319,70 @@ bool icqhook::enabled() const {
 
 bool icqhook::send(const imevent &ev) {
     unsigned int uin = ev.getcontact().uin;
-    Contact *ic = cli.getContact(uin);
+    ContactRef ic = cli.getContact(uin);
+    MessageEvent *sev = 0;
+    ICQMessageEvent *iev;
 
-    if(!ic) {
+    if(!ic.get()) {
 	return false;
     }
 
     if(ev.gettype() == imevent::message) {
 	const immessage *m = static_cast<const immessage *> (&ev);
-	cli.SendEvent(new NormalMessageEvent(ic, rusconv("kw", m->gettext())));
+	sev = new NormalMessageEvent(ic, ruscrlfconv("kw", m->gettext()));
 
     } else if(ev.gettype() == imevent::url) {
 	const imurl *m = static_cast<const imurl *> (&ev);
-	cli.SendEvent(new URLMessageEvent(ic,
-	    rusconv("kw", m->getdescription()),
-	    rusconv("kw", m->geturl())));
+	sev = new URLMessageEvent(ic,
+	    ruscrlfconv("kw", m->getdescription()),
+	    ruscrlfconv("kw", m->geturl()));
 
     } else if(ev.gettype() == imevent::sms) {
 	const imsms *m = static_cast<const imsms *> (&ev);
-	ic->setMobileNo(clist.get(ev.getcontact())->getbasicinfo().cellular);
-	cli.SendEvent(new SMSMessageEvent(ic, rusconv("kw", m->gettext()), false));
+
+	if(!uin) {
+	    ic = cli.getSelfContact();
+	    if(ic->getMainHomeInfo().getMobileNo().empty()) {
+		cli.fetchSelfDetailContactInfo();
+		return false;
+	    }
+
+	} else {
+	    ic->setMobileNo(clist.get(ev.getcontact())->getbasicinfo().cellular);
+	}
+
+	sev = new SMSMessageEvent(ic, ruscrlfconv("kw", m->gettext()), true);
 
     } else if(ev.gettype() == imevent::authorization) {
 	const imauthorization *m = static_cast<const imauthorization *> (&ev);
-	cli.SendEvent(new AuthAckEvent(ic,
-	    rusconv("kw", m->gettext()), m->getgranted()));
-
-    } else if(ev.gettype() == imevent::sms) {
-	const imsms *m = static_cast<const imsms *> (&ev);
-	cli.SendEvent(new SMSMessageEvent(ic, rusconv("kw", m->gettext()),
-	    true));
+	sev = new AuthAckEvent(ic, ruscrlfconv("kw", m->gettext()), m->getgranted());
 
     } else {
 	return false;
     }
 
+    if(sev) {
+	if(ic->getStatus() == STATUS_DND || ic->getStatus() == STATUS_OCCUPIED)
+	if(iev = dynamic_cast<ICQMessageEvent *>(sev))
+	    iev->setUrgent(true);
+
+	cli.SendEvent(sev);
+    }
+
     return true;
 }
 
-void icqhook::sendnewuser(const imcontact c) {
+void icqhook::sendnewuser(const imcontact &c) {
     if(logged()) {
-	Contact ic(c.uin);
-	cli.addContact(ic);
+	cli.addContact(new Contact(c.uin));
 	cli.fetchDetailContactInfo(cli.getContact(c.uin));
     }
 }
 
+void icqhook::removeuser(const imcontact &c) {
+    cli.removeContact(c.uin);
+}
+
 void icqhook::setautostatus(imstatus st) {
     if(st != offline) {
 	if(getstatus() == offline) {
@@ -369,7 +409,7 @@ imstatus icqhook::getstatus() const {
     }
 }
 
-bool icqhook::regconnect(const string aserv) {
+bool icqhook::regconnect(const string &aserv) {
     int pos;
 
     if((pos = aserv.find(":")) != -1) {
@@ -380,31 +420,32 @@ bool icqhook::regconnect(const string aserv) {
     return true;
 }
 
-bool icqhook::regattempt(unsigned int &auin, const string apassword) {
-    fd_set rfds, wfds, efds;
+bool icqhook::regattempt(unsigned int &auin, const string &apassword) {
+    fd_set srfds, swfds, sefds;
     struct timeval tv;
     int hsockfd;
     time_t regtimeout = time(0);
 
     reguin = 0;
 
-    FD_ZERO(&rfds);
-    FD_ZERO(&wfds);
-    FD_ZERO(&efds);
-
     cli.setPassword(apassword);
     cli.RegisterUIN();
 
     while(!reguin && (time(0)-regtimeout < 60)) {
 	hsockfd = 0;
-	getsockets(rfds, wfds, efds, hsockfd);
+
+	FD_ZERO(&srfds);
+	FD_ZERO(&swfds);
+	FD_ZERO(&sefds);
+
+	getsockets(srfds, swfds, sefds, hsockfd);
 
 	tv.tv_sec = 30;
 	tv.tv_usec = 0;
 
-	select(hsockfd+1, &rfds, &wfds, &efds, &tv);
+	select(hsockfd+1, &srfds, &swfds, &sefds, &tv);
 
-	if(isoursocket(rfds, wfds, efds)) {
+	if(isoursocket(srfds, swfds, sefds)) {
 	    main();
 	}
     }
@@ -433,17 +474,16 @@ imstatus icqhook::icq2imstatus(const Status st) const {
     }
 }
 
-void icqhook::requestinfo(const imcontact c) {
+void icqhook::requestinfo(const imcontact &c) {
     if(logged()) {
 	if(c == imcontact(conf.getourid(icq).uin, icq)) {
 	    // Our info is requested
 	    cli.fetchSelfDetailContactInfo();
 	} else {
-	    Contact *icont = cli.getContact(c.uin);
+	    ContactRef icont = cli.getContact(c.uin);
 
-	    if(!icont) {
-		Contact ic(c.uin);
-		cli.addContact(ic);
+	    if(!icont.get()) {
+		cli.addContact(new Contact(c.uin));
 	    } else {
 		cli.fetchDetailContactInfo(icont);
 	    }
@@ -454,6 +494,11 @@ void icqhook::requestinfo(const imcontact c) {
 void icqhook::lookup(const imsearchparams &params, verticalmenu &dest) {
     ICQ2000::Sex sex;
 
+    while(!foundguys.empty()) {
+	delete foundguys.back();
+	foundguys.pop_back();
+    }
+
     searchdest = &dest;
 
     sex = params.gender == genderMale ? ICQ2000::SEX_MALE :
@@ -478,11 +523,13 @@ void icqhook::lookup(const imsearchparams &params, verticalmenu &dest) {
 }
 
 void icqhook::sendupdateuserinfo(const icqcontact &c) {
-    Contact *ic = cli.getSelfContact();
+    ContactRef ic = cli.getSelfContact();
 
-    MainHomeInfo &home = ic->getMainHomeInfo();
-    HomepageInfo &hpage = ic->getHomepageInfo();
-    WorkInfo &work = ic->getWorkInfo();
+    cli.self_contact_userinfo_change_signal.clear();
+
+    Contact::MainHomeInfo &home = ic->getMainHomeInfo();
+    Contact::HomepageInfo &hpage = ic->getHomepageInfo();
+    Contact::WorkInfo &work = ic->getWorkInfo();
 
     icqcontact::basicinfo cbinfo = c.getbasicinfo();
     icqcontact::moreinfo cminfo = c.getmoreinfo();
@@ -545,25 +592,25 @@ void icqhook::sendupdateuserinfo(const icqcontact &c) {
     ic->setWorkInfo(work);
 
     cli.uploadSelfDetails();
+    cli.self_contact_userinfo_change_signal.connect(slot(this, &icqhook::self_contact_userinfo_change_cb));
 }
 
 void icqhook::requestawaymsg(const imcontact &c) {
-    Contact *ic = cli.getContact(c.uin);
+    ContactRef ic = cli.getContact(c.uin);
 
-    if(ic) {
-	MessageEvent *ev = new AwayMessageEvent(ic);
-	cli.SendEvent(ev);
+    if(ic.get()) {
+	cli.SendEvent(new AwayMessageEvent(ic));
     }
 }
 
-void icqhook::updateinforecord(Contact *ic, icqcontact *c) {
-    if(ic && c) {
-	MainHomeInfo &home = ic->getMainHomeInfo();
-	HomepageInfo &hpage = ic->getHomepageInfo();
-	WorkInfo &work = ic->getWorkInfo();
-	PersonalInterestInfo &pint = ic->getPersonalInterestInfo();
-	BackgroundInfo &backg = ic->getBackgroundInfo();
-	EmailInfo &email = ic->getEmailInfo();
+void icqhook::updateinforecord(ContactRef ic, icqcontact *c) {
+    if(ic.get() && c) {
+	Contact::MainHomeInfo &home = ic->getMainHomeInfo();
+	Contact::HomepageInfo &hpage = ic->getHomepageInfo();
+	Contact::WorkInfo &work = ic->getWorkInfo();
+	Contact::PersonalInterestInfo &pint = ic->getPersonalInterestInfo();
+	Contact::BackgroundInfo &backg = ic->getBackgroundInfo();
+	Contact::EmailInfo &email = ic->getEmailInfo();
 
 	icqcontact::basicinfo cbinfo = c->getbasicinfo();
 	icqcontact::moreinfo cminfo = c->getmoreinfo();
@@ -624,16 +671,10 @@ void icqhook::updateinforecord(Contact *ic, icqcontact *c) {
 
 	string sbuf;
 	vector<string> pintinfo;
-	list<PersonalInterestInfo::Interest>::iterator ii;
+	list<Contact::PersonalInterestInfo::Interest>::iterator ii;
 
 	for(ii = pint.interests.begin(); ii != pint.interests.end(); ii++) {
-	    int k = ii->first-Interests_offset;
-
-	    if(k >= 0 && k < Interests_table_size) {
-		sbuf = (string) Interests_table[k];
-	    } else {
-		sbuf = "";
-	    }
+	    sbuf = UserInfoHelpers::getInterestsIDtoString(ii->first);
 
 	    if(!ii->second.empty()) {
 		if(!sbuf.empty()) sbuf += ": ";
@@ -646,18 +687,11 @@ void icqhook::updateinforecord(Contact *ic, icqcontact *c) {
 	/* education background */
 
 	vector<string> backginfo;
-	list<BackgroundInfo::School>::iterator isc;
+	list<Contact::BackgroundInfo::School>::iterator isc;
 
 	for(isc = backg.schools.begin(); isc != backg.schools.end(); isc++) {
-	    sbuf = "";
-
-	    for(int k = 0; k < Background_table_size; k++) {
-		if(Background_table[k].code == isc->first) {
-		    sbuf = (string) Background_table[k].name + ": ";
-		    break;
-		}
-	    }
-
+	    sbuf = UserInfoHelpers::getBackgroundIDtoString(isc->first);
+	    if(!sbuf.empty()) sbuf += ": ";
 	    sbuf += rusconv("wk", isc->second);
 	    if(!sbuf.empty()) backginfo.push_back(sbuf);
 	}
@@ -689,7 +723,7 @@ void icqhook::connected_cb(ConnectedEvent *ev) {
     flogged = true;
 
     time(&timer_poll);
-    timer_resolve = time(0)-PERIOD_RESOLVE+2;
+    logger.putourstatus(icq, offline, manualstatus);
 
     face.log(_("+ [icq] logged in"));
     face.update();
@@ -709,6 +743,9 @@ void icqhook::connected_cb(ConnectedEvent *ev) {
 
 	unlink(conf.getconfigfname("icq-infoset").c_str());
     }
+
+    resolve();
+//    cli.fetchServerBasedContactList();
 }
 
 void icqhook::disconnected_cb(DisconnectedEvent *ev) {
@@ -729,20 +766,26 @@ void icqhook::disconnected_cb(DisconnectedEvent *ev) {
     wfds.clear();
     efds.clear();
 
+    logger.putourstatus(icq, getstatus(), offline);
+
     clist.setoffline(icq);
     fonline = false;
 
     msg = _("+ [icq] disconnected");
+
     if(!reasons[ev->getReason()].empty()) {
 	msg += ", ";
 	msg += reasons[ev->getReason()];
+
+	logger.putmessage((string) _("icq disconnection reason") + ": " +
+	    reasons[ev->getReason()]);
     }
 
     face.log(msg);
     face.update();
 }
 
-bool icqhook::messaged_cb(MessageEvent *ev) {
+void icqhook::messaged_cb(MessageEvent *ev) {
     imcontact ic;
     time_t t;
 
@@ -786,17 +829,6 @@ bool icqhook::messaged_cb(MessageEvent *ev) {
 	    }
 	}
 
-    } else if(ev->getType() == MessageEvent::SMS_Receipt) {
-	SMSReceiptEvent *r;
-	if(r = dynamic_cast<SMSReceiptEvent *>(ev)) {
-	    face.log("sms receipt %s", r->delivered() ? "delivered" : "failed");
-	    face.log(r->getMessage());
-	    face.log(r->getMessageId());
-	    face.log(r->getDestination());
-	    face.log(r->getSubmissionTime());
-	    face.log(r->getDeliveryTime());
-	}
-
     } else if(ev->getType() == MessageEvent::AuthReq) {
 	AuthReqEvent *r;
 	if(r = dynamic_cast<AuthReqEvent *>(ev)) {
@@ -804,7 +836,6 @@ bool icqhook::messaged_cb(MessageEvent *ev) {
 		rusconv("wk", r->getMessage())));
 	}
 
-    } else if(ev->getType() == MessageEvent::AuthAck) {
     } else if(ev->getType() == MessageEvent::EmailEx) {
 	EmailExEvent *r;
 
@@ -848,16 +879,22 @@ bool icqhook::messaged_cb(MessageEvent *ev) {
 
     }
 
-    return true;
+    ev->setDelivered(true);
 }
 
 void icqhook::messageack_cb(MessageEvent *ev) {
     imcontact ic;
     icqcontact *c;
+    ICQMessageEvent *me;
 
     ic = imcontact(ev->getContact()->getUIN(), icq);
     c = clist.get(ic);
 
+    if(!c) {
+	c = clist.get(contactroot);
+	c->setdispnick(cli.getSelfContact()->getAlias());
+    }
+
     if(ev->isFinished()) {
 	/*
 	*
@@ -880,7 +917,7 @@ void icqhook::messageack_cb(MessageEvent *ev) {
 		    if(r = dynamic_cast<AwayMessageEvent *>(ev)) {
 			em.store(immessage(ic, imevent::incoming,
 			    string() + _("* Away message:") + "\n\n" +
-			    rusconv("wk", r->getMessage())));
+			    rusconv("wk", r->getAwayMessage())));
 		    }
 		} else {
 		    face.log(_("+ [icq] cannot fetch away msg from %s, %s"),
@@ -898,66 +935,62 @@ void icqhook::messageack_cb(MessageEvent *ev) {
 }
 
 void icqhook::contactlist_cb(ContactListEvent *ev) {
+}
+
+void icqhook::contact_status_change_signal_cb(StatusChangeEvent *ev) {
     icqcontact *c = clist.get(imcontact(ev->getUIN(), icq));
-    char buf[64];
-    string lastip, sbuf;
-    int ip;
 
-    switch(ev->getType()) {
-	case ContactListEvent::StatusChange:
-	    if(dynamic_cast<StatusChangeEvent*>(ev))
-	    if(c) {
-		Contact *ic = cli.getContact(ev->getUIN());
-		StatusChangeEvent *tev = dynamic_cast<StatusChangeEvent*>(ev);
+    if(c) {
+	ContactRef ic = cli.getContact(ev->getUIN());
+	StatusChangeEvent *tev = dynamic_cast<StatusChangeEvent*>(ev);
+	imstatus nst = icq2imstatus(tev->getStatus());
 
-		logger.putonline(c->getdesc(),
-		    c->getstatus(),
-		    icq2imstatus(tev->getStatus()));
+	if(ic->isInvisible()) nst = invisible;
 
-		c->setstatus(icq2imstatus(tev->getStatus()));
+	logger.putonline(c->getdesc(), c->getstatus(), nst);
+	c->setstatus(nst);
 
-		if(c->getstatus() != offline) {
-		    c->setlastseen();
+	if(c->getstatus() != offline)
+	    c->setlastseen();
+    }
+}
 
-		    if(inet_ntop(AF_INET, &(ip = ntohl(ic->getExtIP())), buf, 64)) {
-			lastip = buf;
-		    }
+void icqhook::contact_userinfo_change_signal_cb(UserInfoChangeEvent *ev) {
+    icqcontact *c = clist.get(imcontact(ev->getUIN(), icq));
+    ContactRef ic = cli.getContact(ev->getUIN());
 
-		    if(lastip.find_first_not_of(".0") != -1) {
-			if(inet_ntop(AF_INET, &(ip = ntohl(ic->getLanIP())), buf, 64)) {
-			    sbuf = buf;
+    if(!c) {
+	c = clist.get(contactroot);
+	/*
+	*
+	* In case we wanna look up info about a user which
+	* is not on our contact list.
+	*
+	*/
+    }
 
-			    if(sbuf.find_first_not_of(".0") != -1) {
-				if(lastip != sbuf) {
-				    if(!lastip.empty()) lastip += " ";
-				    lastip += sbuf;
-				}
-			    }
-			}
+    if(!ev->isTransientDetail()) {
+	updateinforecord(ic, c);
 
-			c->setlastip(lastip);
-		    }
-		}
+    } else {
+	char buf[64];
+	string lastip, sbuf;
+	int ip;
+
+	if(inet_ntop(AF_INET, &(ip = ntohl(ic->getExtIP())), buf, 64))
+	    lastip = buf;
+
+	if(lastip.find_first_not_of(".0") != -1)
+	if(inet_ntop(AF_INET, &(ip = ntohl(ic->getLanIP())), buf, 64)) {
+	    sbuf = buf;
+	
+	    if(sbuf.find_first_not_of(".0") != -1 && lastip != sbuf) {
+		if(!lastip.empty()) lastip += " ";
+		lastip += sbuf;
 	    }
-	    break;
 
-	case ContactListEvent::UserInfoChange:
-	    if(dynamic_cast<UserInfoChangeEvent*>(ev)) {
-		updateinforecord(cli.getContact(ev->getUIN()), c);
-	    }
-	    break;
-
-	case ContactListEvent::UserAdded:
-	    break;
-
-	case ContactListEvent::UserRemoved:
-	    break;
-
-	case ContactListEvent::MessageQueueChanged:
-	    break;
-
-	case ContactListEvent::ServerBasedContact:
-	    break;
+	    c->setlastip(lastip);
+	}
     }
 }
 
@@ -1010,55 +1043,86 @@ void icqhook::socket_cb(SocketEvent *ev) {
     }
 }
 
-void icqhook::want_auto_resp_cb(AwayMessageEvent *ev) {
+void icqhook::want_auto_resp_cb(ICQMessageEvent *ev) {
     char buf[128];
+    string ident;
+    imcontact cont = imcontact(ev->getSenderUIN(), icq);
+    icqcontact *c = clist.get(cont);
+
+    ident = cont.totext();
+    if(c) ident += " (" + c->getdispnick() + ")";
+
+    sprintf(buf, _("%s requested our away message, sent the response"), ident.c_str());
 
-    sprintf(buf, _("%lu requested our icq away message, sent the response"), ev->getSenderUIN());
     logger.putmessage(buf);
-
-    ev->setMessage(conf.getourid(icq).awaymsg);
+    ev->setAwayMessage(rusconv("kw", conf.getawaymsg(icq)));
 }
 
 void icqhook::search_result_cb(SearchResultEvent *ev) {
     if(ev == searchevent) {
 	string line;
-	Contact *c = ev->getLastContactAdded();
+	ContactRef cont = ev->getLastContactAdded();
 
-	if(searchdest && c) {
-	    line = (c->getStatus() == STATUS_ONLINE ? "o " : "  ") +
-		rusconv("wk", c->getAlias());
+	if(searchdest && cont.get()) {
+	    icqcontact *c = new icqcontact(imcontact(cont->getUIN(), icq));
+	    icqcontact::basicinfo binfo = c->getbasicinfo();
+
+	    c->setnick(rusconv("wk", cont->getAlias()));
+	    c->setdispnick(c->getnick());
+
+	    binfo.fname = rusconv("wk", cont->getFirstName());
+	    binfo.lname = rusconv("wk", cont->getLastName());
+	    binfo.email = rusconv("wk", cont->getEmail());
+
+	    c->setbasicinfo(binfo);
+
+	    line = (cont->getStatus() == STATUS_ONLINE ? "o " : "  ") + c->getnick();
 
 	    if(line.size() > 12) line.resize(12);
 	    else line += string(12-line.size(), ' ');
 
-	    line += " " + 
-		rusconv("wk", c->getFirstName()) + " " +
-		rusconv("wk", c->getLastName());
-
-	    if(!c->getEmail().empty())
-		line += " <" + rusconv("wk", c->getEmail()) + ">";
-
-	    searchdest->additem(0, c->getUIN(), line);
+	    line += " " + binfo.fname + " " + binfo.lname;
+	    if(!binfo.email.empty()) line += " <" + binfo.email + ">";
 
+	    foundguys.push_back(c);
+	    searchdest->additem(conf.getcolor(cp_clist_icq), c, line);
 	    searchdest->redraw();
 	}
 
 	if(ev->isFinished()) {
+	    face.findready();
+
 	    face.log(_("+ [icq] whitepages search finished, %d found"),
 		ev->getContactList().size());
+
+	    searchdest = 0;
 	}
     }
 }
 
-void icqhook::self_event_cb(SelfEvent *ev) {
-    switch(ev->getType()) {
-	case SelfEvent::MyStatusChange:
-	    face.update();
-	    break;
-
-	case SelfEvent::MyUserInfoChange:
-	    updateinforecord(cli.getSelfContact(), clist.get(contactroot));
-	    updateinforecord(cli.getSelfContact(), clist.get(imcontact(cli.getUIN(), icq)));
-	    break;
+void icqhook::self_contact_userinfo_change_cb(UserInfoChangeEvent *ev) {
+    if(!ev->isTransientDetail()) {
+	updateinforecord(cli.getSelfContact(), clist.get(contactroot));
+	updateinforecord(cli.getSelfContact(), clist.get(imcontact(cli.getUIN(), icq)));
     }
 }
+
+void icqhook::self_contact_status_change_cb(StatusChangeEvent *ev) {
+    face.update();
+}
+
+void icqhook::server_based_contact_list_cb(ServerBasedContactEvent *ev) {
+    const ContactList &lst = ev->getContactList();
+    ContactList::const_iterator i = lst.begin();
+
+    while(i != lst.end()) {
+	imcontact cont((*i)->getUIN(), icq);
+
+	if(!clist.get(cont))
+	    clist.addnew(cont, true);
+
+	i++;
+    }
+
+//    cli.uploadServerBasedContactList();
+}
