@@ -1,7 +1,7 @@
 /*
 *
 * centericq MSN protocol handling class
-* $Id: msnhook.cc,v 1.13 2002/01/30 17:25:41 konst Exp $
+* $Id: msnhook.cc,v 1.23 2002/04/07 13:21:26 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -34,11 +34,14 @@
 msnhook mhook;
 
 #define TIMESTAMP maketm(d->hour-1, d->minute, d->day, d->month, d->year)
+#define DEFAULT_CHARSET "ISO-8859-1"
 
 msnhook::msnhook() {
     status = offline;
     fonline = false;
-    timer_reconnect = 0;
+
+    fcapabilities =
+	hoptChangableServer;
 
     for(int i = MSN_RNG; i != MSN_NUM_EVENTS; i++) {
 	msn_event[i] = 0;
@@ -71,7 +74,7 @@ void msnhook::connect() {
     face.log(_("+ [msn] connecting to the server"));
 
     if(!MSN_Login(account.nickname.c_str(), account.password.c_str(),
-    DEFAULT_HOST, DEFAULT_PORT)) {
+    account.server.c_str(), account.port)) {
 	face.log(_("+ [msn] logged in"));
 	fonline = true;
 	status = available;
@@ -80,8 +83,7 @@ void msnhook::connect() {
 	face.log(_("+ [msn] unable to connect to the server"));
     }
 
-    time(&mhook.timer_reconnect);
-    msn_Russian = 0;
+    msn_Russian = conf.getrussian();
 }
 
 void msnhook::disconnect() {
@@ -91,20 +93,6 @@ void msnhook::disconnect() {
 }
 
 void msnhook::exectimers() {
-    time_t tcurrent = time(0);
-
-    if(logged()) {
-    } else {
-	if(tcurrent-timer_reconnect > PERIOD_RECONNECT) {
-	    if(online() && !logged()) {
-		disconnect();
-	    } else if(manualstatus != offline) {
-		if(!manager.isopen()) {
-		    connect();
-		}
-	    }
-	}
-    }
 }
 
 void msnhook::main() {
@@ -156,29 +144,30 @@ bool msnhook::send(const imevent &ev) {
 
     if(ev.gettype() == imevent::message) {
 	const immessage *m = static_cast<const immessage *>(&ev);
-	if(m) text = rusconv("kw", m->gettext());
+	if(m) text = m->gettext();
     } else if(ev.gettype() == imevent::url) {
 	const imurl *m = static_cast<const imurl *>(&ev);
-	if(m) text = rusconv("kw", m->geturl()) + "\n\n" + rusconv("kw", m->getdescription());
+	if(m) text = m->geturl() + "\n\n" + m->getdescription();
     }
 
-    if(!text.empty()) {
-	icqcontact *c = clist.get(ev.getcontact());
+    icqcontact *c = clist.get(ev.getcontact());
+    text = siconv(text, conf.getrussian() ? "koi8-r" : DEFAULT_CHARSET, "utf8");
 
-	if(c)
-	if(c->getstatus() != offline) {
-	    MSN_SendMessage(ev.getcontact().nickname.c_str(), text.c_str());
-	    return true;
-	}
+    if(c)
+    if(c->getstatus() != offline) {
+	MSN_SendMessage(ev.getcontact().nickname.c_str(), text.c_str());
+	return true;
     }
 
     return false;
 }
 
-void msnhook::sendnewuser(const imcontact c) {
+void msnhook::sendnewuser(const imcontact &c) {
     if(logged()) {
 	MSN_AddContact(c.nickname.c_str());
     }
+
+    requestinfo(c);
 }
 
 void msnhook::setautostatus(imstatus st) {
@@ -211,12 +200,12 @@ imstatus msnhook::getstatus() const {
     return status;
 }
 
-void msnhook::removeuser(const imcontact ic) {
+void msnhook::removeuser(const imcontact &ic) {
     face.log(_("+ [msn] removing %s from the contacts"), ic.nickname.c_str());
     MSN_RemoveContact(ic.nickname.c_str());
 }
 
-bool msnhook::isourid(const string nick) {
+bool msnhook::isourid(const string &nick) {
     string s1, s2;
     icqconf::imaccount account = conf.getourid(msn);
 
@@ -254,12 +243,30 @@ imstatus msnhook::msn2imstatus(int st) {
     }
 }
 
+void msnhook::requestinfo(const imcontact &ic) {
+    icqcontact *c = clist.get(ic);
+
+    if(c) {
+	icqcontact::moreinfo m = c->getmoreinfo();
+	icqcontact::basicinfo b = c->getbasicinfo();
+
+	b.email = ic.nickname;
+	if(b.email.find("@") == -1) b.email += "@hotmail.com";
+	m.homepage = "http://members.msn.com/" + b.email;
+
+	c->setbasicinfo(b);
+	c->setmoreinfo(m);
+    }
+}
+
+
 // ----------------------------------------------------------------------------
 
 void msnhook::messaged(void *data) {
     MSN_InstantMessage *d = (MSN_InstantMessage *) data;
     imcontact ic(d->sender, msn);
     icqcontact *c;
+    string text;
 
     if(!isourid(d->sender))
     if(strlen(d->msg)) {
@@ -268,10 +275,11 @@ void msnhook::messaged(void *data) {
 	if(!c)
 	if(c = clist.addnew(ic))
 	if(d->friendlyhandle) {
-	    c->setdispnick(d->friendlyhandle);
+	    c->setdispnick(unmime((string) d->friendlyhandle));
 	}
 
-	em.store(immessage(ic, imevent::incoming, d->msg));
+	text = siconv(d->msg, "utf8", conf.getrussian() ? "koi8-r" : DEFAULT_CHARSET);
+	em.store(immessage(ic, imevent::incoming, text));
 
 	if(c)
 	if(c->getstatus() == offline) {
@@ -289,7 +297,7 @@ void msnhook::statuschanged(void *data) {
 	if(!c) {
 	    c = clist.addnew(ic, false);
 	    if(d->friendlyhandle) {
-		c->setdispnick(d->friendlyhandle);
+		c->setdispnick(unmime((string) d->friendlyhandle));
 	    }
 	}
 
@@ -312,13 +320,13 @@ void msnhook::disconnected(void *data) {
     icqcontact *c;
 
     if(mhook.online()) {
+	MSN_Logout();
 	face.log(_("+ [msn] disconnected"));
 	mhook.fonline = false;
     }
 
     clist.setoffline(msn);
     mhook.status = offline;
-    time(&mhook.timer_reconnect);
     face.update();
 }
 
