@@ -6,7 +6,6 @@
 
 class msnhook : public abstracthook {
     protected:
-	time_t timer_reconnect;
 	imstatus status;
 	bool fonline;
 
@@ -19,7 +18,7 @@ class msnhook : public abstracthook {
 	static void mailed(void *data);
 
 	static imstatus msn2imstatus(int st);
-	static bool isourid(const string nick);
+	static bool isourid(const string &nick);
 
     public:
 	msnhook();
@@ -42,8 +41,9 @@ class msnhook : public abstracthook {
 
 	bool send(const imevent &ev);
 
-	void sendnewuser(const imcontact c);
-	void removeuser(const imcontact ic);
+	void sendnewuser(const imcontact &c);
+	void removeuser(const imcontact &ic);
+	void requestinfo(const imcontact &ic);
 
 	void setautostatus(imstatus st);
 	imstatus getstatus() const;
