@@ -1,7 +1,7 @@
 /*
 *
 * centericq yahoo! protocol handling class
-* $Id: yahoohook.cc,v 1.19 2002/01/24 15:11:29 konst Exp $
+* $Id: yahoohook.cc,v 1.39 2002/07/03 09:58:12 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -33,10 +33,22 @@
 
 yahoohook yhook;
 
-yahoohook::yahoohook() {
-    fonline = false;
-    timer_reconnect = 0;
-    context = 0;
+static int stat2int[imstatus_size] = {
+    0,
+    YAHOO_STATUS_AVAILABLE,
+    YAHOO_STATUS_INVISIBLE,
+    YAHOO_STATUS_CUSTOM,
+    YAHOO_STATUS_BUSY,
+    YAHOO_STATUS_NOTATDESK,
+    YAHOO_STATUS_NOTATHOME,
+    YAHOO_STATUS_BRB
+};
+
+yahoohook::yahoohook()
+    : fonline(false), context(0)
+{
+    fcapabilities =
+	hoptCanSetAwayMsg;
 }
 
 yahoohook::~yahoohook() {
@@ -57,11 +69,9 @@ void yahoohook::initcontext(const icqconf::imaccount account) {
     memset(&options, 0, sizeof(options));
     options.connect_mode = YAHOO_CONNECT_NORMAL;
 
-    face.log("+ initializing %s engine",
+    face.log(_("+ initializing %s engine"),
 	conf.getprotocolname(account.pname).c_str());
 
-    alarm(5);
-
     if(context = yahoo_init(account.nickname.c_str(), account.password.c_str(),
     &options)) {
 	context->yahoo_Disconnected = &disconnected;
@@ -79,9 +89,23 @@ void yahoohook::initcontext(const icqconf::imaccount account) {
     } else {
 	face.log("+ unable to init %s engine",
 	    conf.getprotocolname(account.pname).c_str());
-    }
 
-    alarm(0);
+	string err;
+
+	switch(yahoo_geterrno()) {
+	    case YERR_CONNECT:
+		err = _("unable to connect to the pager host");
+		break;
+	    case YERR_INVALID_LOGIN:
+		err = _("username and password mismatch");
+		break;
+	}
+
+	if(!err.empty()) {
+	    logger.putmessage((string) "yahoo: " + err);
+	    face.log((string) "+ [yahoo] " + err);
+	}
+    }
 }
 
 void yahoohook::connect() {
@@ -93,25 +117,27 @@ void yahoohook::connect() {
     if(context) {
 	face.log(_("+ [yahoo] connecting to the server"));
 
-	alarm(5);
 	r = yahoo_connect(context);
-	alarm(0);
-	logger.putourstatus(yahoo, offline, available);
 
 	if(!r) {
 	    face.log(_("+ [yahoo] unable to connect to the server"));
 	    disconnected(context);
 	} else {
+	    logger.putourstatus(yahoo, offline, manualstatus);
 	    yahoo_get_config(context);
-	    if(!yahoo_cmd_logon(context, ourstatus = YAHOO_STATUS_AVAILABLE)) {
+
+	    if(!yahoo_cmd_logon(context, stat2int[ourstatus = available])) {
 		fonline = true;
+
+		if(manualstatus != ourstatus)
+		    setstatus(manualstatus);
+
 		face.log(_("+ [yahoo] logged in"));
 	    }
 	}
     }
 
     yahoo_Russian = 0;
-    time(&timer_reconnect);
 }
 
 void yahoohook::main() {
@@ -139,20 +165,6 @@ void yahoohook::disconnect() {
 }
 
 void yahoohook::exectimers() {
-    time_t tcurrent = time(0);
-
-    if(logged()) {
-    } else {
-	if(tcurrent-timer_reconnect > PERIOD_RECONNECT) {
-	    if(online() && !logged()) {
-		disconnect();
-	    } else if(manualstatus != offline) {
-		if(!manager.isopen()) {
-		    connect();
-		}
-	    }
-	}
-    }
 }
 
 struct tm *yahoohook::timestamp() {
@@ -173,13 +185,11 @@ bool yahoohook::send(const imevent &ev) {
 	    const imurl *m = static_cast<const imurl *>(&ev);
 	    if(m) text = rusconv("kw", m->geturl()) + "\n\n" + rusconv("kw", m->getdescription());
 	}
-    }
 
-    if(!text.empty()) {
 	for(is = text.begin(); is != text.end(); is++)
 	    if((unsigned) *is < 32) *is = ' ';
 
-	if(c->getstatus() != offline) {
+	if((c->getstatus() != offline) && c->inlist()) {
 	    yahoo_cmd_msg(context, conf.getourid(yahoo).nickname.c_str(),
 		ev.getcontact().nickname.c_str(), text.c_str());
 	} else {
@@ -201,7 +211,7 @@ bool yahoohook::logged() const {
     return fonline;
 }
 
-void yahoohook::sendnewuser(const imcontact ic) {
+void yahoohook::sendnewuser(const imcontact &ic) {
     char *group;
 
     if(yahoo)
@@ -218,9 +228,11 @@ void yahoohook::sendnewuser(const imcontact ic) {
 		conf.getourid(yahoo).nickname.c_str(), group, "");
 	}
     }
+
+    requestinfo(ic);
 }
 
-void yahoohook::removeuser(const imcontact ic) {
+void yahoohook::removeuser(const imcontact &ic) {
     if(context)
     if(online()) {
 	face.log(_("+ [yahoo] removing %s from the contacts"), ic.nickname.c_str());
@@ -234,19 +246,23 @@ imstatus yahoohook::yahoo2imstatus(int status) const {
 
     switch(status) {
 	case YAHOO_STATUS_AVAILABLE:
-	case YAHOO_STATUS_CUSTOM:
 	    st = available;
 	    break;
+	case YAHOO_STATUS_CUSTOM:
+	    st = freeforchat;
+	    break;
 	case YAHOO_STATUS_BUSY:
-	    st = occupied;
+	    st = dontdisturb;
 	    break;
 	case YAHOO_STATUS_BRB:
 	case YAHOO_STATUS_IDLE:
 	case YAHOO_STATUS_ONPHONE:
 	    st = away;
 	    break;
-	case YAHOO_STATUS_NOTATHOME:
 	case YAHOO_STATUS_NOTATDESK:
+	    st = occupied;
+	    break;
+	case YAHOO_STATUS_NOTATHOME:
 	case YAHOO_STATUS_NOTINOFFICE:
 	case YAHOO_STATUS_ONVACATION:
 	case YAHOO_STATUS_OUTTOLUNCH:
@@ -269,17 +285,6 @@ bool yahoohook::enabled() const {
 }
 
 void yahoohook::setautostatus(imstatus st) {
-    static int stat2int[imstatus_size] = {
-	0,
-	YAHOO_STATUS_AVAILABLE,
-	YAHOO_STATUS_INVISIBLE,
-	YAHOO_STATUS_CUSTOM,
-	YAHOO_STATUS_BUSY,
-	YAHOO_STATUS_BUSY,
-	YAHOO_STATUS_NOTATHOME,
-	YAHOO_STATUS_IDLE
-    };
-
     if(st == offline) {
 	if(getstatus() != offline) {
 	    disconnect();
@@ -289,15 +294,16 @@ void yahoohook::setautostatus(imstatus st) {
 	    connect();
 	} else {
 	    logger.putourstatus(yahoo, getstatus(), st);
-	    ourstatus = stat2int[st];
+	    ourstatus = st;
 
 	    if(st == available) {
-		yahoo_cmd_set_back_mode(context, ourstatus, "available");
+		yahoo_cmd_set_back_mode(context, stat2int[st], "available");
 	    } else {
-		if(ourstatus == YAHOO_STATUS_IDLE) {
+		if(stat2int[st] == YAHOO_STATUS_IDLE) {
 		    yahoo_cmd_idle(context);
 		} else {
-		    yahoo_cmd_set_away_mode(context, ourstatus, "away");
+		    yahoo_cmd_set_away_mode(context, stat2int[st],
+			rusconv("kw", conf.getawaymsg(aim)).c_str());
 		}
 	    }
 	}
@@ -305,7 +311,22 @@ void yahoohook::setautostatus(imstatus st) {
 }
 
 imstatus yahoohook::getstatus() const {
-    return online() ? yahoo2imstatus(ourstatus) : offline;
+    return online() ? ourstatus : offline;
+}
+
+void yahoohook::requestinfo(const imcontact &ic) {
+    icqcontact *c = clist.get(ic);
+
+    if(c) {
+	icqcontact::moreinfo m = c->getmoreinfo();
+	icqcontact::basicinfo b = c->getbasicinfo();
+
+	m.homepage = "http://profiles.yahoo.com/" + ic.nickname;
+	b.email = ic.nickname + "@yahoo.com";
+
+	c->setbasicinfo(b);
+	c->setmoreinfo(m);
+    }
 }
 
 // ----------------------------------------------------------------------------
@@ -315,38 +336,42 @@ void yahoohook::disconnected(yahoo_context *y) {
     icqcontact *c;
 
     if(yhook.fonline) {
+	close(y->sockfd);
 	face.log(_("+ [yahoo] disconnected from the network"));
 	yhook.fonline = false;
     }
 
     clist.setoffline(yahoo);
-    time(&yhook.timer_reconnect);
     face.update();
 }
 
-void yahoohook::userlogon(yahoo_context *y, const char *nick, int status) {
-    yhook.userstatus(yhook.context, nick, status);
+void yahoohook::userlogon(yahoo_context *y, struct yahoo_idstatus *rec) {
+    yhook.userstatus(yhook.context, rec);
 }
 
-void yahoohook::userlogoff(yahoo_context *y, const char *nick) {
-    yhook.userstatus(yhook.context, nick, -1);
+void yahoohook::userlogoff(yahoo_context *y, struct yahoo_idstatus *rec) {
+    yhook.userstatus(yhook.context, rec);
 }
 
-void yahoohook::userstatus(yahoo_context *y, const char *nick, int status) {
-    if(nick)
-    if(strlen(nick)) {
-	imcontact ic(nick, yahoo);
+void yahoohook::userstatus(yahoo_context *y, struct yahoo_idstatus *rec) {
+    if(rec->id)
+    if(strlen(rec->id)) {
+	imcontact ic(rec->id, yahoo);
 	icqcontact *c = clist.get(ic);
 
-	if(!c) {
+	if(!c)
+	if(conf.getourid(yahoo).nickname != rec->id) {
 	    c = clist.addnew(ic, false);
 	}
 
-	logger.putonline(ic, c->getstatus(), yhook.yahoo2imstatus(status));
-	c->setstatus(yhook.yahoo2imstatus(status));
+	if(c) {
+	    logger.putonline(ic, c->getstatus(), yhook.yahoo2imstatus(rec->status));
 
-	if(c->getstatus() != offline) {
-	    c->setlastseen();
+	    c->setstatus(yhook.yahoo2imstatus(rec->status));
+	    if(c->getstatus() != offline)
+		c->setlastseen();
+
+	    c->setabout(rec->status_msg ? rusconv("wk", rec->status_msg) : "");
 	}
     }
 }
@@ -362,12 +387,6 @@ void yahoohook::recvmessage(yahoo_context *y, const char *nick, const char *msg)
 
 	if(strlen(msg)) {
 	    em.store(immessage(ic, imevent::incoming, rusconv("wk", msg)));
-	    icqcontact *c = clist.get(ic);
-
-	    if(c)
-	    if(c->getstatus() == offline) {
-		c->setstatus(available);
-	    }
 	}
     }
 }
