@@ -8,14 +8,12 @@ class yahoohook: public abstracthook {
     protected:
 	struct yahoo_context *context;
 	bool fonline;
-	int ourstatus;
-
-	time_t timer_reconnect;
+	imstatus ourstatus;
 
 	static void disconnected(yahoo_context *y);
-	static void userlogon(yahoo_context *y, const char *nick, int status);
-	static void userlogoff(yahoo_context *y, const char *nick);
-	static void userstatus(yahoo_context *y, const char *nick, int status);
+	static void userlogon(yahoo_context *y, struct yahoo_idstatus *rec);
+	static void userlogoff(yahoo_context *y, struct yahoo_idstatus *rec);
+	static void userstatus(yahoo_context *y, struct yahoo_idstatus *rec);
 	static void recvbounced(yahoo_context *y, const char *nick);
 	static void recvmessage(yahoo_context *y, const char *nick, const char *msg);
 	static void log(yahoo_context *y, const char *msg);
@@ -43,8 +41,9 @@ class yahoohook: public abstracthook {
 	bool logged() const;
 	bool enabled() const;
 
-	void sendnewuser(const imcontact ic);
-	void removeuser(const imcontact ic);
+	void sendnewuser(const imcontact &ic);
+	void removeuser(const imcontact &ic);
+	void requestinfo(const imcontact &ic);
 
 	bool send(const imevent &ev);
 
