@@ -7,6 +7,8 @@
 #include <fstream>
 #include <algorithm>
 #include <list>
+#include <map>
+#include <memory>
 
 #include <sys/time.h>
 #include <sys/types.h>
@@ -19,6 +21,15 @@
 #include "kkstrtext.h"
 #include "conf.h"
 
+
+#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
+#include <sys/param.h>
+#include <sys/mount.h>
+#else
+#include <sys/vfs.h>
+#endif
+
+
 #define PERIOD_RECONNECT        40
 
 #ifdef ENABLE_NLS
@@ -43,17 +54,8 @@ using namespace std;
 
 #endif
 
-#define PERIOD_KEEPALIVE        100
-#define PERIOD_SELECT           1
-#define PERIOD_TCP              1
-#define PERIOD_RESOLVE          40
-#define PERIOD_DISCONNECT       130
 #define PERIOD_RESEND           20
 #define PERIOD_CHECKMAIL        30
-#define PERIOD_WAITNEWUIN       20
-#define PERIOD_SOCKSALIVE       30
-#define PERIOD_WAIT_KEEPALIVE   15
-#define PERIOD_CLIST_RESEND     5
 #define PERIOD_DISPUPDATE       2
 
 #endif
