@@ -1,7 +1,7 @@
 /*
 *
 * centericq configuration handling routines
-* $Id: icqconf.cc,v 1.37 2002/02/07 10:55:42 konst Exp $
+* $Id: icqconf.cc,v 1.70 2002/05/14 07:31:06 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -30,6 +30,9 @@
 #include "icqface.h"
 #include "icqcontacts.h"
 #include "abstracthook.h"
+#include "imexternal.h"
+#include "eventmanager.h"
+#include "imlogger.h"
 
 icqconf::icqconf() {
     rs = rscard;
@@ -38,8 +41,8 @@ icqconf::icqconf() {
 
     autoaway = autona = 0;
 
-    hideoffline = antispam = russian = makelog = false;
-    savepwd = mailcheck = true;
+    hideoffline = antispam = russian = makelog = askaway = chatmode = false;
+    savepwd = mailcheck = fenoughdiskspace = true;
 
     basedir = (string) getenv("HOME") + "/.centericq/";
 }
@@ -48,33 +51,99 @@ icqconf::~icqconf() {
 }
 
 icqconf::imaccount icqconf::getourid(protocolname pname) {
-    ifstream f;
     imaccount im;
-    string buf, fname;
     vector<imaccount>::iterator i;
 
     i = find(accounts.begin(), accounts.end(), pname);
     im = i == accounts.end() ? imaccount(pname) : *i;
 
-    im.awaymsg = "";
-    fname = conf.getconfigfname("awaymsg-" + getprotocolname(im.pname));
+    return im;
+}
+
+void icqconf::setourid(const imaccount &im) {
+    vector<imaccount>::iterator i;
+
+    static struct {
+	string server;
+	int port;
+    } defservers[protocolname_size] = {
+	{ "login.icq.com", 5190 },
+	{ "", 0 },
+	{ "messenger.hotmail.com", 1863 },
+	{ "toc.oscar.aol.com", 9898 },
+	{ "irc.openprojects.net", 6667 }
+    };
+
+    i = find(accounts.begin(), accounts.end(), im.pname);
+
+    if(i != accounts.end()) {
+	if(im.empty()) {
+	    accounts.erase(i);
+	    i = accounts.end();
+	} else {
+	    *i = im;
+	}
+    } else if(!im.empty()) {
+	accounts.push_back(im);
+	i = accounts.end()-1;
+    }
+
+    if(i != accounts.end()) {
+	if(i->server.empty()) {
+	    i->server = defservers[i->pname].server;
+	    i->port = defservers[i->pname].port;
+	}
+
+	if(!i->port) i->port = defservers[i->pname].port;
+    }
+}
+
+string icqconf::getawaymsg(protocolname pname) const {
+    string r, buf, fname;
+    ifstream f;
+
+    if(!(gethook(pname).getcapabilities() & hoptCanSetAwayMsg))
+	return "";
+
+    fname = conf.getconfigfname("awaymsg-" + getprotocolname(pname));
     f.open(fname.c_str());
 
     if(f.is_open()) {
 	while(!f.eof()) {
 	    getstring(f, buf);
-	    if(!im.awaymsg.empty()) im.awaymsg += "\n";
-	    im.awaymsg += buf;
+	    if(!r.empty()) r += "\n";
+	    r += buf;
 	}
 
 	f.close();
     }
 
-    return im;
+    if(r.empty()) {
+	char buf[512];
+
+	sprintf(buf, _("I do really enjoy the default %s away message of %s %s."),
+	    getprotocolname(pname).c_str(), PACKAGE, VERSION);
+
+	return buf;
+    }
+
+    return r;
 }
 
-int icqconf::getouridcount() const {
-    return accounts.size();
+void icqconf::setawaymsg(protocolname pname, const string &amsg) {
+    string fname;
+    ofstream f;
+
+    if(!(gethook(pname).getcapabilities() & hoptCanSetAwayMsg))
+	return;
+
+    fname = conf.getconfigfname("awaymsg-" + getprotocolname(pname));
+    f.open(fname.c_str());
+
+    if(f.is_open()) {
+	f << amsg;
+	f.close();
+    }
 }
 
 void icqconf::checkdir() {
@@ -87,6 +156,7 @@ void icqconf::load() {
     loadmainconfig();
     loadcolors();
     loadactions();
+    external.load();
 }
 
 void icqconf::loadmainconfig() {
@@ -96,8 +166,9 @@ void icqconf::loadmainconfig() {
     protocolname pname;
 
     if(f.is_open()) {
-	mailcheck = false;
+	mailcheck = askaway = false;
 	savepwd = true;
+	setsmtphost("");
 
 	while(!f.eof()) {
 	    getstring(f, buf);
@@ -116,7 +187,10 @@ void icqconf::loadmainconfig() {
 	    if(param == "sockspass") sockspass = buf; else
 	    if((param == "usegroups") || (param == "group1")) fgroupmode = group1; else
 	    if(param == "group2") fgroupmode = group2; else
-	    if(param == "log") makelog = true; else {
+	    if(param == "smtp") setsmtphost(buf); else
+	    if(param == "log") makelog = true; else
+	    if(param == "chatmode") chatmode = true; else
+	    if(param == "askaway") askaway = true; else {
 		for(pname = icq; pname != protocolname_size; (int) pname += 1) {
 		    buf = getprotocolname(pname);
 		    if(param.substr(0, buf.size()) == buf) {
@@ -132,186 +206,101 @@ void icqconf::loadmainconfig() {
     }
 }
 
-void icqconf::setourid(const imaccount im) {
-    vector<imaccount>::iterator i;
+void icqconf::save() {
+    string fname = getconfigfname("config"), param;
+    int away, na;
 
-    i = find(accounts.begin(), accounts.end(), im.pname);
-
-    if(i != accounts.end()) {
-	if(im.empty()) {
-	    accounts.erase(i);
-	} else {
-	    *i = im;
-	}
-    } else {
-	if(!im.empty()) {
-	    accounts.push_back(im);
-	    i = accounts.end()-1;
-
-	    switch(i->pname) {
-		case icq:
-		    if(i->server.empty()) {
-			i->server = "login.icq.com";
-			i->port = 5190;
-		    }
-		    break;
-	    }
-	}
-    }
-
-    if(gethook(i->pname).getcapabilities() & hoptCanSetAwayMsg) {
-	if(i->awaymsg.empty()) {
-	    char buf[512];
-	    sprintf(buf, _("I do really enjoy the default away message of centericq %s."), VERSION);
-	    i->awaymsg = buf;
-	}
-
-	string fname = conf.getconfigfname("awaymsg-" + getprotocolname(i->pname));
+    if(enoughdiskspace()) {
 	ofstream f(fname.c_str());
 
 	if(f.is_open()) {
-	    f << i->awaymsg;
+	    if(!getsockshost().empty()) {
+		string user, pass;
+		getsocksuser(user, pass);
+		f << "sockshost\t" << getsockshost() + ":" + i2str(getsocksport()) << endl;
+		f << "socksusername\t" << user << endl;
+		f << "sockspass\t" << pass << endl;
+	    }
+
+	    getauto(away, na);
+
+	    if(russian) f << "russian" << endl;
+	    if(away) f << "autoaway\t" << i2str(away) << endl;
+	    if(na) f << "autona\t" << i2str(na) << endl;
+	    if(hideoffline) f << "hideoffline" << endl;
+	    if(getquote()) f << "quotemsgs" << endl;
+	    if(getantispam()) f << "antispam" << endl;
+	    if(getmailcheck()) f << "mailcheck" << endl;
+	    if(getaskaway()) f << "askaway" << endl;
+	    if(getchatmode()) f << "chatmode" << endl;
+
+	    f << "smtp\t" << getsmtphost() << ":" << dec << getsmtpport() << endl;
+
+	    switch(getgroupmode()) {
+		case group1: f << "group1" << endl; break;
+		case group2: f << "group2" << endl; break;
+	    }
+
+	    if(getmakelog()) f << "log" << endl;
+
+	    vector<imaccount>::iterator ia;
+	    for(ia = accounts.begin(); ia != accounts.end(); ia++)
+		ia->write(f);
+
 	    f.close();
 	}
     }
 }
 
-void icqconf::save() {
-    string fname = getconfigfname("config"), param;
-    ofstream f(fname.c_str());
-    int away, na;
-    vector<imaccount>::iterator ia;
-
-    if(f.is_open()) {
-	if(!getsockshost().empty()) {
-	    string user, pass;
-	    getsocksuser(user, pass);
-	    f << "sockshost\t" << getsockshost() + ":" + i2str(getsocksport()) << endl;
-	    f << "socksusername\t" << user << endl;
-	    f << "sockspass\t" << pass << endl;
-	}
-
-	getauto(away, na);
-
-	if(russian) f << "russian" << endl;
-	if(away) f << "autoaway\t" << i2str(away) << endl;
-	if(na) f << "autona\t" << i2str(na) << endl;
-	if(hideoffline) f << "hideoffline" << endl;
-	if(getquote()) f << "quotemsgs" << endl;
-	if(getantispam()) f << "antispam" << endl;
-	if(getmailcheck()) f << "mailcheck" << endl;
-
-
-	switch(getgroupmode()) {
-	    case group1: f << "group1" << endl; break;
-	    case group2: f << "group2" << endl; break;
-	}
-
-	if(getmakelog()) f << "log" << endl;
-
-	for(ia = accounts.begin(); ia != accounts.end(); ia++) {
-	    ia->write(f);
-	}
-
-	f.close();
-    }
-}
-
-int icqconf::getcolor(int npair) const {
-    return
-	find(boldcolors.begin(), boldcolors.end(), npair) != boldcolors.end()
-	? boldcolor(npair) : normalcolor(npair);
-}
-
 void icqconf::loadcolors() {
-    FILE *f;
-    char buf[512], sub[512], *p;
     string tname = getconfigfname("colorscheme");
-    int nfg, nbg, npair;
 
-    if(access(tname.c_str(), F_OK))
-    if(f = fopen(tname.c_str(), "w")) {
-	switch(conf.getregcolor()) {
-	    case rcdark:
-		fprintf(f, "status\tblack/white\n");
-		fprintf(f, "dialog_text\tblack/white\n");
-		fprintf(f, "dialog_menu\tblack/white\n");
-		fprintf(f, "dialog_selected\twhite/black\tbold\n");
-		fprintf(f, "dialog_highlight\tred/white\n");
-		fprintf(f, "dialog_frame\tblue/white\n");
-		fprintf(f, "main_text\tcyan/black\n");
-		fprintf(f, "main_menu\tgreen/black\n");
-		fprintf(f, "main_selected\tblack/white\n");
-		fprintf(f, "main_highlight\tyellow/black\tbold\n");
-		fprintf(f, "main_frame\tblue/black\tbold\n");
-		fprintf(f, "clist_icq\tgreen/black\n");
-		fprintf(f, "clist_msn\tcyan/black\n");
-		fprintf(f, "clist_yahoo\tmagenta/black\n");
-		fprintf(f, "clist_infocard\twhite/black\n");
-		fprintf(f, "clist_root\tred/black\tbold\n");
+    switch(conf.getregcolor()) {
+	case rcdark:
+	    schemer.push(cp_status, "status black/white");
+	    schemer.push(cp_dialog_text, "dialog_text   black/white");
+	    schemer.push(cp_dialog_menu, "dialog_menu   black/white");
+	    schemer.push(cp_dialog_selected, "dialog_selected   white/transparent   bold");
+	    schemer.push(cp_dialog_highlight, "dialog_highlight red/white");
+	    schemer.push(cp_dialog_frame, "dialog_frame blue/white");
+	    schemer.push(cp_main_text, "main_text   cyan/transparent");
+	    schemer.push(cp_main_menu, "main_menu   green/transparent");
+	    schemer.push(cp_main_selected, "main_selected   black/white");
+	    schemer.push(cp_main_highlight, "main_highlight yellow/transparent  bold");
+	    schemer.push(cp_main_frame, "main_frame blue/transparent    bold");
+	    schemer.push(cp_clist_icq, "clist_icq   green/transparent");
+	    schemer.push(cp_clist_yahoo, "clist_yahoo   magenta/transparent");
+	    schemer.push(cp_clist_infocard, "clist_infocard white/transparent");
+	    schemer.push(cp_clist_msn, "clist_msn   cyan/transparent");
+	    schemer.push(cp_clist_aim, "clist_aim   yellow/transparent");
+	    schemer.push(cp_clist_irc, "clist_irc    blue/transparent");
+	    break;
 
-		break;
-	    case rcblue:
-		fprintf(f, "status\tblack/white\n");
-		fprintf(f, "dialog_text\tblack/cyan\n");
-		fprintf(f, "dialog_menu\tblack/cyan\n");
-		fprintf(f, "dialog_selected\twhite/black\tbold\n");
-		fprintf(f, "dialog_highlight\twhite/cyan\tbold\n");
-		fprintf(f, "dialog_frame\tblack/cyan\n");
-		fprintf(f, "main_text\twhite/blue\n");
-		fprintf(f, "main_menu\twhite/blue\n");
-		fprintf(f, "main_selected\tblack/cyan\n");
-		fprintf(f, "main_highlight\twhite/blue\tbold\n");
-		fprintf(f, "main_frame\tblue/blue\tbold\n");
-		fprintf(f, "clist_icq\tgreen/blue\n");
-		fprintf(f, "clist_yahoo\tmagenta/blue\n");
-		fprintf(f, "clist_msn\tcyan/blue\n");
-		fprintf(f, "clist_infocard\twhite/blue\n");
-		fprintf(f, "clist_root\tred/blue\tbold\n");
-		break;
-	}
-	fclose(f);
+	case rcblue:
+	    schemer.push(cp_status, "status  black/white");
+	    schemer.push(cp_dialog_text, "dialog_text black/cyan");
+	    schemer.push(cp_dialog_menu, "dialog_menu black/cyan");
+	    schemer.push(cp_dialog_selected, "dialog_selected white/black bold");
+	    schemer.push(cp_dialog_highlight, "dialog_highlight    white/cyan  bold");
+	    schemer.push(cp_dialog_frame, "dialog_frame    black/cyan");
+	    schemer.push(cp_main_text, "main_text   white/blue");
+	    schemer.push(cp_main_menu, "main_menu   white/blue");
+	    schemer.push(cp_main_selected, "main_selected   black/cyan");
+	    schemer.push(cp_main_highlight, "main_highlight  white/blue  bold");
+	    schemer.push(cp_main_frame, "main_frame  blue/blue   bold");
+	    schemer.push(cp_clist_icq, "clist_icq   green/blue");
+	    schemer.push(cp_clist_yahoo, "clist_yahoo magenta/blue");
+	    schemer.push(cp_clist_msn, "clist_msn   cyan/blue");
+	    schemer.push(cp_clist_infocard, "clist_infocard  white/blue");
+	    schemer.push(cp_clist_aim, "clist_aim   yellow/blue");
+	    schemer.push(cp_clist_irc, "clist_irc   blue/blue   bold");
+	    break;
     }
 
-    if(f = fopen(tname.c_str(), "r")) {
-	boldcolors.clear();
-
-	while(!feof(f)) {
-	    freads(f, buf, 512);
-	    sscanf(buf, "%s", sub);
-	    tname = sub;
-
-	    sscanf(buf+tname.size(), "%s", sub);
-	    if(p = strchr(sub, '/')) {
-		*p = 0;
-		nfg = findcolor(sub);
-		nbg = findcolor(p+1);
-	    }
-
-	    if(tname == "status") npair = cp_status; else
-	    if(tname == "dialog_text") npair = cp_dialog_text; else
-	    if(tname == "dialog_menu") npair = cp_dialog_menu; else
-	    if(tname == "dialog_selected") npair = cp_dialog_selected; else
-	    if(tname == "dialog_highlight") npair = cp_dialog_highlight; else
-	    if(tname == "dialog_frame") npair = cp_dialog_frame; else
-	    if(tname == "main_text") npair = cp_main_text; else
-	    if(tname == "main_menu") npair = cp_main_menu; else
-	    if(tname == "main_selected") npair = cp_main_selected; else
-	    if(tname == "main_highlight") npair = cp_main_highlight; else
-	    if(tname == "main_frame") npair = cp_main_frame; else
-	    if(tname == "clist_icq") npair = cp_clist_icq; else
-	    if(tname == "clist_yahoo") npair = cp_clist_yahoo; else
-	    if(tname == "clist_msn") npair = cp_clist_msn; else
-	    if(tname == "clist_infocard") npair = cp_clist_infocard; else
-	    if(tname == "clist_root") npair = cp_clist_root; else
-	    continue;
-
-	    init_pair(npair, nfg, nbg);
-	    if(!strcmp(buf+strlen(buf)-4, "bold")) {
-		boldcolors.push_back(npair);
-	    }
-	}
-	fclose(f);
+    if(access(tname.c_str(), F_OK)) {
+	schemer.save(tname);
+    } else {
+	schemer.load(tname);
     }
 }
 
@@ -423,32 +412,54 @@ void icqconf::loadsounds() {
 }
 
 void icqconf::loadactions() {
-    string fname = getconfigfname("actions"), buf, name;
+    string fname = getconfigfname("actions"), buf, name, browser;
+    ifstream f;
+    bool cont;
 
     if(access(fname.c_str(), F_OK)) {
+	buf = getenv("PATH") ? getenv("PATH") : "";
+
+	if(pathfind(browser = "mozilla", buf, X_OK).empty()) {
+	    browser = "netscape";
+	}
+
 	ofstream of(fname.c_str());
+
 	if(of.is_open()) {
 	    of << "# This file contains external actions configuration for centericq" << endl;
 	    of << "# Every line should look like: <action> <command>" << endl;
 	    of << "# Possible actions are: openurl" << endl << endl;
 
-	    of << "openurl\t";
-	    of << "(if test ! -z \"`ps x | grep /netscape | grep -v grep`\"; ";
-	    of << "then DISPLAY=0:0 netscape -remote 'openURL($url$, new-window)'; else ";
-	    of << "DISPLAY=0:0 netscape \"$url$\"; fi) >/dev/null 2>&1 &" << endl;
+	    of << "openurl \\" << endl;
+	    of << "    (if test ! -z \"`ps x | grep /" << browser << " | grep -v grep`\"; \\" << endl;
+	    of << "\tthen DISPLAY=0:0 " << browser << " -remote 'openURL($url$, new-window)'; \\" << endl;
+	    of << "\telse DISPLAY=0:0 " << browser << " \"$url$\"; \\" << endl;
+	    of << "    fi) >/dev/null 2>&1 &" << endl;
 
 	    of.close();
 	}
     }
 
-    ifstream f(fname.c_str());
+    f.open(fname.c_str());
+
     if(f.is_open()) {
+	openurlcommand = "";
+	cont = false;
+
 	while(!f.eof()) {
 	    getstring(f, buf);
-	    name = getword(buf);
+
+	    if(!cont) {
+		name = getword(buf);
+	    }
 
 	    if(name == "openurl") {
-		openurlcommand = buf;
+		if(!buf.empty()) {
+		    if(cont = buf.substr(buf.size()-1, 1) == "\\")
+			buf.erase(buf.size()-1, 1);
+		}
+
+		openurlcommand += buf;
 	    }
 	}
 
@@ -472,39 +483,6 @@ void icqconf::setregsound(icqconf::regsound s) {
     rs = s;
 }
 
-bool icqconf::gethideoffline() const {
-    return hideoffline;
-}
-
-void icqconf::initpairs() {
-    boldcolors.clear();
-
-    boldcolors.push_back(cp_dialog_selected);
-    boldcolors.push_back(cp_main_highlight);
-    boldcolors.push_back(cp_main_frame);
-    boldcolors.push_back(cp_clist_root);
-
-    init_pair(cp_status, COLOR_BLACK, COLOR_WHITE);
-
-    init_pair(cp_dialog_text, COLOR_BLACK, COLOR_WHITE);
-    init_pair(cp_dialog_menu, COLOR_BLACK, COLOR_WHITE);
-    init_pair(cp_dialog_selected, COLOR_WHITE, COLOR_BLACK);
-    init_pair(cp_dialog_highlight, COLOR_RED, COLOR_WHITE);
-    init_pair(cp_dialog_frame, COLOR_BLUE, COLOR_WHITE);
-
-    init_pair(cp_main_text, COLOR_CYAN, COLOR_BLACK);
-    init_pair(cp_main_menu, COLOR_GREEN, COLOR_BLACK);
-    init_pair(cp_main_selected, COLOR_BLACK, COLOR_WHITE);
-    init_pair(cp_main_highlight, COLOR_YELLOW, COLOR_BLACK);
-    init_pair(cp_main_frame, COLOR_BLUE, COLOR_BLACK);
-
-    init_pair(cp_clist_icq, COLOR_GREEN, COLOR_BLACK);
-    init_pair(cp_clist_msn, COLOR_CYAN, COLOR_BLACK);
-    init_pair(cp_clist_yahoo, COLOR_YELLOW, COLOR_BLACK);
-    init_pair(cp_clist_infocard, COLOR_WHITE, COLOR_BLACK);
-    init_pair(cp_clist_root, COLOR_RED, COLOR_BLACK);
-}
-
 void icqconf::setauto(int away, int na) {
     autoaway = away;
     autona = na;
@@ -521,15 +499,11 @@ void icqconf::getauto(int &away, int &na) const {
 	away = 0;
 }
 
-bool icqconf::getquote() const {
-    return quote;
-}
-
 void icqconf::setquote(bool use) {
     quote = use;
 }
 
-void icqconf::setsockshost(const string nsockshost) {
+void icqconf::setsockshost(const string &nsockshost) {
     int pos;
 
     if(!nsockshost.empty()) {
@@ -545,7 +519,7 @@ void icqconf::setsockshost(const string nsockshost) {
     }
 }
 
-const string icqconf::getsockshost() const {
+string icqconf::getsockshost() const {
     return sockshost;
 }
 
@@ -558,15 +532,11 @@ void icqconf::getsocksuser(string &name, string &pass) const {
     pass = sockspass;
 }
 
-void icqconf::setsocksuser(const string name, const string pass) {
+void icqconf::setsocksuser(const string &name, const string &pass) {
     socksuser = name;
     sockspass = pass;
 }
 
-bool icqconf::getsavepwd() const {
-    return savepwd;
-}
-
 void icqconf::setsavepwd(bool ssave) {
     savepwd = ssave;
 }
@@ -575,23 +545,23 @@ void icqconf::sethideoffline(bool fho) {
     hideoffline = fho;
 }
 
-bool icqconf::getantispam() const {
-    return antispam;
-}
-
 void icqconf::setantispam(bool fas) {
     antispam = fas;
 }
 
-bool icqconf::getmailcheck() const {
-    return mailcheck;
-}
-
 void icqconf::setmailcheck(bool fmc) {
     mailcheck = fmc;
 }
 
-void icqconf::openurl(const string url) {
+void icqconf::setaskaway(bool faskaway) {
+    askaway = faskaway;
+}
+
+void icqconf::setchatmode(bool fchatmode) {
+    chatmode = fchatmode;
+}
+
+void icqconf::openurl(const string &url) {
     int npos;
     string torun = openurlcommand;
 
@@ -601,14 +571,44 @@ void icqconf::openurl(const string url) {
     system(torun.c_str());
 }
 
-const string icqconf::getprotocolname(protocolname pname) const {
+string icqconf::getprotocolname(protocolname pname) const {
     static const string ptextnames[protocolname_size] = {
-	"icq", "yahoo", "msn", "infocard"
+	"icq", "yahoo", "msn", "aim", "irc", "infocard"
     };
 
     return ptextnames[pname];
 }
 
+protocolname icqconf::getprotocolbyletter(char letter) const {
+    switch(letter) {
+	case 'y': return yahoo;
+	case 'a': return aim;
+	case 'i': return irc;
+	case 'm': return msn;
+	case 'n': return infocard;
+	case '0':
+	case '1':
+	case '2':
+	case '3':
+	case '4':
+	case '5':
+	case '6':
+	case '7':
+	case '8':
+	case '9': return icq;
+    }
+
+    return protocolname_size;
+}
+
+string icqconf::getprotocolprefix(protocolname pname) const {
+    static const string pprefixes[protocolname_size] = {
+	"", "y", "m", "a", "i", "n"
+    };
+
+    return pprefixes[pname];
+}
+
 imstatus icqconf::getstatus(protocolname pname) {
     string fname, buf;
     ifstream f;
@@ -641,74 +641,253 @@ void icqconf::savestatus(protocolname pname, imstatus st) {
 
 int icqconf::getprotcolor(protocolname pname) const {
     switch(pname) {
-	case icq:
-	    return getcolor(cp_clist_icq);
-	case yahoo:
-	    return getcolor(cp_clist_yahoo);
-	case infocard:
-	    return getcolor(cp_clist_infocard);
-	case msn:
-	    return getcolor(cp_clist_msn);
-	default:
-	    return getcolor(cp_main_text);
+	case      icq : return getcolor(cp_clist_icq);
+	case    yahoo : return getcolor(cp_clist_yahoo);
+	case infocard : return getcolor(cp_clist_infocard);
+	case      msn : return getcolor(cp_clist_msn);
+	case      aim : return getcolor(cp_clist_aim);
+	case      irc : return getcolor(cp_clist_irc);
+	default       : return getcolor(cp_main_text);
     }
 }
 
-const string icqconf::getdirname() const {
-    return basedir;
-}
-
-const string icqconf::getconfigfname(const string fname) const {
-    return getdirname() + fname;
-}
-
-bool icqconf::getrussian() const {
-    return russian;
-}
-
 void icqconf::setrussian(bool frussian) {
     russian = frussian;
 }
 
 void icqconf::commandline(int argc, char **argv) {
     int i;
+    string event, proto, dest;
+    char st = 0;
+
+    argv0 = argv[0];
 
     for(i = 1; i < argc; i++) {
 	string args = argv[i];
 
 	if((args == "-a") || (args == "--ascii")) {
 	    kintf_graph = 0;
+
 	} else if((args == "-b") || (args == "--basedir")) {
 	    if(argv[++i]) {
 		basedir = argv[i];
 		if(basedir.substr(basedir.size()-1) != "/") basedir += "/";
 	    }
+
+	} else if((args == "-s") || (args == "--send")) {
+	    if(argv[++i]) event = argv[i];
+
+	} else if((args == "-p") || (args == "--proto")) {
+	    if(argv[++i]) proto = argv[i];
+
+	} else if((args == "-t") || (args == "--to")) {
+	    if(argv[++i]) dest = argv[i];
+
+	} else if((args == "-S") || (args == "--status")) {
+	    if(argv[++i])
+	    if(strlen(argv[i]))
+		st = argv[i][0];
+
 	} else {
-	    kendinterface();
-	    cout << "centericq command line parameters:" << endl << endl;
-	    cout << "\t--ascii or -a : use characters for windows and UI controls" << endl;
-	    cout << "\tanything else : display this stuff" << endl;
+	    usage();
 	    exit(0);
 	}
     }
+
+    constructevent(event, proto, dest);
+    externalstatuschange(st, proto);
 }
 
-bool icqconf::getmakelog() const {
-    return makelog;
+void icqconf::constructevent(const string &event, const string &proto, const string &dest) const {
+    imevent *ev = 0;
+    imcontact cdest;
+    string text, buf;
+    int pos;
+
+    if(event.empty() && dest.empty()) return; else
+    if(event.empty() || proto.empty() || dest.empty()) {
+	cout << _("event sending error: not enough parameters") << endl;
+	exit(1);
+    }
+
+    while(getline(cin, buf)) {
+	if(!text.empty()) text += "\n";
+	text += buf;
+    }
+
+    if(proto == "icq") {
+	if(dest.find_first_not_of("0123456789") != -1) {
+	    cout << _("event sending error: only UINs are allowed with icq protocol") << endl;
+	    exit(1);
+	}
+	cdest = imcontact(strtoul(dest.c_str(), 0, 0), icq);
+    } else {
+	protocolname pname;
+
+	for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+	    if(getprotocolname(pname) == proto) {
+		cdest = imcontact(dest, pname);
+		break;
+	    }
+	}
+
+	if(pname == protocolname_size) {
+	    cout << _("event sending error: unknown IM type") << endl;
+	    exit(1);
+	}
+    }
+
+    if(event == "msg") {
+	ev = new immessage(cdest, imevent::outgoing, text);
+    } else if(event == "url") {
+	string url;
+
+	if((pos = text.find("\n")) != -1) {
+	    url = text.substr(0, pos);
+	    text.erase(0, pos+1);
+	} else {
+	    url = text;
+	    text = "";
+	}
+
+	ev = new imurl(cdest, imevent::outgoing, url, text);
+    } else if(event == "sms") {
+	ev = new imsms(cdest, imevent::outgoing, text);
+    } else {
+	cout << _("event sending error: unknown event type") << endl;
+	exit(1);
+    }
+
+    if(ev) {
+	icqcontact *c = new icqcontact(cdest);
+
+	if(!access(c->getdirname().c_str(), X_OK)) {
+	    clist.add(c);
+	    em.store(*ev);
+
+	    char buf[512];
+	    sprintf(buf, _("%s to %s has been put to the queue"),
+		eventnames[ev->gettype()], c->getdesc().totext().c_str());
+
+	    cout << buf << endl;
+	} else {
+	    cout << _("event sending error: the destination contact doesn't exist on your list") << endl;
+	    exit(1);
+	}
+
+	delete c;
+	delete ev;
+
+	exit(0);
+    }
+}
+
+void icqconf::externalstatuschange(char st, const string &proto) const {
+    imstatus imst;
+    protocolname pname;
+    int rpid;
+
+    if(st) {
+	for(pname = icq; pname != protocolname_size; (int) pname += 1)
+	    if(getprotocolname(pname) == proto)
+		break;
+
+	for(imst = offline; imst != imstatus_size; (int) imst += 1)
+	    if(imstatus2char[imst] == st)
+		break;
+
+	try {
+	    if(pname != protocolname_size) {
+		if(imst != imstatus_size) {
+		    ifstream f(conf.getconfigfname("pid").c_str());
+		    if(f.is_open()) {
+			f >> rpid;
+			f.close();
+
+			if(rpid > 0) {
+			    conf.savestatus(pname, imst);
+			    kill(rpid, SIGUSR1);
+			}
+		    }
+		} else {
+		    throw _("unknown status character was given");
+		}
+	    } else {
+		throw _("unknown IM type");
+	    }
+	} catch(char *p) {
+	    cout << _("status change error: ") << p << endl;
+	    exit(1);
+	}
+
+	exit(0);
+    }
+}
+
+void icqconf::usage() const {
+    cout << "Usage: " << argv0 << " [OPTION].." << endl;
+
+    cout << endl << "General options:" << endl;
+    cout << "  --ascii, -a              use characters for windows and UI controls" << endl;
+    cout << "  --help                   display this stuff" << endl;
+
+    cout << endl << "Events sending options:" << endl;
+    cout << "  -s, --send <event type>  event type; can be msg, sms or url" << endl;
+    cout << "  -S, --status <status>    change the current IM status" << endl;
+    cout << "  -p, --proto <protocol>   protocol type; can be icq, yahoo, msn, aim or irc" << endl;
+    cout << "  -t, --to <destination>   destination UIN or nick (depends on protocol)" << endl;
+
+    cout << endl << "Report bugs to <centericq-bugs@konst.org.ua>." << endl;
 }
 
 void icqconf::setmakelog(bool slog) {
     makelog = slog;
 }
 
-icqconf::groupmode icqconf::getgroupmode() const {
-    return fgroupmode;
-}
-
 void icqconf::setgroupmode(icqconf::groupmode amode) {
     fgroupmode = amode;
 }
 
+void icqconf::setsmtphost(const string &asmtphost) {
+    int pos;
+
+    smtphost = "";
+    smtpport = 0;
+
+    if(!asmtphost.empty()) {
+	if((pos = asmtphost.find(":")) != -1) {
+	    smtphost = asmtphost.substr(0, pos);
+	    smtpport = atol(asmtphost.substr(pos+1).c_str());
+	} else {
+	    smtphost = asmtphost;
+	    smtpport = 25;
+	}
+    }
+
+    if(smtphost.empty() || !smtpport) {
+	smtphost = getsmtphost();
+	smtpport = getsmtpport();
+    }
+}
+
+string icqconf::getsmtphost() const {
+    return smtphost.empty() ? "localhost" : smtphost;
+}
+
+unsigned int icqconf::getsmtpport() const {
+    return smtpport ? smtpport : 25;
+}
+
+void icqconf::checkdiskspace() {
+    struct statfs st;
+
+    fenoughdiskspace = true;
+
+    if(!statfs(conf.getdirname().c_str(), &st)) {
+	fenoughdiskspace = ((double) st.f_bavail) * st.f_bsize >= 10240;
+    }
+}
+
 // ----------------------------------------------------------------------------
 
 icqconf::imaccount::imaccount() {
@@ -739,7 +918,7 @@ void icqconf::imaccount::write(ofstream &f) {
     }
 }
 
-void icqconf::imaccount::read(const string spec) {
+void icqconf::imaccount::read(const string &spec) {
     int pos = spec.find("_");
     string spname, buf;
 
