@@ -1,7 +1,7 @@
 /*
 *
 * centericq contact list class
-* $Id: icqcontacts.cc,v 1.28 2002/01/29 18:20:14 konst Exp $
+* $Id: icqcontacts.cc,v 1.36 2002/06/20 15:11:34 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -34,42 +34,25 @@ icqcontacts::icqcontacts() {
 icqcontacts::~icqcontacts() {
 }
 
-icqcontact *icqcontacts::addnew(const imcontact cinfo, bool notinlist = true) {
+icqcontact *icqcontacts::addnew(const imcontact &cinfo, bool notinlist ) {
     icqcontact *c = new icqcontact(cinfo);
-    icqcontact::moreinfo m;
-    icqcontact::basicinfo b;
 
     switch(cinfo.pname) {
 	case icq:
 	    c->setnick(i2str(cinfo.uin));
 	    c->setdispnick(i2str(cinfo.uin));
-	    gethook(icq).sendnewuser(c->getdesc());
 	    break;
 
 	case yahoo:
-	    c->setnick(cinfo.nickname);
-	    c->setdispnick(cinfo.nickname);
-
-	    m.homepage = "http://profiles.yahoo.com/" + cinfo.nickname;
-	    b.email = cinfo.nickname + "@yahoo.com";
-
-	    c->setbasicinfo(b);
-	    c->setmoreinfo(m);
-	    break;
-
 	case msn:
+	case aim:
+	case irc:
 	    c->setnick(cinfo.nickname);
 	    c->setdispnick(cinfo.nickname);
-
-	    b.email = cinfo.nickname;
-	    if(b.email.find("@") == -1) b.email += "@hotmail.com";
-	    m.homepage = "http://members.msn.com/" + b.email;
-
-	    c->setbasicinfo(b);
-	    c->setmoreinfo(m);
 	    break;
     }
 
+    gethook(cinfo.pname).sendnewuser(cinfo);
     c->save();
     add(c);
 
@@ -91,6 +74,7 @@ void icqcontacts::load() {
     FILE *f;
     icqcontact *c;
     imcontact cinfo;
+    protocolname pname;
 
     empty();
 
@@ -101,32 +85,24 @@ void icqcontacts::load() {
 
 	    if(S_ISDIR(st.st_mode)) {
 		c = 0;
+		pname = conf.getprotocolbyletter(ent->d_name[0]);
 
-		switch(ent->d_name[0]) {
-		    case 'n':
-			c = new icqcontact(imcontact(atol(ent->d_name+1),
-			    infocard));
+		switch(pname) {
+		    case icq:
+			c = new icqcontact(imcontact(atol(ent->d_name), pname));
 			break;
-		    case 'y':
-			c = new icqcontact(imcontact(ent->d_name+1, yahoo));
+
+		    case infocard:
+			c = new icqcontact(imcontact(atol(ent->d_name+1), pname));
+			break;
+
+		    case yahoo:
+		    case msn:
+		    case aim:
+		    case irc:
+			c = new icqcontact(imcontact(ent->d_name+1, pname));
 			c->setnick(ent->d_name+1);
 			break;
-		    case 'm':
-			c = new icqcontact(imcontact(ent->d_name+1, msn));
-			c->setnick(ent->d_name+1);
-			break;
-		    case '0':
-		    case '1':
-		    case '2':
-		    case '3':
-		    case '4':
-		    case '5':
-		    case '6':
-		    case '7':
-		    case '8':
-		    case '9':
-			c = new icqcontact(imcontact(atol(ent->d_name), icq));
-			break;
 		}
 
 		if(c) {
@@ -165,12 +141,6 @@ void icqcontacts::checkdefault() {
 		case icq:
 		    addnew(imcontact(17502151, pname), false);
 		    break;
-		case yahoo:
-		    addnew(imcontact("thekonst", pname), false);
-		    break;
-		case msn:
-		    addnew(imcontact("konst@konst.org.ua", pname), false);
-		    break;
 	    }
 	}
     }
@@ -190,7 +160,7 @@ void icqcontacts::save() {
     }
 }
 
-void icqcontacts::remove(const imcontact cinfo) {
+void icqcontacts::remove(const imcontact &cinfo) {
     int i;
     icqcontact *c;
 
@@ -205,7 +175,7 @@ void icqcontacts::remove(const imcontact cinfo) {
     }
 }
 
-icqcontact *icqcontacts::get(const imcontact cinfo) {
+icqcontact *icqcontacts::get(const imcontact &cinfo) {
     int i;
     icqcontact *c;
 
