@@ -1,7 +1,7 @@
 /*
 *
 * centericq user interface class, dialogs related part
-* $Id: icqdialogs.cc,v 1.50 2002/02/07 17:33:55 konst Exp $
+* $Id: icqdialogs.cc,v 1.75 2002/07/03 14:35:20 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -31,16 +31,113 @@
 #include "icqgroups.h"
 #include "abstracthook.h"
 
+bool icqface::sprofmanager(string &name, string &act) {
+    dialogbox db;
+    string buf, tname;
+    bool finished, r;
+    int n, b, i, dx, dy;
+
+    map<string, imsearchparams> profiles;
+    map<string, imsearchparams>::iterator ip;
+
+    string pfname = conf.getconfigfname("search-profiles");
+
+    ifstream f(pfname.c_str());
+
+    if(f.is_open()) {
+	while(!f.eof()) {
+	    getstring(f, buf);
+	    tname = getword(buf, "\t");
+
+	    if(!tname.empty())
+		profiles[tname] = imsearchparams(tname);
+	}
+
+	f.close();
+    }
+
+    dx = (COLS-sizeDlg.width)/2+sizeDlg.width;
+    dy = (LINES-sizeDlg.height)/2+sizeDlg.height;
+
+    db.setwindow(new textwindow(dx-(int) (sizeDlg.width*0.6), dy-(int) (sizeDlg.height*0.6),
+	dx, dy, conf.getcolor(cp_dialog_frame), 0, conf.getcolor(cp_dialog_highlight),
+	_(" Search profiles ")));
+
+    db.setmenu(new verticalmenu(conf.getcolor(cp_dialog_text),
+	conf.getcolor(cp_dialog_selected)));
+
+    db.setbar(new horizontalbar(conf.getcolor(cp_dialog_text),
+	conf.getcolor(cp_dialog_selected),
+	_("Remove"), _("Load"), 0));
+
+    db.addkey(KEY_DC, 0);
+
+    db.addautokeys();
+    db.idle = &dialogidle;
+    verticalmenu &m = *db.getmenu();
+    db.getbar()->item = 1;
+
+    finished = r = false;
+
+    while(!finished) {
+	m.clear();
+
+	for(ip = profiles.begin(); ip != profiles.end(); ip++) {
+	    m.additemf(0, (void *) ip->first.c_str(), " %s: %s",
+		conf.getprotocolname(ip->second.pname).c_str(),
+		ip->first.c_str());
+	}
+
+	finished = !db.open(n, b);
+
+	if(!finished) {
+	    if(!profiles.empty()) {
+		tname = (char *) m.getref(n-1);
+		ip = profiles.find(tname);
+	    } else {
+		ip = profiles.end();
+	    }
+
+	    switch(b) {
+		case 0:
+		    if(ip != profiles.end())
+			profiles.erase(ip);
+		    break;
+
+		case 1:
+		    if(ip != profiles.end()) {
+			finished = r = true;
+			name = tname;
+			act = "load";
+		    }
+		    break;
+	    }
+	}
+    }
+
+    db.close();
+
+    unlink(conf.getconfigfname("search-profiles").c_str());
+
+    for(ip = profiles.begin(); ip != profiles.end(); ip++) {
+	ip->second.save(ip->first);
+    }
+
+    return r;
+}
+
 bool icqface::finddialog(imsearchparams &s) {
     int n, b, i;
     int nuin, ninfo, nloc, nwork, nonl;
-    bool finished, ret;
+    bool finished, ret, proceed;
     dialogbox db;
     vector<protocolname> penabled;
-    vector<protocolname>::iterator ipname;
+    vector<protocolname>::iterator ipname, ipfname;
+    string tname, act;
+    imsearchparams ts;
 
     for(protocolname apname = icq; apname != protocolname_size; (int) apname += 1) {
-	if(gethook(apname).online() || apname == infocard) {
+	if(gethook(apname).logged() || apname == infocard) {
 	    penabled.push_back(apname);
 	}
     }
@@ -50,15 +147,15 @@ bool icqface::finddialog(imsearchparams &s) {
 	return false;
     }
 
-    if((ipname = find(penabled.begin(), penabled.end(),
-    s.pname)) == penabled.end()) {
+    if((ipname = find(penabled.begin(), penabled.end(), s.pname)) == penabled.end()) {
 	ipname = penabled.begin();
+	s.pname = *ipname;
     }
 
     blockmainscreen();
     finished = ret = false;
 
-    db.setwindow(new textwindow(0, 0, DIALOG_WIDTH, DIALOG_HEIGHT,
+    db.setwindow(new textwindow(0, 0, sizeDlg.width, sizeDlg.height,
 	conf.getcolor(cp_dialog_frame), TW_CENTERED,
 	conf.getcolor(cp_dialog_highlight), _(" Find/add user(s) ")));
 
@@ -69,69 +166,111 @@ bool icqface::finddialog(imsearchparams &s) {
 
     db.setbar(new horizontalbar(conf.getcolor(cp_dialog_text),
 	conf.getcolor(cp_dialog_selected),
-	_("cLear"), _("Change"), _("Search/Add"), 0));
+	_("lOad"), _("sAve"), _("cLear"), _("Change"), _("Search/Add"), 0));
 
     db.addautokeys();
     db.idle = &dialogidle;
     treeview &tree = *db.gettree();
-    db.getbar()->item = 1;
+    db.getbar()->item = 3;
 
     while(!finished) {
 	tree.clear();
 
 	i = tree.addnode(_(" Network "));
-	tree.addleaf(i, 0, 1, " " + conf.getprotocolname(*ipname) + " ");
+	tree.addleaf(i, 0, 1, " " + conf.getprotocolname(s.pname) + " ");
 
-	switch(*ipname) {
+	switch(s.pname) {
 	    case icq:
 		i = tree.addnode(_(" UIN "));
 		tree.addleaff(i, 0, 10, " %s ", strint(s.uin));
 
-		i = tree.addnode(_(" Details "));
-		tree.addleaff(i, 0, 11, _(" Nickname : %s "), s.nick.c_str());
-		tree.addleaff(i, 0, 12, _(" E-Mail : %s "), s.email.c_str());
-		tree.addleaff(i, 0, 13, _(" First name : %s "), s.firstname.c_str());
-		tree.addleaff(i, 0, 14, _(" Last name : %s "), s.lastname.c_str());
+		if(!s.uin) {
+		    i = tree.addnode(_(" Details "));
+		    tree.addleaff(i, 0, 11, _(" Nickname : %s "), s.nick.c_str());
+		    tree.addleaff(i, 0, 12, _(" E-Mail : %s "), s.email.c_str());
+		    tree.addleaff(i, 0, 13, _(" First name : %s "), s.firstname.c_str());
+		    tree.addleaff(i, 0, 14, _(" Last name : %s "), s.lastname.c_str());
 
-		tree.addleaff(i, 0, 15, _(" Age range : %s "), stragerange[s.agerange]);
-		tree.addleaff(i, 0, 17, _(" Gender : %s "), strgender[s.gender]);
-		tree.addleaff(i, 0, 18, _(" Language : %s "), strlanguage(s.language));
+		    tree.addleaff(i, 0, 15, _(" Age range : %s "), stragerange[s.agerange]);
+		    tree.addleaff(i, 0, 17, _(" Gender : %s "), strgender[s.gender]);
+		    tree.addleaff(i, 0, 18, _(" Language : %s "),
+			ICQ2000::UserInfoHelpers::getLanguageIDtoString(s.language).c_str());
 
-		i = tree.addnode(_(" Location "));
-		tree.addleaff(i, 0, 19, _(" City : %s "), s.city.c_str());
-		tree.addleaff(i, 0, 20, _(" State : %s "), s.state.c_str());
-		tree.addleaff(i, 0, 21, _(" Country : %s "), strcountry(s.country));
+		    i = tree.addnode(_(" Location "));
+		    tree.addleaff(i, 0, 19, _(" City : %s "), s.city.c_str());
+		    tree.addleaff(i, 0, 20, _(" State : %s "), s.state.c_str());
+		    tree.addleaff(i, 0, 21, _(" Country : %s "),
+			ICQ2000::UserInfoHelpers::getCountryIDtoString(s.country).c_str());
 
-		i = tree.addnode(_(" Work "));
-		tree.addleaff(i, 0, 22, _(" Company : %s "), s.company.c_str());
-		tree.addleaff(i, 0, 23, _(" Department : %s "), s.department.c_str());
-		tree.addleaff(i, 0, 24, _(" Position : %s "), s.position.c_str());
+		    i = tree.addnode(_(" Work "));
+		    tree.addleaff(i, 0, 22, _(" Company : %s "), s.company.c_str());
+		    tree.addleaff(i, 0, 23, _(" Department : %s "), s.department.c_str());
+		    tree.addleaff(i, 0, 24, _(" Position : %s "), s.position.c_str());
 
-		i = tree.addnode(_(" Online only "));
-		tree.addleaff(i, 0, 25, " %s ", stryesno[s.onlineonly]);
+		    i = tree.addnode(_(" Online only "));
+		    tree.addleaff(i, 0, 25, " %s ", stryesno[s.onlineonly]);
+		}
 		break;
 
-	    case yahoo:
-	    case msn:
+	    default:
 		i = tree.addnode(_(" Nickname "));
 		tree.addleaf(i, 0, 11, " " + s.nick + " ");
 		break;
 	}
 
+	if(s.pname == irc && s.nick.empty()) {
+	    i = tree.addnode(_(" Details "));
+	    tree.addleaff(i, 0, 26, _(" Channel : %s "), s.room.c_str());
+
+	    if(!s.room.empty()) {
+		tree.addleaff(i, 0, 27, _(" Name : %s "), s.firstname.c_str());
+	    }
+
+	    tree.addleaff(i, 0, 12, _(" E-Mail : %s "), s.email.c_str());
+	}
+
 	finished = !db.open(n, b, (void **) &i);
 
 	if(!finished)
 	switch(b) {
 	    case 0:
-		s = imsearchparams();
+		if(sprofmanager(tname, act)) {
+		    if(act == "load") {
+			imsearchparams ts(tname);
+
+			if((ipfname = find(penabled.begin(), penabled.end(),
+			ts.pname)) != penabled.end()) {
+			    s = ts;
+			    ipname = ipfname;
+			}
+		    }
+		}
 		break;
+
 	    case 1:
+		tname = inputstr(_("New profile name: "));
+
+		if(proceed = !tname.empty())
+		if(ts.load(tname))
+		    proceed = ask(_("The profile with this name already exists. Do you want to overwrite it?"),
+			ASK_YES | ASK_NO, ASK_NO) == ASK_YES;
+
+		if(proceed)
+		    s.save(tname); 
+		break;
+
+	    case 2:
+		s = imsearchparams();
+		s.pname = *ipname;
+		break;
+
+	    case 3:
 		switch(i) {
 		    case 1:
-			if(++ipname == penabled.end()) {
-			    ipname = penabled.begin();
-			}
+			if(++ipname == penabled.end()) ipname = penabled.begin();
+			s.pname = *ipname;
 			break;
+
 		    case 10: s.uin = atol(inputstr(_("UIN: "), strint(s.uin)).c_str()); break;
 		    case 11: s.nick = inputstr(_("Nickname: "), s.nick); break;
 		    case 12: s.email = inputstr(_("E-Mail: "), s.email); break;
@@ -147,12 +286,13 @@ bool icqface::finddialog(imsearchparams &s) {
 		    case 23: s.department = inputstr(_("Department: "), s.department); break;
 		    case 24: s.position = inputstr(_("Position: "), s.position); break;
 		    case 25: s.onlineonly = !s.onlineonly; break;
+		    case 26: s.room = inputstr(_("Channel: "), s.room); break;
+		    case 27: s.firstname = inputstr(_("Name: "), s.firstname); break;
 		}
 		break;
 
-	    case 2:
+	    case 4:
 		ret = finished = true;
-		s.pname = *ipname;
 		break;
 	}
     }
@@ -163,7 +303,7 @@ bool icqface::finddialog(imsearchparams &s) {
     return ret;
 }
 
-void icqface::gendetails(treeview *tree, icqcontact *c = 0) {
+void icqface::gendetails(treeview *tree, icqcontact *c) {
     int saveitem, savefirst, i;
 
     if(!c) c = clist.get(contactroot);
@@ -172,54 +312,70 @@ void icqface::gendetails(treeview *tree, icqcontact *c = 0) {
     icqcontact::moreinfo mi = c->getmoreinfo();
     icqcontact::workinfo wi = c->getworkinfo();
     string about = c->getabout();
+    bool ourdetails = c->getdesc() == contactroot;
 
     tree->menu.getpos(saveitem, savefirst);
     tree->clear();
 
-    i = tree->addnode(_(" General "));
+    i = 0;
+
+    if(passinfo.pname == infocard || (gethook(passinfo.pname).getcapabilities() & hoptCanChangeNick) || !ourdetails) {
+	if(!i) i = tree->addnode(_(" General "));
 
-    if(c->getdesc().pname == infocard)
 	tree->addleaff(i, 0, 10, _(" Nickname : %s "), c->getnick().c_str());
 
-    tree->addleaff(i, 0, 11, _(" First name : %s "), bi.fname.c_str());
-    tree->addleaff(i, 0, 12, _(" Last name : %s "), bi.lname.c_str());
-    tree->addleaff(i, 0, 13, _(" E-mail : %s "), bi.email.c_str());
-    tree->addleaff(i, 0, 14, _(" Gender : %s "), strgender[mi.gender]);
-    tree->addleaff(i, 0, 15, _(" Birthdate : %s "), mi.strbirthdate().c_str());
-    tree->addleaff(i, 0, 16, _(" Age : %s "), strint(mi.age));
+    }
 
-    i = tree->addnode(_(" Home "));
-    tree->addleaff(i, 0, 17, _(" City : %s "), bi.city.c_str());
-    tree->addleaff(i, 0, 18, _(" State : %s "), bi.state.c_str());
-    tree->addleaff(i, 0, 19, _(" Country : %s "), strcountry(bi.country));
-    tree->addleaff(i, 0, 20, _(" Street address : %s "), bi.street.c_str());
-    tree->addleaff(i, 0, 21, _(" Zip code : %s "), bi.zip.c_str());
-    tree->addleaff(i, 0, 22, _(" Phone : %s "), bi.phone.c_str());
-    tree->addleaff(i, 0, 23, _(" Fax : %s "), bi.fax.c_str());
-    tree->addleaff(i, 0, 24, _(" Cellular phone : %s "), bi.cellular.c_str());
+    if(passinfo.pname != aim && c->getdesc() == contactroot || !ourdetails) {
+	if(!i) i = tree->addnode(_(" General "));
 
-    i = tree->addnode(_(" Work "));
-    tree->addleaff(i, 0, 25, _(" City : %s "), wi.city.c_str());
-    tree->addleaff(i, 0, 26, _(" State : %s "), wi.state.c_str());
-    tree->addleaff(i, 0, 27, _(" Country : %s "), strcountry(wi.country));
-    tree->addleaff(i, 0, 28, _(" Street address : %s "), wi.street.c_str());
-    tree->addleaff(i, 0, 29, _(" Company : %s "), wi.company.c_str());
-    tree->addleaff(i, 0, 30, _(" Department : %s "), wi.dept.c_str());
-    tree->addleaff(i, 0, 31, _(" Position : %s "), wi.position.c_str());
-    tree->addleaff(i, 0, 32, _(" Homepage : %s "), wi.homepage.c_str());
-    tree->addleaff(i, 0, 33, _(" Phone : %s "), wi.phone.c_str());
-    tree->addleaff(i, 0, 34, _(" Fax : %s "), wi.fax.c_str());
+	tree->addleaff(i, 0, 11, _(" First name : %s "), bi.fname.c_str());
+	tree->addleaff(i, 0, 12, _(" Last name : %s "), bi.lname.c_str());
+	tree->addleaff(i, 0, 13, _(" E-mail : %s "), bi.email.c_str());
+	tree->addleaff(i, 0, 14, _(" Gender : %s "), strgender[mi.gender]);
+	tree->addleaff(i, 0, 15, _(" Birthdate : %s "), mi.strbirthdate().c_str());
+	tree->addleaff(i, 0, 16, _(" Age : %s "), strint(mi.age));
 
-    i = tree->addnode(_(" More "));
-    tree->addleaff(i, 0, 35, _(" Homepage : %s "), mi.homepage.c_str());
-    tree->addleaff(i, 0, 36, _(" 1st language : %s "), strlanguage(mi.lang1));
-    tree->addleaff(i, 0, 37, _(" 2nd language : %s "), strlanguage(mi.lang2));
-    tree->addleaff(i, 0, 38, _(" 3rd language : %s "), strlanguage(mi.lang3));
+	i = tree->addnode(_(" Home "));
+	tree->addleaff(i, 0, 17, _(" City : %s "), bi.city.c_str());
+	tree->addleaff(i, 0, 18, _(" State : %s "), bi.state.c_str());
+	tree->addleaff(i, 0, 19, _(" Country : %s "),
+	    ICQ2000::UserInfoHelpers::getCountryIDtoString(bi.country).c_str());
 
-    if(c->getdesc().pname == icq)
-    if(c->getdesc() == contactroot)
-	tree->addleaff(i, 0, 40, _(" Require authorization : %s "),
-	    stryesno[bi.requiresauth]);
+	tree->addleaff(i, 0, 20, _(" Street address : %s "), bi.street.c_str());
+	tree->addleaff(i, 0, 21, _(" Zip code : %s "), bi.zip.c_str());
+	tree->addleaff(i, 0, 22, _(" Phone : %s "), bi.phone.c_str());
+	tree->addleaff(i, 0, 23, _(" Fax : %s "), bi.fax.c_str());
+	tree->addleaff(i, 0, 24, _(" Cellular phone : %s "), bi.cellular.c_str());
+
+	i = tree->addnode(_(" Work "));
+	tree->addleaff(i, 0, 25, _(" City : %s "), wi.city.c_str());
+	tree->addleaff(i, 0, 26, _(" State : %s "), wi.state.c_str());
+	tree->addleaff(i, 0, 27, _(" Country : %s "),
+	    ICQ2000::UserInfoHelpers::getCountryIDtoString(wi.country).c_str());
+
+	tree->addleaff(i, 0, 28, _(" Street address : %s "), wi.street.c_str());
+	tree->addleaff(i, 0, 41, _(" Zip code : %s "), wi.zip.c_str());
+	tree->addleaff(i, 0, 29, _(" Company : %s "), wi.company.c_str());
+	tree->addleaff(i, 0, 30, _(" Department : %s "), wi.dept.c_str());
+	tree->addleaff(i, 0, 31, _(" Position : %s "), wi.position.c_str());
+	tree->addleaff(i, 0, 32, _(" Homepage : %s "), wi.homepage.c_str());
+	tree->addleaff(i, 0, 33, _(" Phone : %s "), wi.phone.c_str());
+	tree->addleaff(i, 0, 34, _(" Fax : %s "), wi.fax.c_str());
+
+	i = tree->addnode(_(" More "));
+	tree->addleaff(i, 0, 35, _(" Homepage : %s "), mi.homepage.c_str());
+
+	tree->addleaff(i, 0, 36, _(" 1st language : %s "),
+	    ICQ2000::UserInfoHelpers::getLanguageIDtoString(mi.lang1).c_str());
+
+	tree->addleaff(i, 0, 37, _(" 2nd language : %s "),
+	    ICQ2000::UserInfoHelpers::getLanguageIDtoString(mi.lang2).c_str());
+
+	tree->addleaff(i, 0, 38, _(" 3rd language : %s "),
+	    ICQ2000::UserInfoHelpers::getLanguageIDtoString(mi.lang3).c_str());
+
+    }
 
     i = tree->addnode(_(" About "));
     tree->addleaff(i, 0, 39, " %s ", about.c_str());
@@ -227,7 +383,7 @@ void icqface::gendetails(treeview *tree, icqcontact *c = 0) {
     tree->menu.setpos(saveitem, savefirst);
 }
 
-bool icqface::updatedetails(icqcontact *c = 0) {
+bool icqface::updatedetails(icqcontact *c, protocolname upname) {
     string about, tmp;
     icqcontact::basicinfo bi;
     icqcontact::workinfo wi;
@@ -239,23 +395,27 @@ bool icqface::updatedetails(icqcontact *c = 0) {
     if(!c) {
 	status(_("Fetching your details"));
 	c = clist.get(contactroot);
-//        if(mainscreenblock) return false;
-	    // Another dialog is already on top
     }
 
     if(!(msb = mainscreenblock)) {
 	blockmainscreen();
     }
 
-    textwindow w(0, 0, DIALOG_WIDTH, DIALOG_HEIGHT, conf.getcolor(cp_dialog_frame), TW_CENTERED);
+    passinfo.pname = upname;
 
-    if(!c->getdesc().uin) {
-	w.set_title(conf.getcolor(cp_dialog_highlight), _(" Your details "));
+    textwindow w(0, 0, sizeDlg.width, sizeDlg.height,
+	conf.getcolor(cp_dialog_frame), TW_CENTERED);
+
+    if(c->getdesc() == contactroot) {
+	w.set_titlef(conf.getcolor(cp_dialog_highlight), _(" Your %s details "),
+	    conf.getprotocolname(upname).c_str());
     } else {
 	w.set_titlef(conf.getcolor(cp_dialog_highlight), _(" %s: details "),
 	    c->getdesc().totext().c_str());
     }
 
+    detailsfetched = (c->getdesc() != contactroot);
+
     db.setwindow(&w, false);
 
     db.settree(new treeview(conf.getcolor(cp_dialog_text),
@@ -336,6 +496,7 @@ bool icqface::updatedetails(icqcontact *c = 0) {
 		case 26: wi.state = inputstr(_("State: "), wi.state); break;
 		case 27: selectcountry(wi.country); break;
 		case 28: wi.street = inputstr(_("Street address: "), wi.street); break;
+		case 41: wi.zip = inputstr(_("Zip code: "), wi.zip); break;
 		case 29: wi.company = inputstr(_("Company: "), wi.company); break;
 		case 30: wi.dept = inputstr(_("Department: "), wi.dept); break;
 		case 31: wi.position = inputstr(_("Position: "), wi.position); break;
@@ -444,9 +605,12 @@ void icqface::selectagerange(ICQ2000::AgeRange &r) {
     if(i) r = (ICQ2000::AgeRange) ((int) m.getref(i-1));
 }
 
-bool icqface::edit(string &txt, const string header) {
+bool icqface::edit(string &txt, const string &header) {
+    bool msb = mainscreenblock;
+    if(!msb) blockmainscreen();
+
     texteditor se;
-    textwindow w(0, 0, DIALOG_WIDTH, DIALOG_HEIGHT, conf.getcolor(cp_dialog_frame), TW_CENTERED);
+    textwindow w(0, 0, sizeDlg.width, sizeDlg.height, conf.getcolor(cp_dialog_frame), TW_CENTERED);
     w.set_title(conf.getcolor(cp_dialog_highlight), (string) " " + header + _(" [Ctrl-X save, Esc cancel] "));
     w.open();
 
@@ -467,10 +631,12 @@ bool icqface::edit(string &txt, const string header) {
     }
 
     w.close();
+    if(!msb) unblockmainscreen();
+
     return editdone;
 }
 
-bool icqface::sendfiles(const imcontact cinfo, string &msg, linkedlist &flist) {
+bool icqface::sendfiles(const imcontact &cinfo, string &msg, linkedlist &flist) {
     int n, i, b;
     bool finished = false;
     string fname;
@@ -481,7 +647,7 @@ bool icqface::sendfiles(const imcontact cinfo, string &msg, linkedlist &flist) {
 
     blockmainscreen();
 
-    textwindow w(0, 0, DIALOG_WIDTH, DIALOG_HEIGHT, conf.getcolor(cp_dialog_frame), TW_CENTERED);
+    textwindow w(0, 0, sizeDlg.width, sizeDlg.height, conf.getcolor(cp_dialog_frame), TW_CENTERED);
 
     w.set_titlef(conf.getcolor(cp_dialog_highlight),
 	_(" Send file(s) to %s, %lu "),
@@ -544,6 +710,7 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
     int nopt, n, i, b, nproxy, nconf, ncomm, aaway, ana, noth, nfeat, ncl;
     string tmp, phidden, socksuser, sockspass;
     string prserv = conf.getsockshost() + ":" + i2str(conf.getsocksport());
+    string smtp = conf.getsmtphost() + ":" + i2str(conf.getsmtpport());
 
     bool quote = conf.getquote();
     bool rus = conf.getrussian();
@@ -553,16 +720,22 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
     bool antispam = conf.getantispam();
     bool mailcheck = conf.getmailcheck();
     bool makelog = conf.getmakelog();
+    bool askaway = conf.getaskaway();
+    bool chatmode = conf.getchatmode();
 
     icqconf::groupmode gmode = conf.getgroupmode();
 
     dialogbox db;
 
     finished = success = false;
+
     if(conf.getsockshost().empty()) prserv = "";
+
     conf.getauto(aaway, ana);
 
-    textwindow w(0, 0, BIGDIALOG_WIDTH, BIGDIALOG_HEIGHT, conf.getcolor(cp_dialog_frame), TW_CENTERED);
+    textwindow w(0, 0, sizeBigDlg.width, sizeBigDlg.height,
+	conf.getcolor(cp_dialog_frame), TW_CENTERED);
+
     w.set_title(conf.getcolor(cp_dialog_highlight), _(" CenterICQ configuration "));
 
     db.setwindow(&w, false);
@@ -595,8 +768,13 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 	t.addleaff(i, 0,  8, _(" Quote a message on reply : %s "), stryesno[quote]);
 	t.addleaff(i, 0, 15, _(" Check the local mailbox : %s "), stryesno[mailcheck]);
 	t.addleaff(i, 0, 13, _(" Remember passwords : %s "), stryesno[savepwd]);
-/*
+	t.addleaff(i, 0,  7, _(" Edit away message on status change : %s "), stryesno[askaway]);
+	t.addleaff(i, 0, 16, _(" Chat messaging mode : %s "), stryesno[chatmode]);
+
 	i = t.addnode(_(" Communications "));
+	t.addleaff(i, 0, 19, _(" SMTP server : %s "), smtp.c_str());
+
+/*
 	t.addleaff(i, 0, 9, _(" Use SOCKS proxy : %s "), stryesno[socks]);
 */
 	i = t.addnode(_(" Miscellaneous "));
@@ -641,6 +819,7 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 			if(tmp.size()) ana = atol(tmp.c_str());
 			break;
 		    case 6: hideoffl = !hideoffl; break;
+		    case 7: askaway = !askaway; break;
 		    case 8: quote = !quote; break;
 		    case 9: socks = !socks; break;
 		    case 10:
@@ -658,6 +837,7 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 		    case 13: savepwd = !savepwd; break;
 		    case 14: antispam = !antispam; break;
 		    case 15: mailcheck = !mailcheck; break;
+		    case 16: chatmode = !chatmode; break;
 		    case 17:
 			gmode =
 			    gmode == icqconf::group1 ? icqconf::group2 :
@@ -665,6 +845,10 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 				icqconf::group1;
 			break;
 		    case 18: makelog = !makelog; break;
+		    case 19:
+			tmp = inputstr(_("SMTP server hostname: "), smtp);
+			if(!tmp.empty()) smtp = tmp;
+			break;
 		}
 		break;
 	    case 1:
@@ -677,6 +861,8 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 		conf.setmailcheck(mailcheck);
 		conf.setrussian(rus);
 		conf.setmakelog(makelog);
+		conf.setaskaway(askaway);
+		conf.setchatmode(chatmode);
 
 		if(conf.getgroupmode() != gmode) {
 		    conf.setgroupmode(gmode);
@@ -684,6 +870,7 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 		}
 
 		conf.setsockshost(socks ? prserv : "");
+		conf.setsmtphost(smtp);
 		break;
 	}
     }
@@ -706,16 +893,19 @@ int icqface::editaboutkeys(texteditor &e, int k) {
 void icqface::detailsidle(dialogbox &db) {
     icqcontact *c;
 
-    if(!cicq.idle(HIDL_SOCKEXIT))
+    if(!face.detailsfetched)
     if(c = clist.get(contactroot))
     if(c->updated()) {
 	face.gendetails(db.gettree());
 	db.gettree()->redraw();
 	face.status(_("Your details have been fetched"));
+	face.detailsfetched = true;
     }
+
+    cicq.idle(HIDL_SOCKEXIT);
 }
 
-int icqface::selectgroup(const string text) {
+int icqface::selectgroup(const string &text) {
     return groupmanager(text, true);
 }
 
@@ -723,14 +913,14 @@ void icqface::organizegroups() {
     groupmanager(_("Organize contact groups"), false);
 }
 
-int icqface::groupmanager(const string text, bool sel) {
+int icqface::groupmanager(const string &text, bool sel) {
     dialogbox db;
     static int n = 0;
     int r, ngrp, b, id;
     vector<icqgroup>::iterator i;
     string gname;
 
-    textwindow w(0, 0, DIALOG_WIDTH, DIALOG_HEIGHT,
+    textwindow w(0, 0, sizeDlg.width, sizeDlg.height,
 	conf.getcolor(cp_dialog_frame), TW_CENTERED);
 
     w.set_title(conf.getcolor(cp_dialog_highlight), " " + text + " ");
@@ -743,10 +933,11 @@ int icqface::groupmanager(const string text, bool sel) {
 
     db.setbar(new horizontalbar(conf.getcolor(cp_dialog_text),
 	conf.getcolor(cp_dialog_selected),
-	_("Add"), _("Rename"), _("Remove"), sel ? _("Select") : _("Done"), 0));
+	_("Add"), _("Rename"), _("rEmove"), _("move Up"), _("move Down"),
+	sel ? _("Select") : _("Done"), 0));
 
     db.addautokeys();
-    db.getbar()->item = 3;
+    db.getbar()->item = 5;
 
     r = 0;
     db.idle = &dialogidle;
@@ -760,6 +951,8 @@ int icqface::groupmanager(const string text, bool sel) {
 	t.clear();
 	ngrp = t.addnode(_(" Groups "));
 
+	sort(groups.begin(), groups.end());
+
 	for(i = groups.begin(); i != groups.end(); i++) {
 	    b = i-groups.begin()+1;
 	    id = t.addleaff(ngrp, 0, b, " %s ", i->getname().c_str());
@@ -799,6 +992,22 @@ int icqface::groupmanager(const string text, bool sel) {
 		break;
 
 	    case 3:
+		if(n) {
+		    i = groups.begin()+n-1;
+		    i->moveup();
+		    n--;
+		}
+		break;
+
+	    case 4:
+		if(n) {
+		    i = groups.begin()+n-1;
+		    i->movedown();
+		    n++;
+		}
+		break;
+
+	    case 5:
 		if(n) {
 		    i = groups.begin()+n-1;
 		    r = i->getid();
