@@ -14,18 +14,7 @@
 #include "icqmlist.h"
 #include "imcontroller.h"
 
-#include <libicq2000/userinfoconstants.h>
-
-#define DIALOG_WIDTH    60
-#define DIALOG_HEIGHT   15
-
-#define BIGDIALOG_WIDTH    70
-#define BIGDIALOG_HEIGHT   18
-
-#define WORKAREA_X1     25
-#define WORKAREA_X2     (COLS-1)
-#define WORKAREA_Y1     1
-#define WORKAREA_Y2     (LINES-6)
+#include <libicq2000/userinfohelpers.h>
 
 #define ASK_YES         2
 #define ASK_NO          4
@@ -58,13 +47,14 @@
 #define ACT_ORG_GROUPS  -35
 #define ACT_HIDEOFFLINE -36
 #define ACT_FETCHAWAY   -37
+#define ACT_EMAIL       -38
 
 extern class centericq cicq;
 
 class icqface {
     public:
 	enum eventviewresult {
-	    cancel, forward, reply, ok, open,
+	    ok, next, cancel, forward, reply, open,
 	    accept, reject, eventviewresult_size
 	};
 
@@ -78,10 +68,13 @@ class icqface {
 		~icqprogress();
 
 		void log(const char *fmt, ...);
-		void show(string title = "");
+		void show(const string &title = "");
 		void hide();
 	} progress;
 
+	struct { int x1, y1, x2, y2; } sizeWArea;
+	struct { int width, height; } sizeDlg, sizeBigDlg;
+
     protected:
 	treeview *mcontacts;
 	textinputline input;
@@ -90,14 +83,17 @@ class icqface {
 	verticalmenu mhist;
 
 	ofstream flog;
-	vector<string> extractedurls;
+	vector<string> extractedurls, lastlog;
+	string fstatus;
 
 	linkedlist workareas;
 
 	bool editdone, mainscreenblock, inited, onlinefolder,
-	    dotermresize, fneedupdate;
+	    dotermresize, fneedupdate, detailsfetched, inchat;
 
-	int extk;
+	int extk, chatlines;
+
+	time_t chatlastread;
 
 	imcontact passinfo;
 
@@ -108,12 +104,14 @@ class icqface {
 	static int multiplekeys(verticalmenu &m, int k);
 	static int historykeys(dialogbox &m, int k);
 	static int userinfokeys(dialogbox &db, int k);
+	static int findreskeys(dialogbox &db, int k);
 
 	static void editidle(texteditor &e);
 	static void textbrowseridle(textbrowser &b);
 	static void textinputidle(textinputline &il);
 	static void freeworkareabuf(void *p);
 	static void detailsidle(dialogbox &db);
+	static void editchatidle(texteditor &e);
 
 	static void termresize(void);
 
@@ -137,9 +135,12 @@ class icqface {
 	void workarealine(int l, chtype c = HLINE);
 
 	void showextractedurls();
-	void extracturls(const string buf);
+	void extracturls(const string &buf);
 
-	int groupmanager(const string text, bool sel);
+	int groupmanager(const string &text, bool sel);
+
+	void renderchathistory();
+	bool sprofmanager(string &name, string &act);
 
     public:
 	vector<imcontact> muins;
@@ -163,10 +164,10 @@ class icqface {
 	icqcontact *mainloop(int &action);
 	void fillcontactlist();
 
-	void log(const string text);
+	void log(const string &text);
 	void log(const char *fmt, ...);
 
-	void status(const string text);
+	void status(const string &text);
 
 	bool changestatus(protocolname &pname, imstatus &st);
 	int contextmenu(icqcontact *c);
@@ -174,29 +175,30 @@ class icqface {
 
 	void modelist(contactstatus cs);
 
-	bool multicontacts(const string head = "");
-	void userinfo(const imcontact cinfo, const imcontact realinfo);
+	bool multicontacts(const string &head = "");
+	void userinfo(const imcontact &cinfo, const imcontact &realinfo);
 
 	bool updateconf(icqconf::regsound &s, icqconf::regcolor &c);
 
 	bool finddialog(imsearchparams &s);
 	bool findresults(const imsearchparams &sp);
+	void findready();
 
-	bool updatedetails(icqcontact *c = 0);
-	bool sendfiles(const imcontact cinfo, string &msg, linkedlist &flist);
+	bool updatedetails(icqcontact *c = 0, protocolname upname = icq);
+	bool sendfiles(const imcontact &cinfo, string &msg, linkedlist &flist);
 
 	void blockmainscreen();
 	void unblockmainscreen();
 
 	int ask(string q, int options, int deflt = -1);
 
-	const string inputstr(const string q, const string defl = "", char passwdchar = 0);
-	const string inputfile(const string q, const string defl = "");
+	string inputstr(const string &q, const string &defl = "", char passwdchar = 0);
+	string inputfile(const string &q, const string &defl = "");
 	int getlastinputkey() const;
 
 	void quickfind(verticalmenu *multi = 0);
 
-	int selectgroup(const string text);
+	int selectgroup(const string &text);
 	void organizegroups();
 	void makeprotocolmenu(verticalmenu &m);
 
@@ -204,9 +206,12 @@ class icqface {
 	bool histexec(imevent *&im);
 
 	bool eventedit(imevent &ev);
-	eventviewresult eventview(const imevent *ev);
 
-	bool edit(string &txt, const string header);
+	eventviewresult eventview(const imevent *ev,
+	    vector<eventviewresult> abuttons = vector<eventviewresult>());
+
+	bool edit(string &txt, const string &header);
+	void chat(const imcontact &ic);
 };
 
 extern icqface face;
@@ -216,8 +221,6 @@ string getbdate(unsigned char fbday, unsigned char fbmonth, unsigned char fbyear
 const char *strregsound(icqconf::regsound s);
 const char *strregcolor(icqconf::regcolor c);
 const char *strint(unsigned int i);
-const char *strcountry(unsigned int code);
-const char *strlanguage(unsigned int code);
 const char *strgroupmode(icqconf::groupmode gmode);
 
 static const char *stryesno[true+1] = {
@@ -241,8 +244,10 @@ static const char *stragerange[ICQ2000::range_60_above+1] = {
 };
 
 static const char *eventviewresultnames[icqface::eventviewresult_size] = {
-    "", _("Fwd"), _("Reply"), _("Ok"),
-    _("Open"), _("Accept"), _("Reject")
+    _("Ok"), _("Next"), "",
+    _("Fwd"), _("Reply"),
+    _("Open"), _("Accept"),
+    _("Reject")
 };
 
 #endif
