@@ -7,7 +7,7 @@ icqgroups::icqgroups() {
 icqgroups::~icqgroups() {
 }
 
-const string icqgroups::getfname() const {
+string icqgroups::getfname() const {
     return conf.getdirname() + "groups";
 }
 
@@ -15,6 +15,7 @@ void icqgroups::load() {
     string buf;
     ifstream f;
     int gid;
+    vector<icqgroup>::iterator ig;
 
     clear();
     f.open(getfname().c_str());
@@ -24,8 +25,13 @@ void icqgroups::load() {
 	    getstring(f, buf);
 	    gid = atol(getword(buf).c_str());
 
-	    if(gid && !buf.empty()) {
-		push_back(icqgroup(gid, buf));
+	    if(gid) {
+		if(!buf.empty()) {
+		    push_back(icqgroup(gid, buf));
+		} else {
+		    ig = find(begin(), end(), gid);
+		    if(ig != end()) ig->changecollapsed();
+		}
 	    }
 	}
 
@@ -41,18 +47,21 @@ void icqgroups::save() {
     ofstream f;
     iterator i;
 
-    f.open(getfname().c_str());
+    if(conf.enoughdiskspace()) {
+	f.open(getfname().c_str());
 
-    if(f.is_open()) {
-	for(i = begin(); i != end(); i++) {
-	    f << i->getid() << "\t" << i->getname() << endl;
+	if(f.is_open()) {
+	    for(i = begin(); i != end(); i++) {
+		f << i->getid() << "\t" << i->getname() << endl;
+	    }
+            for(i = begin(); i != end(); i++)
+                if(i->iscollapsed()) f << i->getid() << endl;
+	    f.close();
 	}
-
-	f.close();
     }
 }
 
-int icqgroups::add(const string aname) {
+int icqgroups::add(const string &aname) {
     int i;
 
     for(i = 1; find(begin(), end(), i) != end(); i++);
