@@ -9,18 +9,17 @@ class icqgroups: private vector<icqgroup> {
 	icqgroups();
 	~icqgroups();
 
-	const string getfname() const;
+	string getfname() const;
 
 	void load();
 	void save();
 
-	int add(const string aname);
+	int add(const string &aname);
 	void remove(int gid);
 
-	iterator begin()
-	    { return vector<icqgroup>::begin(); }
-	iterator end()
-	    { return vector<icqgroup>::end(); }
+	int size() const { return vector<icqgroup>::size(); }
+	iterator begin() { return vector<icqgroup>::begin(); }
+	iterator end() { return vector<icqgroup>::end(); }
 };
 
 extern icqgroups groups;
