@@ -1,7 +1,7 @@
 /*
 *
 * centericq user mode list class
-* $Id: icqmlist.cc,v 1.9 2001/12/06 16:56:33 konst Exp $
+* $Id: icqmlist.cc,v 1.14 2002/04/11 17:14:35 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -24,6 +24,7 @@
 
 #include "icqmlist.h"
 #include "icqconf.h"
+#include "abstracthook.h"
 
 icqlist::icqlist() {
 }
@@ -31,12 +32,13 @@ icqlist::icqlist() {
 icqlist::~icqlist() {
 }
 
-const string icqlist::getfname() const {
-    return conf.getdirname() + "modelist";
+string icqlist::getfname() const {
+    return conf.getconfigfname("modelist");
 }
 	
 void icqlist::load() {
     string buf, tok, nick;
+    protocolname pname;
     ifstream f;
     int i;
 
@@ -50,13 +52,23 @@ void icqlist::load() {
 	    if(!tok.empty())
 	    if(i = atoi(tok.c_str())) {
 		nick = unmime(getword(buf));
-		if(!nick.empty()) {
-		    if(buf == "y") {
-			lst.push_back(modelistitem(nick,
-			    imcontact(nick, yahoo), (contactstatus) i));
-		    } else {
-			lst.push_back(modelistitem(nick,
-			    imcontact(strtoul(buf.c_str(), 0, 0), icq), (contactstatus) i));
+
+		if(!nick.empty() && !buf.empty()) {
+		    pname = conf.getprotocolbyletter(buf[0]);
+
+		    switch(pname) {
+			case yahoo:
+			case msn:
+			case aim:
+			case irc:
+			    lst.push_back(modelistitem(nick, imcontact(nick,
+				pname), (contactstatus) i));
+			    break;
+
+			case icq:
+			    lst.push_back(modelistitem(nick, imcontact(strtoul(buf.c_str(),
+				0, 0), icq), (contactstatus) i));
+			    break;
 		    }
 		}
 	    }
@@ -69,39 +81,42 @@ void icqlist::save() {
     vector<modelistitem>::iterator i;
     ofstream f;
 
-    f.open(getfname().c_str());
+    if(conf.enoughdiskspace()) {
+	f.open(getfname().c_str());
 
-    if(f.is_open()) {
-	for(i = begin(); i != end(); i++) {
-	    f << (int) i->getstatus() << "\t";
-	    f << mime(i->getnick()) << "\t";
+	if(f.is_open()) {
+	    for(i = begin(); i != end(); i++) {
+		f << (int) i->getstatus() << "\t";
+		f << mime(i->getnick()) << "\t";
+		f << conf.getprotocolprefix(i->getdesc().pname);
 
-	    switch(i->getdesc().pname) {
-		case icq: f << i->getdesc().uin; break;
-		case yahoo: f << "y"; break;
+		if(i->getdesc().uin)
+		    f << i->getdesc().uin;
+
+		f << endl;
 	    }
 
-	    f << endl;
+	    f.close();
 	}
-
-	f.close();
     }
 }
 
 void icqlist::fillmenu(verticalmenu *m, contactstatus ncs) {
+    int capab;
     vector<modelistitem>::iterator i;
+
     m->clear();
     menucontents.clear();
 
     for(i = begin(); i != end(); i++) {
 	if(i->getstatus() == ncs) {
-	    m->additem(" " + i->getnick());
+	    m->additem(conf.getprotcolor(i->getdesc().pname), 0, " " + i->getnick());
 	    menucontents.push_back(*i);
 	}
     }
 }
 
-bool icqlist::inlist(const imcontact cinfo, contactstatus ncs) const {
+bool icqlist::inlist(const imcontact &cinfo, contactstatus ncs) const {
     vector<modelistitem>::const_iterator i;
 
     i = begin();
@@ -113,7 +128,7 @@ bool icqlist::inlist(const imcontact cinfo, contactstatus ncs) const {
     return false;
 }
 
-void icqlist::del(const imcontact cinfo, contactstatus ncs) {
+void icqlist::del(const imcontact &cinfo, contactstatus ncs) {
     vector<modelistitem>::iterator i;
 
     i = begin();
@@ -132,7 +147,7 @@ modelistitem icqlist::menuat(int pos) const {
 
 // ----------------------------------------------------------------------------
 
-const string modelistitem::getnick() const {
+string modelistitem::getnick() const {
     return nick;
 }
 
