@@ -1,4 +1,5 @@
 #include "imcontact.h"
+#include "icqconf.h"
 
 imcontact contactroot(0, icq);
 
@@ -10,17 +11,28 @@ imcontact::imcontact(unsigned long auin, protocolname apname) {
     pname = apname;
 }
 
-imcontact::imcontact(const string anick, protocolname apname) {
+imcontact::imcontact(const string &anick, protocolname apname) {
     nickname = anick;
     pname = apname;
     uin = 0;
 }
 
 bool imcontact::operator == (const imcontact &ainfo) const {
-    return
-	(ainfo.uin == uin) &&
-	(ainfo.pname == pname) &&
-	(ainfo.nickname == nickname);
+    bool r = (ainfo.uin == uin) && (ainfo.pname == pname);
+
+    if(pname == irc) {
+	int k;
+	string nick1, nick2;
+
+	for(k = 0; k < ainfo.nickname.size(); k++) nick1 += toupper(ainfo.nickname[k]);
+	for(k = 0; k < nickname.size(); k++) nick2 += toupper(nickname[k]);
+
+	r = r & (nick1 == nick2);
+    } else {
+	r = r & (ainfo.nickname == nickname);
+    }
+
+    return r;
 }
 
 bool imcontact::operator != (const imcontact &ainfo) const {
@@ -39,31 +51,28 @@ bool imcontact::empty() const {
     return (!uin && pname == icq) || (nickname.empty() && pname == yahoo);
 }
 
-const string imcontact::totext() const {
+string imcontact::totext() const {
     string r;
 
     if(*this == contactroot) {
-	r = "[root]";
-    } else
-    switch(pname) {
-	case icq:
-	    r = "[icq] " + i2str(uin);
-	    break;
-	case yahoo:
-	    r = "[yahoo] " + nickname;
-	    break;
-	case msn:
-	    r = "[msn] " + nickname;
-	    break;
-	case infocard:
-	    r = "[infocard] " + i2str(uin);
-	    break;
+	r = "[self]";
+    } else {
+	switch(pname) {
+	    case icq:
+	    case infocard:
+		r = "[" + conf.getprotocolname(pname) + "] " + i2str(uin);
+		break;
+
+	    default:
+		r = "[" + conf.getprotocolname(pname) + "] " + nickname;
+		break;
+	}
     }
 
     return r;
 }
 
-const string imcontact::getshortservicename() const {
+string imcontact::getshortservicename() const {
     string r;
 
     switch(pname) {
