@@ -4,9 +4,12 @@
 #include "icqcommon.h"
 
 enum protocolname {
+    proto_all = -1,
     icq = 0,
     yahoo,
     msn,
+    aim,
+    irc,
     infocard,
 
     protocolname_size
@@ -43,7 +46,7 @@ static char imstatus2char[imstatus_size] = {
 struct imcontact {
     imcontact();
     imcontact(unsigned long auin, protocolname apname);
-    imcontact(const string anick, protocolname apname);
+    imcontact(const string &anick, protocolname apname);
 
     string nickname;
     unsigned long uin;
@@ -56,8 +59,8 @@ struct imcontact {
     bool operator != (protocolname pname) const;
 
     bool empty() const;
-    const string totext() const;
-    const string getshortservicename() const;
+    string totext() const;
+    string getshortservicename() const;
 };
 
 extern imcontact contactroot;
