@@ -2,7 +2,8 @@
 #define __IMCONTROLLER_H__
 
 #include "icqconf.h"
-#include <libicq2000/constants.h>
+
+#include <libicq2000/userinfoconstants.h>
 
 struct imsearchparams {
     imsearchparams() {
@@ -13,14 +14,21 @@ struct imsearchparams {
 	agerange = ICQ2000::range_NoRange;
     };
 
+    imsearchparams(const string &name) {
+	load(name);
+    };
+
+    protocolname pname;
     bool onlineonly;
     unsigned int uin;
     unsigned short minage, maxage, country, language;
     ICQ2000::AgeRange agerange;
     imgender gender;
     string firstname, lastname, nick, city, state;
-    string company, department, position, email;
-    protocolname pname;
+    string company, department, position, email, room;
+
+    void save(const string &prname) const;
+    bool load(const string &prname);
 };
 
 class imcontroller {
@@ -33,6 +41,8 @@ class imcontroller {
 
 	bool icqregistration(icqconf::imaccount &account);
 	void icqupdatedetails();
+	void aimupdateprofile();
+	void ircchannels();
 
     public:
 	imcontroller();
@@ -40,6 +50,7 @@ class imcontroller {
 
 	void registration(icqconf::imaccount &account);
 	void updateinfo(icqconf::imaccount &account);
+	void channels(icqconf::imaccount &account);
 };
 
 extern imcontroller imcontrol;
