@@ -15,8 +15,18 @@ static const string seventtype[imevent::imeventtype_size] = {
 
 // -- basic imevent class -----------------------------------------------------
 
-imevent::imevent() {
-    time(&timestamp);
+imevent::imevent(): timestamp(time(0)) {
+}
+
+imevent::imevent(const imcontact &acont, imdirection adir, imeventtype atype)
+: contact(acont), direction(adir), timestamp(time(0)), type(atype) {
+}
+
+imevent::imevent(ifstream &f) {
+    read(f);
+}
+
+imevent::~imevent() {
 }
 
 imevent::imeventtype imevent::gettype() const {
@@ -35,6 +45,10 @@ time_t imevent::gettimestamp() const {
     return timestamp;
 }
 
+string imevent::gettext() const {
+    return "";
+}
+
 void imevent::settimestamp(time_t atimestamp) {
     timestamp = atimestamp;
 }
@@ -84,7 +98,8 @@ void imevent::read(ifstream &f) {
     }
 }
 
-const string imevent::readblock(ifstream &f) {
+string imevent::readblock(ifstream &f) {
+    int pos;
     string rdbuf, r;
 
     while(!f.eof()) {
@@ -93,6 +108,9 @@ const string imevent::readblock(ifstream &f) {
 	if(rdbuf == "\f") break;
 	if(f.eof() && rdbuf.empty()) break;
 
+	while((pos = rdbuf.find_first_of(evlinebreak)) != -1)
+	    rdbuf.erase(pos, 1);
+
 	if(!r.empty()) r += evlinebreak;
 	r += rdbuf;
     }
@@ -121,22 +139,14 @@ imevent *imevent::getevent() const {
     }
 }
 
-bool imevent::contains(const string text) const {
+bool imevent::contains(const string &text) const {
     return false;
 }
 
 // -- immessage class ---------------------------------------------------------
 
-immessage::immessage() {
-    type = message;
-}
-
-immessage::immessage(const imcontact acont, imdirection adirection,
-const string atext) {
-    type = message;
-    contact = acont;
-    direction = adirection;
-    text = atext;
+immessage::immessage(const imcontact &acont, imdirection adirection, const string &atext)
+: imevent(acont, adirection, message), text(atext) {
 }
 
 immessage::immessage(const imevent &ev) {
@@ -146,12 +156,13 @@ immessage::immessage(const imevent &ev) {
     timestamp = ev.timestamp;
 
     const immessage *m = dynamic_cast<const immessage *>(&ev);
+
     if(m) {
 	text = m->text;
     }
 }
 
-const string immessage::gettext() const {
+string immessage::gettext() const {
     return text;
 }
 
@@ -168,23 +179,14 @@ bool immessage::empty() const {
     return text.empty();
 }
 
-bool immessage::contains(const string atext) const {
+bool immessage::contains(const string &atext) const {
     return text.find(atext) != -1;
 }
 
 // -- imurl class -------------------------------------------------------------
 
-imurl::imurl() {
-    type = imevent::url;
-}
-
-imurl::imurl(const imcontact acont, imdirection adirection, const string aurl,
-const string adescription) {
-    type = imevent::url;
-    contact = acont;
-    direction = adirection;
-    url = aurl;
-    description = adescription;
+imurl::imurl(const imcontact &acont, imdirection adirection, const string &aurl, const string &adescription)
+: imevent(acont, adirection, imevent::url), url(aurl), description(adescription) {
 }
 
 imurl::imurl(const imevent &ev) {
@@ -201,14 +203,18 @@ imurl::imurl(const imevent &ev) {
     }
 }
 
-const string imurl::geturl() const {
+string imurl::geturl() const {
     return url;
 }
 
-const string imurl::getdescription() const {
+string imurl::getdescription() const {
     return description;
 }
 
+string imurl::gettext() const {
+    return url + "\n----\n" + description;
+}
+
 void imurl::write(ofstream &f) const {
     imevent::write(f);
     f << url << endl <<
@@ -224,28 +230,15 @@ bool imurl::empty() const {
     return url.empty() && description.empty();
 }
 
-bool imurl::contains(const string atext) const {
+bool imurl::contains(const string &atext) const {
     return (url.find(atext) != -1) || (description.find(atext) != -1);
 }
 
 // -- imauthorization class ---------------------------------------------------
 
-imauthorization::imauthorization() {
-    type = authorization;
-}
-
-imauthorization::imauthorization(const imcontact acont, imdirection adirection,
-bool agranted, const string atext) {
-    type = authorization;
-    contact = acont;
-    direction = adirection;
-    granted = agranted;
-
-    if(atext.empty()) {
-	text = _("Empty authorization request message");
-    } else {
-	text = atext;
-    }
+imauthorization::imauthorization(const imcontact &acont, imdirection adirection, bool agranted, const string &atext)
+: imevent(acont, adirection, authorization), granted(agranted),
+  text(atext.empty() ? _("Empty authorization request message") : atext) {
 }
 
 imauthorization::imauthorization(const imevent &ev) {
@@ -262,7 +255,7 @@ imauthorization::imauthorization(const imevent &ev) {
     }
 }
 
-const string imauthorization::gettext() const {
+string imauthorization::gettext() const {
     return text;
 }
 
@@ -290,22 +283,14 @@ bool imauthorization::getgranted() const {
     return granted;
 }
 
-bool imauthorization::contains(const string atext) const {
+bool imauthorization::contains(const string &atext) const {
     return text.find(atext) != -1;
 }
 
 // -- imsms class -------------------------------------------------------------
 
-imsms::imsms() {
-    type = sms;
-}
-
-imsms::imsms(const imcontact acont, imdirection adirection,
-const string atext) {
-    type = sms;
-    contact = acont;
-    direction = adirection;
-    text = atext;
+imsms::imsms(const imcontact &acont, imdirection adirection, const string &atext)
+: imevent(acont, adirection, sms), text(atext) {
 }
 
 imsms::imsms(const imevent &ev) {
@@ -315,12 +300,13 @@ imsms::imsms(const imevent &ev) {
     timestamp = ev.timestamp;
 
     const imsms *m = dynamic_cast<const imsms *>(&ev);
+
     if(m) {
 	text = m->text;
     }
 }
 
-const string imsms::gettext() const {
+string imsms::gettext() const {
     return text;
 }
 
@@ -337,22 +323,14 @@ bool imsms::empty() const {
     return text.empty();
 }
 
-bool imsms::contains(const string atext) const {
+bool imsms::contains(const string &atext) const {
     return text.find(atext) != -1;
 }
 
 // -- imemail class -----------------------------------------------------------
 
-imemail::imemail() {
-    type = email;
-}
-
-imemail::imemail(const imcontact acont, imdirection adirection,
-const string atext) {
-    type = email;
-    contact = acont;
-    direction = adirection;
-    text = atext;
+imemail::imemail(const imcontact &acont, imdirection adirection, const string &atext)
+: imevent(acont, adirection, email), text(atext) {
 }
 
 imemail::imemail(const imevent &ev) {
@@ -362,12 +340,13 @@ imemail::imemail(const imevent &ev) {
     timestamp = ev.timestamp;
 
     const imemail *m = dynamic_cast<const imemail *>(&ev);
+
     if(m) {
 	text = m->text;
     }
 }
 
-const string imemail::gettext() const {
+string imemail::gettext() const {
     return text;
 }
 
@@ -375,7 +354,7 @@ bool imemail::empty() const {
     return text.empty();
 }
 
-bool imemail::contains(const string atext) const {
+bool imemail::contains(const string &atext) const {
     return text.find(atext) != -1;
 }
 
@@ -387,3 +366,9 @@ void imemail::write(ofstream &f) const {
 void imemail::read(ifstream &f) {
     text = readblock(f);
 }
+
+// -- imrawevent class --------------------------------------------------------
+
+imrawevent::imrawevent(imeventtype atype, const imcontact &acont, imdirection adirection)
+: imevent(acont, adirection, atype) {
+}
